;**FILE ctmrdisp.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2000 - 2002 TIMEX Corporation.
; 
; This document is the property of, and contains information proprietary to
; TIMEX Corporation.  It is to be returned upon request.  Reproduction by any
; means, disclosure, or use of its contents for other than the expressed
; purpose for which it is loaned, without the written consent of TIMEX
; Corporation, is strictly prohibited.
;
; File Name    : ctmrdisp.asm
;
; Module Names : ctmrDisplayWorkHMSAtBDMOrL1    - Display the working HMS data either 
;                                                   large or small font.
;
;                ctmrDisplayWorkHMSAtBDM        - Display the working HMS data using 
;                                                   large font.
;
;                ctmrDisplayWorkHMSAtLine1      - Display the working HMS data at line 1 
;                                                   using fixed-width character.
;                
;                ctmrDisplayUserSetSecondAtBDM  - Display user set seconds at line 2. This 
;                                                   will go along with large fonts.
;                
;                ctmrDisplayUserSetHourMinuteAtBDM - Displays user set hour and minute 
;                                                      data at BDM.
;
;                ctmrDisplayUserSetHMSAtSegment - Display user set HMS data at 9 segment.
;
;                ctmrDisplayActionAtEndFlags    - Displays the action at end flags.
;
;                ctmrDisplayActionAtEndMsg      - Displays the action at end message at line 1.
;
;                ctmrDisplayOneHalfAtSDM        - Displays "1/2" at SDM if halfway reminder is 
;                                                   enabled.
;
;                ctmrDisplayTimerNameAtLine2    - Displays the timer name at line 2.
;
;                ctmrClearSecondsLDM            - Clear the seconds data in LDM.
;
;                ctmrStaticDisplayOfField       - Displays that will not be repeated when changing 
;                                                   the data on a specific field.  This is to reduce 
;                                                   the cycle time when the value of a data is being 
;                                                   changed.
;                
; Assumptions  : The data to be checked/updated/manipulated is in the variables
;                  declared in ctmrvars.h
;               
; Purpose      : Countdown and Interval Timer display routines.
;
; File Change History:
;
;               12/12/2001  - Created for M851 development.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

$CASE ON
$LIST OFF

        INCLUDE '..\..\CORE\H\MODEL.H'
        INCLUDE '..\..\HW\H\HW88349.H'
        INCLUDE '..\..\HW\H\HWI2CEE.H'
        INCLUDE '..\..\HW\H\HWBOND.H'
        INCLUDE '..\..\HW\H\HWKBD.H'
        INCLUDE '..\..\HW\H\HWMACRO.H'
        INCLUDE '..\..\HW\H\HWMAIN.H'
        INCLUDE '..\..\HW\H\HWRESET.H'
        INCLUDE '..\..\HW\H\HWVARS.H'
        INCLUDE '..\..\HW\H\HWSERMAC.H'
        INCLUDE '..\..\CORE\H\CORE.H'
        INCLUDE '..\..\CORE\H\COREMAC.H'
        INCLUDE '..\..\CORE\H\CORESVMC.H'
        INCLUDE '..\..\COMM\H\COMM.H'
        INCLUDE '..\..\LCD\H\LCD.H'
        INCLUDE '..\..\LCD\H\LCDMACRO.H'
        INCLUDE '..\..\LCD\H\LCDRAM.H'
        INCLUDE '..\..\DATABASE\H\DB.H'
        INCLUDE '..\..\DATABASE\H\DBMACRO.H'
        INCLUDE '..\..\AUDIO\H\AUD.H'
        INCLUDE '..\..\AUDIO\H\AUDMAC.H'
        INCLUDE '..\..\UTIL\H\UTLMATH.H'
        INCLUDE '..\..\UTIL\H\UTLMTHMC.H'
        INCLUDE '..\..\UTIL\H\UTLEDT.H'
        INCLUDE '..\..\UTIL\H\UTLEDTMC.H'
        INCLUDE '..\..\UTIL\H\UTLDSP.H'
        INCLUDE '..\..\UTIL\H\UTLDSPMC.H'
        INCLUDE '..\..\UTIL\H\UTLACCMC.H'
        INCLUDE '..\..\UTIL\H\UTLMSC.H'
        INCLUDE '..\..\UTIL\H\UTLMSCMC.H'
        INCLUDE '..\..\UTIL\H\UTLVARS.H'
        INCLUDE '..\..\KRES\H\KRES.H'
        INCLUDE '..\..\KRES\H\KRESMAC.H'
        INCLUDE '..\..\KRES\H\KSTP.H'
        INCLUDE '..\..\KRES\H\KSTPMAC.H'
        INCLUDE '..\..\KRES\H\KSYN.H'
        INCLUDE '..\..\KRES\H\KSYNMAC.H'
        INCLUDE '..\..\KRES\H\KTMR.H'
        INCLUDE '..\..\KRES\H\KTMRMAC.H'
        INCLUDE '..\..\KRES\H\KTOD.H'
        INCLUDE '..\..\KRES\H\KTODMAC.H'
        INCLUDE '..\..\KRES\H\KTZC.H'
        INCLUDE '..\..\KRES\H\KTZCMAC.H'
        INCLUDE '..\..\KRES\H\KBCK.H'
        INCLUDE '..\..\KRES\H\KBCKMAC.H'
        INCLUDE '..\..\USAGE\H\USG.H'
        INCLUDE '..\..\USAGE\H\USGVARS.H'
        INCLUDE '..\..\USAGE\H\USGMAC.H'
        INCLUDE '..\..\CORE\H\SECTION.H'

        INCLUDE '..\..\TOD\H\TOD.H'
        INCLUDE '..\..\TOD\H\TODVARS.H'

        INCLUDE '..\..\CTMR\h\CTMR.H'
        INCLUDE '..\..\CTMR\h\CTMRVARS.H'

$LIST ON

        SECT ".text_commontimer"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrDisplayWorkHMSAtBDMOrL1
;
; Description : Display the working HMS data either large or small font.
;
; Assumptions : CTMRActiveResource   - Resource index of the working data to be displayed.
;               CTMR10SecondsCounter - Counter to indicate whether to display large or small
;                                        font.
;
; Input(s)    : BReg.bKTMRSecondUpd - Update seconds data.
;                   .bKTMRMinuteUpd - Update minute data.
;                   .bKTMRHourUpd   - Update hour data.
;               
; Output(s)   : None
;
;               ( Destroyed: BAReg, HLReg, IXReg, IYReg )
;
; Module Change History:
;
;               03/06/2002  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrDisplayWorkHMSAtBDMOrL1()
{
    //-----------------------------------------------------------------
    // Copy the resource running data to a working data buffer.
    // Destroys HLReg, IYReg.
    // Input: AReg  - Resource index
    //        IYReg - Start address of buffer to write into.
    //-----------------------------------------------------------------
    AReg = CTMRActiveResource;
    IYReg = &CTMRWorkSecond;
    KTMR_COPY_RESOURCE_TO_BUFFER;

    // Check what type of display we want.
    if( CTMR10SecondsCounter == 0 )
    {
        //-------------------------------------------------------------
        // Since the 10 second counter has expired, a large font will
        //   be displayed.
        // The data to be displayed should be in CTMRWorkSecond, 
        //   CTMRWorkMinute and CTMRWorkHour.
        // Input: BReg - Update bits.
        //-------------------------------------------------------------
        ctmrDisplayWorkHMSAtBDM();
    }

    else
    {
        //-------------------------------------------------------------
        // Since the 10 seconds has not expired yet, display using
        //   small font at line 1.
        // The data to be displayed should be in CTMRWorkSecond, 
        //   CTMRWorkMinute and CTMRWorkHour.
        // Input: BReg - Update bits.
        //-------------------------------------------------------------
        ctmrDisplayWorkHMSAtLine1();
    }
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrDisplayWorkHMSAtBDMOrL1'"

        GLOBAL  ctmrDisplayWorkHMSAtBDMOrL1


ctmrDisplayWorkHMSAtBDMOrL1:                        ; **SUBROUTINE ctmrDisplayWorkHMSAtBDMOrL1

                ;--------------------------------------------------------------
                ; Copy the resource running data to a working data buffer.
                ; Destroys HLReg, IYReg.
                ; Input: AReg  - Resource index
                ;        IYReg - Start address of buffer to write into.
                ;--------------------------------------------------------------
                ld      A, [CTMRActiveResource]
                ld      IY, #CTMRWorkSecond
                KTMR_COPY_RESOURCE_TO_BUFFER
                
                ; Check what type of display we want.
                ld      A, [CTMR10SecondsCounter]
                cp      A, #0

                ;--------------------------------------------------------------
                ; Since the 10 second counter has expired, a large font will
                ;   be displayed.
                ; The data to be displayed should be in CTMRWorkSecond, 
                ;   CTMRWorkMinute and CTMRWorkHour.
                ; Input: BReg - Update bits.
                ;--------------------------------------------------------------
                jr      Z, ctmrDisplayWorkHMSAtBDM  ; **EXTERNAL JUMP

                ;--------------------------------------------------------------
                ; Since the 10 seconds has not expired yet, display using
                ;   small font at line 1.
                ; The data to be displayed should be in CTMRWorkSecond, 
                ;   CTMRWorkMinute and CTMRWorkHour.
                ; Input: BReg - Update bits.
                ;--------------------------------------------------------------
                jr      ctmrDisplayWorkHMSAtLine1   ; **EXTERNAL JUMP


                                                    ; **END SUBROUTINE ctmrDisplayWorkHMSAtBDMOrL1



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrDisplayWorkHMSAtBDM
;
; Description : Display the working HMS data using large font.
;
; Assumptions : Structure format: Second, Minute, Hour.
;               
;               The data to be display is in the CTMRWorkSecond, CTMRWorkMinute
;                 and CTMRWorkHour
;       
; Input(s)    : BReg.bKTMRSecondUpd - Update seconds data.
;                   .bKTMRMinuteUpd - Update minute data.
;                   .bKTMRHourUpd   - Update hour data.
;               
; Output(s)   : None
;
;               ( Destroyed: BAReg, HLReg, IXReg, IYReg )
;
; Module Change History:
;
;               11/14/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrDisplayWorkHMSAtBDM()
{

    //*************************************************************************
    // An overview of the LCD macro's used in this routine.
    //
    //  LCD macro's that displays 2 digit data either large or small fonts:
    //      - Destroys BAReg, HLReg, IXReg
    //      - Input: IXReg - Starting DM column
    //               AReg  - Packed 10's and 1's digit BCD data.
    //
    //  LCD macro to display punctuation:
    //      - Destroys LReg, IXReg
    //      - Input: IXReg - Starting DM column
    //               LReg  - Character to be displayed.
    //*************************************************************************

    //=========================================================================
    //
    //                      DISPLAY SECONDS DATA
    //
    //=========================================================================

    //-------------------------------------------------------------------------
    // No need to check if seconds data has updated coz this will only be 
    //   called when seconds have been updated.
    //-------------------------------------------------------------------------

    push    (BAReg);

    // Set the display position and the data to be displayed in IXReg and AReg
    //   respectively.
    IXReg = CTMRLDMPOSITIONSECOND;
    AReg  = CTMRWorkSecond;

    // Load the working minute and hour data and check if it is zero.
    HLReg = CTMRWorkMinute;
    if( HLReg == 0 )
    {
        // Since hour and minute data is zero, display the seconds
        //   data suppressing '0' in MSD.
        LCD_DISP_SMALL_PROP_WIDTH_2DIG_DM_DATA_SUP_ZERO_MSD;       
    }
    else
    {
        // Since hour or minute data is non-zero, display the seconds
        //   data without suppressing it.
        LCD_DISP_SMALL_PROP_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP;
    }

    pop     (BAReg);

    //=========================================================================
    //
    //                      DISPLAY MINUTE DATA
    //
    //=========================================================================

    push    (BAReg);

    // Check if minute data has been updated.
    if( BReg.bKTMRMinuteUpd == TRUE )
    {

        // Set the display position and the data to be displayed at IXReg and 
        //   AReg respectively.
        IXReg = CTMRLDMPOSITIONMINUTE;
        BAReg = CTMRWorkMinute;

        // Check if hour data is zero.
        if( BReg == 0 )
        {
            push    (BAReg);
            push    (IXReg);

            // Clear colon.
            IXReg = CTMRLDMPOSITIONCOLON;
            LReg  = DM8_SPACE;
            LCD_DISP_BIG_DM_CHAR;

            pop     (IXReg);
            pop     (BAReg);

            // Since hour data is zero, display the minute data with suppression
            //   on both zeroes.
            LCD_DISP_BIG_2DIGIT_DM_DATA_SUP_ZERO;

        }

        else
        {
            // Since hour data is non-zero, display the minute data without
            //   suppressing it.
            LCD_DISP_BIG_2DIGIT_DM_DATA_NO_ZERO_SUP;

            // Display colon.
            IXReg = CTMRLDMPOSITIONCOLON;
            LReg  = DM8_COLON;
            LCD_DISP_BIG_DM_CHAR;
        }
    }

    pop     (BAReg);

    //=========================================================================
    //
    //                      DISPLAY HOUR DATA
    //
    //=========================================================================

    // Check if hour data has been updated.
    if( BReg.bKTMRHourUpd == TRUE )
    {
        // Set the display position and the data to be displayed at IXReg 
        //   and AReg respectively.
        IXReg = CTMRLDMPOSITIONHOUR;
        AReg  = CTMRWorkHour;
                                                                     
        LCD_DISP_BIG_2DIGIT_DM_DATA_SUP_ZERO;
    }
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrDisplayWorkHMSAtBDM'"

        GLOBAL  ctmrDisplayWorkHMSAtBDM

ctmrDisplayWorkHMSAtBDM:                            ; **SUBROUTINE ctmrDisplayWorkHMSAtBDM

                ;**************************************************************
                ; An overview of the LCD macro's used in this routine.
                ;
                ;  LCD macro's that displays 2 digit data either large or 
                ;    small fonts:
                ;      - Destroys BAReg, HLReg, IXReg
                ;      - Input: IXReg - Starting DM column
                ;               AReg  - Packed 10's and 1's digit BCD data.
                ;
                ;  LCD macro to display punctuation:
                ;      - Destroys LReg, IXReg
                ;      - Input: IXReg - Starting DM column
                ;               LReg  - Character to be displayed.
                ;**************************************************************

                ;==============================================================
                ;
                ;                      DISPLAY SECONDS DATA
                ;
                ;==============================================================

                ;--------------------------------------------------------------
                ; No need to check if seconds data has updated coz this will 
                ;   only be called when seconds have been updated.
                ;--------------------------------------------------------------

                push    BA
                
                ; Set the display position and the data to be displayed in IXReg and AReg
                ;   respectively.
                ld      IX, #CTMRLDMPOSITIONSECOND
                ld      A, [CTMRWorkSecond]
                
                ; Load the working minute and hour data and check if it is zero.
                ld      HL, [CTMRWorkMinute]
                cp      HL, #0
                jr      NZ, ctmrDispWorkSecBDMNoSup

                ; Since hour and minute data is zero, display the seconds
                ;   data suppressing '0' in MSD.
                LCD_DISP_SMALL_PROP_WIDTH_2DIG_DM_DATA_SUP_ZERO_MSD

                jr      ctmrDispWorkBDMCheckMin

ctmrDispWorkSecBDMNoSup:
                
                ; Since hour or minute data is non-zero, display the seconds
                ;   data without suppressing it.
                LCD_DISP_SMALL_PROP_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP

ctmrDispWorkBDMCheckMin:

                pop     BA

                ;==============================================================
                ;
                ;                      DISPLAY MINUTE DATA
                ;
                ;==============================================================

                push    BA
                
                ; Check if minute data has been updated.
                bit     B, #bKTMRMinuteUpd
                jr      Z, ctmrDispWorkBDMCheckHour

                ; Set the display position and the data to be displayed at IXReg and 
                ;   AReg respectively.
                ld      IX, #CTMRLDMPOSITIONMINUTE
                ld      BA, [CTMRWorkMinute]
                
                ; Check if hour data is zero.
                cp      B, #0
                jr      NZ, ctmrDispWorkMinBDMNoSup

                push    BA
                push    IX
                                
                ; Clear colon.
                ld      IX, #CTMRLDMPOSITIONCOLON
                ld      L, #DM8_SPACE
                LCD_DISP_BIG_DM_CHAR

                pop     IX
                pop     BA

                ; Since hour data is zero, display the minute data with suppression
                ;   on both zeroes.
                LCD_DISP_BIG_2DIGIT_DM_DATA_SUP_ZERO
                            
                jr      ctmrDispWorkBDMCheckHour

ctmrDispWorkMinBDMNoSup:
                
                ; Since hour data is non-zero, display the minute data without
                ;   suppressing it.
                LCD_DISP_BIG_2DIGIT_DM_DATA_NO_ZERO_SUP
                
                ; Display colon.
                ld      IX, #CTMRLDMPOSITIONCOLON
                ld      L, #DM8_COLON
                LCD_DISP_BIG_DM_CHAR


ctmrDispWorkBDMCheckHour:

                pop     BA

                ;==============================================================
                ;
                ;                      DISPLAY HOUR DATA
                ;
                ;==============================================================
                
                ; Check if hour data has been updated.
                bit     B, #bKTMRHourUpd
                jr      Z, ctmrDispWorkHMSBDMExit

                ; Set the display position and the data to be displayed at IXReg 
                ;   and AReg respectively.
                ld      IX, #CTMRLDMPOSITIONHOUR
                ld      A, [CTMRWorkHour]
                                                                             
                LCD_DISP_BIG_2DIGIT_DM_DATA_SUP_ZERO
                

ctmrDispWorkHMSBDMExit:

                ret


                                                    ; **END SUBROUTINE ctmrDisplayWorkHMSAtBDM



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrDisplayWorkHMSAtLine1
;
; Description : Display the working HMS data at line 1 using fixed-width
;                 character.
;
; Assumptions : Structure format: Second, Minute, Hour.
;               
;               The data to be display is in the CTMRWorkSecond, CTMRWorkMinute
;                 and CTMRWorkHour
;
; Input(s)    : BReg.bKTMRSecondUpd - Update seconds data.
;                   .bKTMRMinuteUpd - Update minute data.
;                   .bKTMRHourUpd   - Update hour data.
;               
; Output(s)   : None
;
;               ( Destroyed: BAReg, HLReg, IXReg, IYReg )
;
; Module Change History:
;
;               11/14/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrDisplayWorkHMSAtLine1()
{

    //*************************************************************************
    // An overview of the LCD macro's used in this routine.
    //
    //  LCD macro's that displays 2 digit fixe-width data:
    //      - Destroys BAReg, HLReg, IXReg
    //      - Input: IXReg - Starting DM column
    //               AReg  - Packed 10's and 1's digit BCD data.
    //
    //  LCD macro to display punctuation:
    //      - Destroys BAReg, HLReg, IXReg
    //      - Input: IXReg - Starting DM column
    //               LReg  - Character to be displayed.
    //*************************************************************************

    //=========================================================================
    //
    //                      DISPLAY SECONDS DATA
    //
    //=========================================================================

    //-------------------------------------------------------------------------
    // No need to check if seconds data has updated coz this will only be 
    //   called when seconds have been updated.
    //-------------------------------------------------------------------------

    push    (BAReg);

    // Set the display position and the data to be displayed in IXReg and AReg
    //   respectively.
    IXReg = CTMRDML1POSITIONSECOND;
    AReg  = CTMRWorkSecond;

    // Load the working minute and hour data and check if it is zero.
    HLReg = CTMRWorkMinute;
    if( HLReg == 0 )
    {
        push    (BAReg);
        push    (IXReg);
        
        // Clear dot on line 1.
        IXReg = CTMRDML1POSITIONDOT;
        LReg  = DM5_SPACE;
        LCD_DISP_SMALL_PROP_WIDTH_DM_CHAR;

        pop     (IXReg);
        pop     (BAReg);

        // Since hour and minute data is zero, display the seconds
        //   data suppressing '0' in MSD.
        LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_SUP_ZERO_MSD;
    }

    else
    {
        // Since hour or minute data is non-zero, display the seconds
        //   data without suppressing it.
        LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP;

        // Display dot on line 1.
        IXReg = CTMRDML1POSITIONDOT;
        LReg  = DM5_PERIOD;
        LCD_DISP_SMALL_PROP_WIDTH_DM_CHAR
    }

    pop     (BAReg);

    //=========================================================================
    //
    //                      DISPLAY MINUTE DATA
    //
    //=========================================================================

    push    (BAReg);

    // Check if minute data has been updated.
    if( BReg.bKTMRMinuteUpd == TRUE )
    {

        // Set the display position and the data to be displayed at IXReg and 
        //   AReg respectively.
        IXReg = CTMRDML1POSITIONMINUTE;
        BAReg = CTMRWorkMinute;

        // Check if hour data is zero.
        if( BReg == 0 )
        {
            push    (BAReg);
            push    (IXReg);

            // Clear colon at line 1.
            IXReg = CTMRDML1POSITIONCOLON;
            LReg  = DM5_SPACE;
            LCD_DISP_SMALL_PROP_WIDTH_DM_CHAR;

            pop     (IXReg);
            pop     (BAReg);
            
            // Since hour data is zero, display the minute data with suppression
            //   on both zeroes.
            LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_SUP_ZERO;
        }

        else
        {
            // Since hour data is non-zero, display the minute data without
            //   suppressing it.
            LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP;

            // Display colon at line 1.
            IXReg = CTMRDML1POSITIONCOLON;
            LReg  = DM5_COLON;
            LCD_DISP_SMALL_PROP_WIDTH_DM_CHAR
        }
    }

    pop     (BAReg);


    //=========================================================================
    //
    //                      DISPLAY HOUR DATA
    //
    //=========================================================================

    // Check if hour data has been updated.
    if( BReg.bKTMRHourUpd == TRUE )
    {
        // Set the display position and the data to be displayed at IXReg 
        //   and AReg respectively.
        IXReg = CTMRDML1POSITIONHOUR;
        AReg  = CTMRWorkHour;
                                                                     
        LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_SUP_ZERO;
    }

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrDisplayWorkHMSAtLine1'"

        GLOBAL  ctmrDisplayWorkHMSAtLine1

ctmrDisplayWorkHMSAtLine1:                          ; **SUBROUTINE ctmrDisplayWorkHMSAtLine1

                ;**************************************************************
                ; An overview of the LCD macro's used in this routine.
                ;
                ;  LCD macro's that displays 2 digit fixe-width data:
                ;      - Destroys BAReg, HLReg, IXReg
                ;      - Input: IXReg - Starting DM column
                ;               AReg  - Packed 10's and 1's digit BCD data.
                ;
                ;  LCD macro to display punctuation:
                ;      - Destroys BAReg, HLReg, IXReg
                ;      - Input: IXReg - Starting DM column
                ;               LReg  - Character to be displayed.
                ;**************************************************************

                ;==============================================================
                ;
                ;                      DISPLAY SECONDS DATA
                ;
                ;==============================================================

                ;--------------------------------------------------------------
                ; No need to check if seconds data has updated coz this will 
                ;   only be called when seconds have been updated.
                ;--------------------------------------------------------------

                push    BA

                ; Set the display position and the data to be displayed in IXReg 
                ;   and AReg respectively.
                ld      IX, #CTMRDML1POSITIONSECOND
                ld      A, [CTMRWorkSecond]
                
                ; Load the working minute and hour data and check if it is zero.
                ld      HL, [CTMRWorkMinute]
                cp      HL, #0
                jr      NZ, ctmrDispWorkSecL1NoSup

                push    BA
                push    IX

                ; Clear dot on line 1.
                ld      IX, #CTMRDML1POSITIONDOT
                ld      L, #DM5_SPACE
                LCD_DISP_SMALL_PROP_WIDTH_DM_CHAR

                pop     IX
                pop     BA

                ; Since hour and minute data is zero, display the seconds
                ;   data suppressing '0' in MSD.
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_SUP_ZERO_MSD
                        
                jr      ctmrDispWorkL1CheckMin

ctmrDispWorkSecL1NoSup:
                
                ; Since hour or minute data is non-zero, display the seconds
                ;   data without suppressing it.
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP
                
                ; Display dot on line 1.
                ld      IX, #CTMRDML1POSITIONDOT
                ld      L, #DM5_PERIOD
                LCD_DISP_SMALL_PROP_WIDTH_DM_CHAR
            
ctmrDispWorkL1CheckMin:
                        
                pop     BA

                ;==============================================================
                ;
                ;                      DISPLAY MINUTE DATA
                ;
                ;==============================================================
                        
                push    BA

                ; Check if minute data has been updated.
                bit     B, #bKTMRMinuteUpd
                jr      Z, ctmrDispWorkL1CheckHour
                 
                ; Set the display position and the data to be displayed at IXReg and 
                ;   AReg respectively.
                ld      IX, #CTMRDML1POSITIONMINUTE
                ld      BA, [CTMRWorkMinute]
                
                ; Check if hour data is zero.
                cp      B, #0
                jr      NZ, ctmrDispWorkMinL1NoSup
                 
                push    BA
                push    IX

                ; Clear colon at line 1.
                ld      IX, #CTMRDML1POSITIONCOLON
                ld      L, #DM5_SPACE
                LCD_DISP_SMALL_PROP_WIDTH_DM_CHAR

                pop     IX
                pop     BA

                ; Since hour data is zero, display the minute data with suppression
                ;   on both zeroes.
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_SUP_ZERO
                    
                jr      ctmrDispWorkL1CheckHour
                      
ctmrDispWorkMinL1NoSup:
                
                ; Since hour data is non-zero, display the minute data without
                ;   suppressing it.
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP
                
                ; Display colon at line 1.
                ld      IX, #CTMRDML1POSITIONCOLON
                ld      L, #DM5_COLON
                LCD_DISP_SMALL_PROP_WIDTH_DM_CHAR

ctmrDispWorkL1CheckHour:

                pop     BA

                ;==============================================================
                ;
                ;                      DISPLAY HOUR DATA
                ;
                ;==============================================================

                ; Check if hour data has been updated.
                bit     B, #bKTMRHourUpd
                jr      Z, ctmrDispWorkHMSL1Exit

                ; Set the display position and the data to be displayed at IXReg 
                ;   and AReg respectively.
                ld      IX, #CTMRDML1POSITIONHOUR
                ld      A, [CTMRWorkHour]
                                                                             
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_SUP_ZERO
                
ctmrDispWorkHMSL1Exit:


                ret


                                                    ; **END SUBROUTINE ctmrDisplayWorkHMSAtLine1



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrDisplayUserSetSecondAtBDM
;
; Description : Display user set seconds at line 2. This will go along with 
;                 large fonts.
;                 
;               This will be used in timer set state.  
;
; Assumptions : None
;
; Input(s)    : None
;               
; Output(s)   : None
;
;               ( Destroyed: BAReg, HLReg, IXReg )
;
; Module Change History:
;
;               11/14/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrDisplayUserSetSecondAtBDM'"

        GLOBAL  ctmrDisplayUserSetSecondAtBDM

ctmrDisplayUserSetSecondAtBDM:                      ; **SUBROUTINE ctmrDisplayUserSetSecondAtBDM

                ;--------------------------------------------------------------
                ; Display 2-digit data with no suppression on both zeros.
                ; Destroys BAReg, HLReg, IXReg.
                ; Input:  AReg - Packed 10's and 1's digit BCD data.
                ;         IXReg- Starting DM column.
                ;--------------------------------------------------------------
                ld      IX, #CTMRLDMPOSITIONSECOND
                ld      A, [CTMRUserSetSecond]
                LCD_DISP_SMALL_PROP_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP

                ret



                                                    ; **END SUBROUTINE ctmrDisplayUserSetSecondAtBDM



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrDisplayUserSetHourMinuteAtBDM
;
; Description : Displays user set hour and minute data at BDM.
;
;               This will be used in timer set state.  
;
; Assumptions : None
;
; Input(s)    : None
;               
; Output(s)   : None
;
;               ( Destroyed: BAReg, HLReg, IXReg )
;
; Module Change History:
;
;               11/14/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrDisplayUserSetHourMinuteAtBDM()
{
    //-------------------------------------------------------------------------
    // DISPLAY MINUTE DATA
    // Display 2-digit DM data with no zero suppression on all digits.
    // Destroys BAReg, HLReg, IXReg.
    // Input: AReg  - Packed 10's and 1's digit data.
    //        IXReg - Starting DM column.
    //-------------------------------------------------------------------------
    IXReg = CTMRLDMPOSITIONMINUTE;
    AReg  = CTMRUserSetMinute;
    LCD_DISP_BIG_2DIGIT_DM_DATA_NO_ZERO_SUP;

    //-------------------------------------------------------------------------
    // DISPLAY COLON
    // Displays a large font, dot matrix character.
    // Destroys IXReg, LReg.
    // Input: LReg  - Character to be displayed.
    //        IXReg - Starting DM column.
    //-------------------------------------------------------------------------
    IXReg = CTMRLDMPOSITIONCOLON;
    LReg  = DM8_COLON;
    LCD_DISP_BIG_DM_CHAR;

    //-------------------------------------------------------------------------
    // DISPLAY HOUR DATA
    // Displays 2-digit DM data with zero suppression on the MSD.
    // Destroys BAReg, HLReg, IXReg.
    // Input: AReg  - Packed 10's and 1's digit data.
    //        IXReg - Starting DM column.
    //-------------------------------------------------------------------------
    IXReg = CTMRLDMPOSITIONHOUR;
    AReg  = CTMRUserSetHour;
    LCD_DISP_BIG_2DIGIT_DM_DATA_SUP_ZERO_MSD;

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrDisplayUserSetHourMinuteAtBDM'"

        GLOBAL  ctmrDisplayUserSetHourMinuteAtBDM

ctmrDisplayUserSetHourMinuteAtBDM:                  ; **SUBROUTINE ctmrDisplayUserSetHourMinuteAtBDM

                ;--------------------------------------------------------------
                ; DISPLAY MINUTE DATA
                ; Display 2-digit DM data with no zero suppression on all digits.
                ; Destroys BAReg, HLReg, IXReg.
                ; Input: AReg  - Packed 10's and 1's digit data.
                ;        IXReg - Starting DM column.
                ;--------------------------------------------------------------
                ld      IX, #CTMRLDMPOSITIONMINUTE
                ld      A, [CTMRUserSetMinute]
                LCD_DISP_BIG_2DIGIT_DM_DATA_NO_ZERO_SUP

                ;--------------------------------------------------------------
                ; DISPLAY COLON
                ; Displays a large font, dot matrix character.
                ; Destroys IXReg, LReg.
                ; Input: LReg  - Character to be displayed.
                ;        IXReg - Starting DM column.
                ;--------------------------------------------------------------
                ld      IX, #CTMRLDMPOSITIONCOLON
                ld      L, #DM8_COLON
                LCD_DISP_BIG_DM_CHAR

                ;--------------------------------------------------------------
                ; DISPLAY HOUR DATA
                ; Displays 2-digit DM data with zero suppression on the MSD.
                ; Destroys BAReg, HLReg, IXReg.
                ; Input: AReg  - Packed 10's and 1's digit data.
                ;        IXReg - Starting DM column.
                ;--------------------------------------------------------------
                ld      IX, #CTMRLDMPOSITIONHOUR
                ld      A, [CTMRUserSetHour]
                LCD_DISP_BIG_2DIGIT_DM_DATA_SUP_ZERO_MSD

                ret


                                                    ; **END SUBROUTINE ctmrDisplayUserSetHourMinuteAtBDM



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrDisplayUserSetHMSAtSegment
;
; Description : Display user set HMS data at 9 segment.
;
; Assumptions : Structure format: Second, Minute, Hour.
;
;               LCD routines should not destroy IYReg.
;
; Input(s)    : None
;
; Output(s)   : None
;
;               ( Destroyed: BAReg, HLReg, IXReg, IYReg )
;
; Module Change History:
;
;               11/14/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrDisplayUserSetHMSAtSegment()
{

    //*************************************************************************
    // An overview of the LCD macro's used in this routine.
    //
    //  LCD macro that clears the entire segment display.
    //      - Destroys BAReg, IXReg, LReg
    //
    //  LCD macro's that displays 2 digit data:
    //      - Destroys BAReg, LReg, IXReg
    //      - Input: IXReg - Starting digit column.
    //               AReg  - Packed 10's and 1's digit BCD data.
    //
    //  LCD macro to display punctuation:
    //      - Destroys HLReg
    //*************************************************************************

    // Clear entire 9 segment display.
    LCD_CLR_SEG_LINE;

    //=========================================================================
    //
    //                      DISPLAY SECONDS DATA
    //
    //=========================================================================

    // Set the display position and the data to be displayed at IXReg and AReg.
    IXReg = CTMRSEGDIGIT5;
    AReg  = CTMRUserSetSecond;

    // Load the user set minute and hour data and check if it is zero.
    HLReg = CTMRUserSetMinute;
    if( HLReg == 0 )
    {       
        // Since hour and minute data is zero, display the seconds
        //   data suppressing '0' in MSD.
        LCD_DISP_2DIG_SEG_DATA_SUP_ZERO_MSD;

        // Turn off dot flag. Minute and second separator.
        LCD_UPD_PER2_FLAG   OFF;

    }

    else
    {
        // Since hour or minute data is non-zero, display the seconds
        //   data without suppressing it.
        LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP;

        // Turn on dot flag. Minute and second separator.
        LCD_UPD_PER2_FLAG   ON;
    }

    //=========================================================================
    //
    //                      DISPLAY MINUTE DATA
    //
    //=========================================================================

    // Set the display position and the data to be displayed at IXReg and AReg.
    IXReg = CTMRSEGDIGIT3;
    BAReg = CTMRUserSetMinute;

    // Check if hour data is zero.
    if( BReg == 0 )
    {
        // Since hour data is zero, display the minute data with suppression
        //   on both zeroes.
        LCD_DISP_2DIG_SEG_DATA_WITH_ZERO_SUP;

        // Turn off colon flag. Hour and minute separator.
        LCD_UPD_COLON1_FLAG OFF;

    }
    else
    {
        // Since hour data is non-zero, display the minute data without
        //   suppressing it.
        LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP;

        // Turn on colon flag. Hour and minute separator.
        LCD_UPD_COLON1_FLAG ON;
    }

    //=========================================================================
    //
    //                      DISPLAY HOUR DATA
    //
    //=========================================================================

    // Set the display position and the data to be displayed at IXReg 
    //   and AReg respectively.
    IXReg = CTMRSEGDIGIT1;
    AReg  = CTMRUserSetHour;

    LCD_DISP_2DIG_SEG_DATA_WITH_ZERO_SUP;

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrDisplayUserSetHMSAtSegment'"

        GLOBAL  ctmrDisplayUserSetHMSAtSegment

ctmrDisplayUserSetHMSAtSegment:                     ; **SUBROUTINE ctmrDisplayUserSetHMSAtSegment

                ;**************************************************************
                ; An overview of the LCD macro's used in this routine.
                ;
                ;  LCD macro that clears the entire segment display.
                ;      - Destroys BAReg, IXReg, LReg
                ;
                ;  LCD macro's that displays 2 digit data:
                ;      - Destroys BAReg, LReg, IXReg
                ;      - Input: IXReg - Starting digit column.
                ;               AReg  - Packed 10's and 1's digit BCD data.
                ;
                ;  LCD macro to display punctuation:
                ;      - Destroys HLReg
                ;**************************************************************

                ; Clear entire 9 segment display.
                LCD_CLR_SEG_LINE

                ;==============================================================
                ;
                ;                      DISPLAY SECONDS DATA
                ;
                ;==============================================================

                ; Set the display position and the data to be displayed at IXReg and AReg.
                ld      IX, #CTMRSEGDIGIT5
                ld      A, [CTMRUserSetSecond]
                
                ; Load the user set minute and hour data and check if it is zero.
                ld      HL, [CTMRUserSetMinute]
                cp      HL, #0
                jr      NZ, ctmrDispWorkSecSegNoSup
                
                ; Since hour and minute data is zero, display the seconds
                ;   data suppressing '0' in MSD.
                LCD_DISP_2DIG_SEG_DATA_SUP_ZERO_MSD
                        
                ; Turn off dot flag. Minute and second separator.
                LCD_UPD_PER2_FLAG   OFF
                
                jr      ctmrDispWorkSegMin

ctmrDispWorkSecSegNoSup:

                ; Since hour or minute data is non-zero, display the seconds
                ;   data without suppressing it.
                LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP
                
                ; Turn on dot flag. Minute and second separator.
                LCD_UPD_PER2_FLAG   ON

ctmrDispWorkSegMin:

                ;==============================================================
                ;
                ;                      DISPLAY MINUTE DATA
                ;
                ;==============================================================

                ; Set the display position and the data to be displayed at IXReg and AReg.
                ld      IX, #CTMRSEGDIGIT3
                ld      BA, [CTMRUserSetMinute]
                
                ; Check if hour data is zero.
                cp      B, #0
                jr      NZ, ctmrDispWorkMinSegNoSup

                ; Since hour data is zero, display the minute data with suppression
                ;   on both zeroes.
                LCD_DISP_2DIG_SEG_DATA_WITH_ZERO_SUP
                        
                ; Turn off colon flag. Hour and minute separator.
                LCD_UPD_COLON1_FLAG OFF

                jr      ctmrDispWorkSegHour

ctmrDispWorkMinSegNoSup:
                
                ; Since hour data is non-zero, display the minute data without
                ;   suppressing it.
                LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP
                
                ; Turn on colon flag. Hour and minute separator.
                LCD_UPD_COLON1_FLAG ON

ctmrDispWorkSegHour:

                ;==============================================================
                ;
                ;                      DISPLAY HOUR DATA
                ;
                ;==============================================================

                ; Set the display position and the data to be displayed at IXReg 
                ;   and AReg respectively.
                ld      IX, #CTMRSEGDIGIT1
                ld      A, [CTMRUserSetHour]

                LCD_DISP_2DIG_SEG_DATA_WITH_ZERO_SUP

                ret

                                                    ; **END SUBROUTINE ctmrDisplayUserSetHMSAtSegment





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrDisplayActionAtEndFlags
;
; Description : Displays the action at end flags.
;
; Assumptions : No timer related action at end flags are displayed.
;               
; Input(s)    : BReg - Contains the flag values that contains the action at end.
;
; Output(s)   : None
;
;               ( Destroyed: HLReg, IXReg )
;
; Module Change History:
;
;               11/14/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrDisplayActionAtEndFlags()
{
    push    (BAReg);

    //-------------------------------------------------------------------------
    // Clear timer related flags.
    // Destroys BAReg, HLReg, IXReg.
    //-------------------------------------------------------------------------
    LCD_UPD_TAIL_FLAG   OFF;
    LCD_UPD_ARROW_FLAG  OFF;
    LCD_UPD_CHR_FLAG    OFF;

    //-------------------------------------------------------------------------
    // Display the TMR flag.
    // Destroys HLReg.
    //-------------------------------------------------------------------------
    LCD_UPD_TMR_FLAG    ON;

    pop     (BAReg);

    // Clear the other value leaving only the action at end bits.
    BReg &= CTMRACTIONATENDMASKBITS;

    // Check if its repeat at end.
    if( BReg == CTMRREPEATATEND )
    {
        // Display "TAIL" flag.
        LCD_UPD_TAIL_FLAG   ON;
    }

    // Check if its chrono at end.
    elseif( BReg == CTMRCHRONOATEND )
    {
        // Display "CHR" and "ARROW" flag.
        LCD_UPD_ARROW_FLAG  ON;
        LCD_UPD_CHR_FLAG    ON;
    }
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrDisplayActionAtEndFlags'"

        GLOBAL  ctmrDisplayActionAtEndFlags

ctmrDisplayActionAtEndFlags:                        ; **SUBROUTINE ctmrDisplayActionAtEndFlags

                push    BA

                ;--------------------------------------------------------------
                ; Clear timer related flags.
                ; Destroys BAReg, HLReg, IXReg.
                ;--------------------------------------------------------------
                LCD_UPD_TAIL_FLAG   OFF
                LCD_UPD_ARROW_FLAG  OFF
                LCD_UPD_CHR_FLAG    OFF

                ;--------------------------------------------------------------
                ; Display the TMR flag.
                ; Destroys HLReg.
                ;--------------------------------------------------------------
                LCD_UPD_TMR_FLAG    ON

                pop     BA

                ; Clear the other value leaving only the action at end bits.
                and     B, #CTMRACTIONATENDMASKBITS
                
                ; Check if its repeat at end.
                cp      B, #CTMRREPEATATEND
                jr      NZ, ctmrDispActionAtEndFlagChk2
                
                ; Display "TAIL" flag.
                LCD_UPD_TAIL_FLAG   ON

                ret

ctmrDispActionAtEndFlagChk2:
                
                ; Check if its chrono at end.
                cp      B, #CTMRCHRONOATEND
                jr      NZ, ctmrDispActionAtEndFlagsExit
                
                ; Display "CHR" and "ARROW" flag.
                LCD_UPD_ARROW_FLAG  ON
                LCD_UPD_CHR_FLAG    ON

ctmrDispActionAtEndFlagsExit:

                ret


                                                    ; **END SUBROUTINE ctmrDisplayActionAtEndFlags



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrDisplayActionAtEndMsg
;
; Description : Displays the action at end message at line 1.
;
; Assumptions : None
;               
; Input(s)    : BReg - Contains the flag values that contains the action at end.
;
; Output(s)   : None
;
;               ( Destroyed: BReg )
;
; Module Change History:
;
;               11/14/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrDisplayActionAtEndMsg()
{

    // Clear line 1 DM.
    // Destroys AReg, HLReg.
    LCD_CLR_MAIN_DM_LINE1;

    // Clear the other value leaving only the action at end bits.
    BReg &= CTMRACTIONATENDMASKBITS;

    if( BReg == CTMRSTOPATEND )                          
    {                                                    
        LCD_DISP_SMALL_DM_MSG_STOP_AT_END;
    }                                                    
                                                         
    if( BReg == CTMRREPEATATEND )                        
    {
        LCD_DISP_SMALL_DM_MSG_REPEAT_AT_END;
    }

    elseif( BReg == CTMRCHRONOATEND )
    {
        LCD_DISP_SMALL_DM_MSG_CHRONO_AT_END;
    }
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrDisplayActionAtEndMsg'"

        GLOBAL  ctmrDisplayActionAtEndMsg

ctmrDisplayActionAtEndMsg:                          ; **SUBROUTINE ctmrDisplayActionAtEndMsg
                
                ; Clear line 1 DM.
                ; Destroys AReg, HLReg.
                LCD_CLR_MAIN_DM_LINE1

                ; Clear the other value leaving only the action at end bits.
                and     B, #CTMRACTIONATENDMASKBITS

                ; Check if its stop at end.
                cp      B, #CTMRSTOPATEND
                jr      NZ, ctmrDispActionAtEndMsgChk2

                LCD_DISP_SMALL_DM_MSG_STOP_AT_END

                ret
                
ctmrDispActionAtEndMsgChk2:

                ; Check if its repeat at end.
                cp      B, #CTMRREPEATATEND
                jr      NZ, ctmrDispActionAtEndMsgChk3

                LCD_DISP_SMALL_DM_MSG_REPEAT_AT_END

                ret

ctmrDispActionAtEndMsgChk3:
                
                LCD_DISP_SMALL_DM_MSG_CHRONO_AT_END

                ret


                                                    ; **END SUBROUTINE ctmrDisplayActionAtEndMsg




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrDisplayOneHalfAtSDM
;
; Description : Displays "1/2" at SDM if halfway reminder is enabled.
;
; Assumptions : The halfway reminder bit is in CTMRFlags.bCTMRHalfwayEnable
;               
; Input(s)    : None
;
; Output(s)   : None
;
;               ( Destroyed: BAReg, HLReg, IXReg )
;
; Module Change History:
;
;               11/14/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void tmrDisplayOneHalfAtSDM()
{
    // Check if the halfway reminder is enabled.
    AReg = CTMRFlags;
    if( AReg.bCTMRHalfwayEnable == TRUE )
    {
        //---------------------------------------------------------------------
        // Display "1/2" on the upper DM area.
        // Destroys BAReg, HLReg.
        //---------------------------------------------------------------------
        LCD_DISP_UPPER_DM_MSG_ONE_HALF;                 
    }

    else
    {
        //---------------------------------------------------------------------
        // Clears the upper DM area.
        // Destroys BAReg, IXReg.
        //---------------------------------------------------------------------
        LCD_CLEAR_UPPER_DM;
    }
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrDisplayOneHalfAtSDM'"

        GLOBAL  ctmrDisplayOneHalfAtSDM

ctmrDisplayOneHalfAtSDM:                            ; **SUBROUTINE ctmrDisplayOneHalfAtSDM

                ; Check if the halfway reminder is enabled.
                ld      A, [CTMRFlags]
                bit     A, #bCTMRHalfwayEnable
                jr      Z, ctmrClearUpperDM

                ;--------------------------------------------------------------
                ; Display "1/2" on the upper DM area.
                ; Destroys BAReg, HLReg.
                ;--------------------------------------------------------------
                LCD_DISP_UPPER_DM_MSG_ONE_HALF

                ret

ctmrClearUpperDM:

                ;--------------------------------------------------------------
                ; Clears the upper DM area.
                ; Destroys BAReg, IXReg.
                ;--------------------------------------------------------------
                LCD_CLEAR_UPPER_DM

                ret

                                                    ; **END SUBROUTINE ctmrDisplayOneHalfAtSDM






;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrDisplayTimerNameAtLine2
;
; Description : Displays the timer name at line 2.
;
; Assumptions : The first character of the name should be in CTMRName0.
;               
; Input(s)    : None
;
; Output(s)   : None
;
;               ( Destroyed: BReg, IXReg, LReg )
;
; Module Change History:
;
;               11/14/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrDisplayTimerNameAtLine2'"

        GLOBAL  ctmrDisplayTimerNameAtLine2

ctmrDisplayTimerNameAtLine2:                        ; **SUBROUTINE ctmrDisplayTimerNameAtLine2

                push    IY
                ;--------------------------------------------------------------
                ; Display an unformatted, small-font, proportional-width DM 
                ;   message.
                ; Destroys BAReg, HLReg, IXReg.
                ; Input: BReg  - Number of characters to be displayed.
                ;        IYReg - Starting address of the message.
                ;        IXReg - Starting display address.
                ;--------------------------------------------------------------
                ld      B, #CTMRNAMELENGTH
                ld      IX, #CTMRNAMEINITIALPOSITION
                ld      IY, #CTMRName0
                LCD_DISP_UNFORMATTED_SMALL_PROP_WIDTH_DM_MSG

                pop     IY
                ret

                                                    ; **END SUBROUTINE ctmrDisplayTimerNameAtLine2


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrClearSecondsLDM
;
; Description : Clear the seconds data in LDM.
;
; Assumptions : None
;               
; Input(s)    : None
;
; Output(s)   : None
;
;               ( Destroyed: TBD )
;
; Module Change History:
;
;               12/17/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrClearSecondsLDM'"

        GLOBAL  ctmrClearSecondsLDM

ctmrClearSecondsLDM:                                ; **SUBROUTINE ctmrClearSecondsLDM

                ;--------------------------------------------------------------
                ; Clears a small font, proportional-width 2-digit data.
                ; Destroys BAReg, HLReg, IXReg.
                ; Input: IXReg - Starting DM column.
                ;--------------------------------------------------------------
                ld      IX, #CTMRLDMPOSITIONSECOND
                LCD_CLR_SMALL_PROP_WIDTH_2DIG_DM_DATA

                ret

                                                    ; **END SUBROUTINE ctmrClearSecondsLDM


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrStaticDisplayOfField
;
; Description : Displays that will not be repeated when changing the data on
;                 a specific field.  This is to reduce the cycle time when
;                 the value of a data is being changed.
;
; Assumptions : None
;               
; Input(s)    : BReg - Contains the flag values that contains the action at end.
;
; Output(s)   : None
;
;               ( Destroyed: IYReg, IXReg, BAReg )
;
; Module Change History:
;
;               12/17/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrStaticDisplayOfField()
{
    push    (BAReg);
    LCD_CLR_DISPLAY;
    pop     (BAReg);

    // Display action at end flag.
    ctmrDisplayActionAtEndFlags();

    switch( CTMRCurrentSetPosition )
    {
        case    CTMRSETMINHOUR:
            //=================================================================
            //
            //                      HOUR AND MINUTE
            //
            //=================================================================
                            
            LCD_DISP_SEG_MSG_HR_MIN;
            LCD_UPD_DASH1_FLAG  ON;

            //-----------------------------------------------------------------         
            // Display user set seconds data.
            // Destroys BAReg, HLReg, IXReg.
            //-----------------------------------------------------------------         
            ctmrDisplayUserSetSecondAtBDM();
                                                                                                    
            break;                                                                 

        case    CTMRSETSECOND:
            //=================================================================
            //
            //                          SECOND
            //
            //=================================================================

            LCD_DISP_SEG_MSG_SECOND;
                
            //-----------------------------------------------------------------         
            // Display user set hour and minute data.
            // Destroys BAReg, HLReg, IXReg.
            //-----------------------------------------------------------------         
            ctmrDisplayUserSetHourMinuteAtBDM();

            break;

        case    CTMRSETACTIONATEND:
            //=================================================================
            //
            //                       ACTION AT END
            //
            //=================================================================

            break;

    } 

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrStaticDisplayOfField'"

        GLOBAL  ctmrStaticDisplayOfField

ctmrStaticDisplayOfField:                           ; **SUBROUTINE ctmrStaticDisplayOfField

                push    BA
                LCD_CLR_DISPLAY
                pop     BA
                
                ; Display action at end flag.
                car     ctmrDisplayActionAtEndFlags


                ld      A, [CTMRCurrentSetPosition]

                cp      A, #CTMRSETMINHOUR
                jr      Z, ctmrStaticDispHourMinute

                cp      A, #CTMRSETSECOND
                jr      Z, ctmrStaticDispSecond

                ;==============================================================
                ;
                ;                       ACTION AT END
                ;
                ;==============================================================
                
                ret

ctmrStaticDispHourMinute:
                ;==============================================================
                ;
                ;                      HOUR AND MINUTE
                ;
                ;==============================================================
                                            
                LCD_DISP_SEG_MSG_HR_MIN
                LCD_UPD_DASH1_FLAG  ON

                ;--------------------------------------------------------------
                ; Display user set seconds data.
                ; Destroys BAReg, HLReg, IXReg.
                ;--------------------------------------------------------------
                jr      ctmrDisplayUserSetSecondAtBDM   ; **EXTERNAL JUMP
                        
ctmrStaticDispSecond:                                                                                                   
                ;==============================================================
                ;
                ;                          SECOND
                ;
                ;==============================================================
                
                LCD_DISP_SEG_MSG_SECOND
                
                ;--------------------------------------------------------------
                ; Display user set hour and minute data.
                ; Destroys BAReg, HLReg, IXReg.
                ;--------------------------------------------------------------
                jr      ctmrDisplayUserSetHourMinuteAtBDM   ; **EXTERNAL JUMP


                                                    ; **END SUBROUTINE ctmrStaticDisplayOfField



        CALLS   SUBROUTINE, 'ctmrDisplayActionAtEndFlags'

;**END FILE ctmrdisp.asm


        END

















