;**FILE ctmrutil.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2000 - 2002 TIMEX Corporation.
; 
; This document is the property of, and contains information proprietary to
; TIMEX Corporation.  It is to be returned upon request.  Reproduction by any
; means, disclosure, or use of its contents for other than the expressed
; purpose for which it is loaned, without the written consent of TIMEX
; Corporation, is strictly prohibited.
;
; File Name    : ctmrutil.asm
;
; Module Names : ctmrNextActionAtEnd            - Point to the next action at end.
;                ctmrPrevActionAtEnd            - Point to the previous action at end.
;
;                ctmrUpdateFlagStatus           - Updates the status of the timer 
;                                                   according to its data.
;               
;                ctmrGoToNextSetItem            - Go to the next set item.
;                ctmrGoToPrevSetItem            - Go to the previous set item.
;                
;                ctmrSetupHalfwayReminder       - Sets up the halfway reminder if enabled.
;
;                ctmrIncField                   - Increments the user set data or action at end.
;                ctmrDecField                   - Decrements the user set data or action at end.
;                
;                ctmrCopyUserSetToWorkBuffer    - Copies the user set data to work buffer.
;
;                ctmrReadNextRecord             - Reads the next DB record. Resets the active 
;                                                   resource.  Clears the entire screen.
;                ctmrReadPrevRecord             - Reads the previous DB record. Resets the active 
;                                                   resource.  Clears the entire screen.
;                
;                ctmrReadOneRecord              - Reads a record from the timer database.
;
;                ctmrWriteStatusFlagToEEPROM    - Writes the status flag to EEPROM
;
;                ctmrWriteStatusFlagAndHMSDataToEEPROM - Writes the status flag and user set 
;                                                          data to EEPROM.
;
;                ctmrUpdateTurnOffResourceFlags - Turns off the LCD resource flags.
;               
;                ctmrUpdateResourceFlags        - Updates the LCD resource flags.
;
;                
;
;                
;
;
;
; Purpose      : Countdown and Interval Timer common utilities.
;
; File Change History:
;
;               12/12/2001  - Created for M851 development.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

$CASE ON
$LIST OFF

        INCLUDE '..\..\CORE\H\MODEL.H'
        INCLUDE '..\..\HW\H\HW88349.H'
        INCLUDE '..\..\HW\H\HWI2CEE.H'
        INCLUDE '..\..\HW\H\HWBOND.H'
        INCLUDE '..\..\HW\H\HWKBD.H'
        INCLUDE '..\..\HW\H\HWMACRO.H'
        INCLUDE '..\..\HW\H\HWMAIN.H'
        INCLUDE '..\..\HW\H\HWRESET.H'
        INCLUDE '..\..\HW\H\HWVARS.H'
        INCLUDE '..\..\HW\H\HWSERMAC.H'
        INCLUDE '..\..\CORE\H\CORE.H'
        INCLUDE '..\..\CORE\H\COREMAC.H'
        INCLUDE '..\..\CORE\H\CORESVMC.H'
        INCLUDE '..\..\COMM\H\COMM.H'
        INCLUDE '..\..\LCD\H\LCD.H'
        INCLUDE '..\..\LCD\H\LCDMACRO.H'
        INCLUDE '..\..\LCD\H\LCDRAM.H'
        INCLUDE '..\..\DATABASE\H\DB.H'
        INCLUDE '..\..\DATABASE\H\DBMACRO.H'
        INCLUDE '..\..\AUDIO\H\AUD.H'
        INCLUDE '..\..\AUDIO\H\AUDMAC.H'
        INCLUDE '..\..\UTIL\H\UTLMATH.H'
        INCLUDE '..\..\UTIL\H\UTLMTHMC.H'
        INCLUDE '..\..\UTIL\H\UTLEDT.H'
        INCLUDE '..\..\UTIL\H\UTLEDTMC.H'
        INCLUDE '..\..\UTIL\H\UTLDSP.H'
        INCLUDE '..\..\UTIL\H\UTLDSPMC.H'
        INCLUDE '..\..\UTIL\H\UTLACCMC.H'
        INCLUDE '..\..\UTIL\H\UTLMSC.H'
        INCLUDE '..\..\UTIL\H\UTLMSCMC.H'
        INCLUDE '..\..\UTIL\H\UTLVARS.H'
        INCLUDE '..\..\KRES\H\KRES.H'
        INCLUDE '..\..\KRES\H\KRESMAC.H'
        INCLUDE '..\..\KRES\H\KSTP.H'
        INCLUDE '..\..\KRES\H\KSTPMAC.H'
        INCLUDE '..\..\KRES\H\KSYN.H'
        INCLUDE '..\..\KRES\H\KSYNMAC.H'
        INCLUDE '..\..\KRES\H\KTMR.H'
        INCLUDE '..\..\KRES\H\KTMRMAC.H'
        INCLUDE '..\..\KRES\H\KTOD.H'
        INCLUDE '..\..\KRES\H\KTODMAC.H'
        INCLUDE '..\..\KRES\H\KTZC.H'
        INCLUDE '..\..\KRES\H\KTZCMAC.H'
        INCLUDE '..\..\KRES\H\KBCK.H'
        INCLUDE '..\..\KRES\H\KBCKMAC.H'
        INCLUDE '..\..\USAGE\H\USG.H'
        INCLUDE '..\..\USAGE\H\USGVARS.H'
        INCLUDE '..\..\USAGE\H\USGMAC.H'
        INCLUDE '..\..\CORE\H\SECTION.H'

        INCLUDE '..\..\TOD\H\TOD.H'
        INCLUDE '..\..\TOD\H\TODVARS.H'

        INCLUDE '..\..\CTMR\h\CTMR.H'
        INCLUDE '..\..\CTMR\h\CTMRVARS.H'

$LIST ON


        SECT    ".text_commontimer"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrNextActionAtEnd
;
; Description : Point to the next action at end.
;
; Assumptions : Action at end bits must use bit 0 and 1.
;
; Input(s)    : HLReg - Address that contains the action at end data.
;               
; Output(s)   : None
;
;               ( Destroyed: AReg )
;
; Module Change History:
;
;               12/12/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


COMMENT `

void ctmrNextActionAtEnd()
{
    // Get the action at end and clear other bits leaving only the action at end.
    AReg = *HLReg;
    AReg &= CTMRACTIONATENDMASKBITS;

    // Point to the next action at end and check if it has exceeded the maximum
    //   type of action at end.
    AReg++;
    if( AReg >= CTMRMAXACTIONATENDITEM )
    {
        // Yup, get the first action at end type.
        AReg = CTMRFIRSTACTIONATENDITEM;
    }

    // Delete/clear the previous action at end and load the new action at end.
    *HLReg &= ~CTMRACTIONATENDMASKBITS;
    *HLReg |= AReg;

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrNextActionAtEnd'"

        GLOBAL  ctmrNextActionAtEnd


ctmrNextActionAtEnd:                                ; **SUBROUTINE ctmrNextActionAtEnd

                ; Get the action at end and clear other bits leaving only the action at end.
                ld      A, [HL]
                and     A, #CTMRACTIONATENDMASKBITS

                ; Point to the next action at end and check if it has exceeded the maximum
                ;   type of action at end.
                inc     A
                cp      A, #CTMRMAXACTIONATENDITEM
                jr      C, ctmrLoadNewActionAtEndCommon
                
                ; Yup, get the first action at end type.
                ld      A, #CTMRFIRSTACTIONATENDITEM
                
                jr      ctmrLoadNewActionAtEndCommon; **EXTERNAL JUMP


                                                    ; **END SUBROUTINE ctmrNextActionAtEnd




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrPrevActionAtEnd
;
; Description : Point to the previous action at end.
;
; Assumptions : Action at end bits must use bit 0 and 1.
;
; Input(s)    : HLReg - Address that contains the action at end data.
;                   
; Output(s)   : None
;
;               ( Destroyed: AReg )
;
; Module Change History:
;
;               12/12/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrPrevActionAtEnd()
{

    // Get the action at end and clear other bits leaving only the action at end.
    AReg = *HLReg;
    AReg &= CTMRACTIONATENDMASKBITS;

    // Point to the prev action at end.  Use "SUB".  I need the carry flag result.
    AReg -= 1;
    if( bCarryFlag == TRUE )
    {
        // Get the last action at end type.
        AReg = CTMRLASTACTIONATENDITEM;
    }

    // Delete/clear the previous action at end and load the new action at end.
    *HLReg &= ~CTMRACTIONATENDMASKBITS;
    *HLReg |= AReg;

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrPrevActionAtEnd'"

        GLOBAL  ctmrPrevActionAtEnd


ctmrPrevActionAtEnd:                                ; **SUBROUTINE ctmrPrevActionAtEnd

                ; Get the action at end and clear other bits leaving only the action at end.
                ld      A, [HL]
                and     A, #CTMRACTIONATENDMASKBITS
                
                ; Point to the prev action at end.  Use "SUB".  I need the carry flag result.
                sub     A, #1
                jr      NC, ctmrLoadNewActionAtEndCommon
                                    
                ; Get the last action at end type.
                ld      A, #CTMRLASTACTIONATENDITEM

ctmrLoadNewActionAtEndCommon:
                
                ; Delete/clear the previous action at end and load the new action at end.
                and     [HL], #@LOW(~CTMRACTIONATENDMASKBITS)
                or      [HL], A
                
                ret
    

                                                    ; **END SUBROUTINE ctmrPrevActionAtEnd




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrUpdateFlagStatus
;
; Description : Updates the status of the timer according to its data.
;
; Assumptions : Structure format: Status flag, second, minute, hour.
;
;               The data to be checked can be found in CTMRFlags, CTMRUserSetSecond,
;                 CTMRUserSetMinute, CTMRUserSetHour
;                   
; Input(s)    : None
;               
; Output(s)   : bCTMRLess1Min      - 1, if data is less than 1 minute.
;               bCTMRLess15Sec     - 1, if data is less than 15 seconds.
;               bCTMRDataIsZero    - 1, if data is 0.
;               bCTMRHalfwayEnable - 0, if data is less than 1 minute.
;               
;               ( Destroyed: IXReg )
;               
; Module Change History:
;
;               02/28/2002  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrUpdateFlagStatus()
{

    push    (HLReg);
    push    (BAReg);

    // Assume that the data is more than 1 minute.  If it is more than 1 minute,
    //   definitely it is more than 15 seconds and not equal to 0.
    HLReg = &CTMRFlags;
    *HLReg &= ~(bCTMRLess1Min | bCTMRLess15Sec | bCTMRDataIsZero);

    // Check if data is less than 1 minute.  Data is less than 1 minute if hour
    //   and minute data is 0.
    IXReg = &CTMRUserSetMinute;
    BAReg = *IXReg;
    if( BAReg == 0 )
    {
        // Since it is less than 1 minute, set the less 1 minute flag and force 
        //   the halfway alert tracking to be disabled.
        *HLReg |= bCTMRLess1Min;
        *HLReg &= ~bCTMRHalfwayEnable;

        // Point to the second data address and check if its less than 15.
        IXReg--;
        AReg = *IXReg;
        if( AReg < 15h )
        {
            // Since it is less than 15, set the less 15 seconds flag.
            *HLReg |= bCTMRLess15Sec;

            // Check if it is 0.
            if( AReg == 0 )
            {
                // Since it is 0, set the data is zero bit.
                *HLReg |= bCTMRDataIsZero;
            }
        }
    }

    pop     (BAReg);
    pop     (HLReg);
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrUpdateFlagStatus'"

        GLOBAL  ctmrUpdateFlagStatus

ctmrUpdateFlagStatus:                               ; **SUBROUTINE ctmrUpdateFlagStatus

                push    HL
                push    BA
                
                ; Assume that the data is more than 1 minute.  If it is more than 1 minute,
                ;   definitely it is more than 15 seconds and not equal to 0.
                ld      HL, #CTMRFlags
                and     [HL], #@LOW(~(bCTMRLess1Min | bCTMRLess15Sec | bCTMRDataIsZero))
                
                ; Check if data is less than 1 minute.  Data is less than 1 minute if hour
                ;   and minute data is 0.
                ld      IX, #CTMRUserSetMinute
                ld      BA, [IX]
                cp      BA, #0
                jr      NZ, ctmrUpdFlagStatusExit

                ; Since it is less than 1 minute, set the less 1 minute flag and force 
                ;   the halfway alert tracking to be disabled.
                or      [HL], #bCTMRLess1Min
                and     [HL], #@LOW(~bCTMRHalfwayEnable)
                
                ; Point to the second data address and check if its less than 15.
                dec     IX
                ld      A, [IX]
                cp      A, #15h
                jr      NC, ctmrUpdFlagStatusExit
                
                ; Since it is less than 15, set the less 15 seconds flag.
                or      [HL], #bCTMRLess15Sec
                
                ; Check if it is 0.
                cp      A, #0
                jr      NZ, ctmrUpdFlagStatusExit
                
                ; Since it is 0, set the data is zero bit.
                or      [HL], #bCTMRDataIsZero

ctmrUpdFlagStatusExit:
                        
                pop     BA
                pop     HL

                ret


                                                    ; **END SUBROUTINE ctmrUpdateFlagStatus


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrGoToNextSetItem
;
; Description : Go to the next set item.
;
; Assumptions : None
;
; Input(s)    : None
;
; Output(s)   : None
;
;               ( Destroyed: HLReg )
;
; Module Change History:
;
;               12/17/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrGoToNextSetItem()
{
    // Go to the next set item.
    CTMRCurrentSetPosition++;

    if( CTMRCurrentSetPosition >= CTMRMAXSETITEMS )
    {
        CTMRCurrentSetPosition = CTMRFIRSTSETITEM;  
    }
}


ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrGoToNextSetItem'"

        GLOBAL  ctmrGoToNextSetItem


ctmrGoToNextSetItem:                                ; **SUBROUTINE ctmrGoToNextSetItem

                ld      HL, #CTMRCurrentSetPosition

                ; Go to the next set item.
                inc     [HL]

                cp      [HL], #CTMRMAXSETITEMS
                jr      C, ctmrGoToNextSetItemExit
                
                ld      [HL], #CTMRFIRSTSETITEM


ctmrGoToNextSetItemExit:

                ret


                                                    ; **END SUBROUTINE ctmrGoToNextSetItem






;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrGoToPrevSetItem
;
; Description : Go to the previous set item.
;
; Assumptions : None
;
; Input(s)    : None
;
; Output(s)   : None
;
;               ( Destroyed: HLReg )
;
; Module Change History:
;
;               12/17/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrGoToPrevSetItem()
{
    // Go to the previous set item.
    CTMRCurrentSetPosition -= 1;

    if( bCarryFlag == TRUE )
    {
        CTMRCurrentSetPosition = CTMRLASTSETITEM;   
    }
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrGoToPrevSetItem'"

        GLOBAL  ctmrGoToPrevSetItem

ctmrGoToPrevSetItem:                                ; **SUBROUTINE ctmrGoToPrevSetItem

                
                ld      HL, #CTMRCurrentSetPosition

                ; Go to the previous set item.
                sub     [HL], #1

                jr      NC, ctmrGoToPrevSetItemExit

                ld      [HL], #CTMRLASTSETITEM

ctmrGoToPrevSetItemExit:

                ret


                                                    ; **END SUBROUTINE ctmrGoToPrevSetItem


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrSetupHalfwayReminder
;
; Description : Sets up the halfway reminder if enabled.
;
; Assumptions : None
;
; Input(s)    : AReg - Resource index.
;               BReg - Contains the halfways reminder bit.
;               IYReg - Seconds address of the user set data.               
;
; Output(s)   : None
; 
;               ( Destroyed: IYReg, IXReg, HLReg )
;           
; Module Change History:
;
;               12/14/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrSetupHalfwayReminder()
{

    //-------------------------------------------------------------------------
    // Disable halfway reminder tracking assuming that it is disabled.
    // Destroys nothing.
    // Input: AReg - Resouce index.
    //-------------------------------------------------------------------------
    KTMR_DISABLE_PREWARNING_TRACKING;

    // Check if halfway reminder is enabled.
    if( BReg.bCTMRHalfwayEnable == TRUE )
    {

        push    (BAReg);

        //---------------------------------------------------------------------
        // Sets up the data to be used to compute for the halftime value.
        //---------------------------------------------------------------------
        CTMRBackgroundHalfHundredths = 0;

        // Copy user set time to halftime computation buffer.
        BReg = (KTMRUSERHOUROFFSET - KTMRUSERSECONDOFFSET) + 1;
        IXReg = &CTMRBackgroundHalfSecond; 
        push    (IYReg);
        UTL_COPY_IYREG_TO_IXREG;
               
        //---------------------------------------------------------------------
        // Compute the halfway reminder time.
        // Destroys BReg, HLReg.
        // Input: IXReg - Points to the LSB of the data buffer (BCD Format).
        //        AReg  - Divisor (HEX Format)
        //---------------------------------------------------------------------
        IXReg = &CTMRBackgroundHalfHundredths;                                    
        AReg = 2;                                                                 
        UTL_DIVIDE_HMSH_BY_N_IN_BCD;                                              
                                                                                  
        //---------------------------------------------------------------------   
        // Check if user set time is an odd number.  If yes, then increment
        //   halftime so that it would generate an alert 0.5 seconds earlier.
        //---------------------------------------------------------------------
        pop     (IYReg);
        AReg = *IYReg;                                 
        if( AReg.0 == TRUE )                           
        {
            // Setup KRESxxxBuffer with the value to be added to halftime.
            COREEventArgument = 1;
            UTLACCELERATION_1SEC;
            
            // Setup the maximum hours for timer data.
            KRESHrRollOverValue = 24h;

            // Update the halftime data.
            HLReg = &CTMRBackgroundHalfSecond;
            KRES_SECMINHR_UPD;
        }                                              

        //---------------------------------------------------------------------
        // Sets up the pre-warning data.
        // Input: AReg  - Resource index.
        //        IYReg - Points to the beginning of the seconds, minutes and
        //                  hour structure of the user RAM area.
        //---------------------------------------------------------------------
        pop     (BAReg);

        IYReg = &CTMRBackgroundHalfSecond;
        KTMR_SETUP_PREWARNING_HMS;

    }
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrSetupHalfwayReminder'"

        GLOBAL  ctmrSetupHalfwayReminder

ctmrSetupHalfwayReminder:                           ; **SUBROUTINE ctmrSetupHalfwayReminder

                ;--------------------------------------------------------------
                ; Disable halfway reminder tracking assuming that it is disabled.
                ; Destroys nothing.
                ; Input: AReg - Resouce index.
                ;--------------------------------------------------------------
                KTMR_DISABLE_PREWARNING_TRACKING
                
                ; Check if halfway reminder is enabled.
                bit     B, #bCTMRHalfwayEnable
                jr      Z, ctmrSetupHalfwayExit

                push    BA

                ;--------------------------------------------------------------
                ; Sets up the data to be used to compute for the halftime value.
                ;--------------------------------------------------------------
                ld      A, #0
                ld      [CTMRBackgroundHalfHundredths], A
                
                ; Copy user set time to halftime computation buffer.
                ld      B, #(KTMRUSERHOUROFFSET - KTMRUSERSECONDOFFSET) + 1
                ld      IX, #CTMRBackgroundHalfSecond
                push    IY
                UTL_COPY_IYREG_TO_IXREG

                ;--------------------------------------------------------------
                ; Compute the halfway reminder time.
                ; Destroys BReg, HLReg.
                ; Input: IXReg - Points to the LSB of the data buffer (BCD Format).
                ;        AReg  - Divisor (HEX Format)
                ;--------------------------------------------------------------
                ld      IX, #CTMRBackgroundHalfHundredths
                ld      A, #2
                UTL_DIVIDE_HMSH_BY_N_IN_BCD

                ;--------------------------------------------------------------
                ; Check if user set time is an odd number.  If yes, then 
                ;   increment halftime so that it would generate an alert 
                ;   0.5 seconds earlier.
                ;--------------------------------------------------------------
                pop     IY
                ld      A, [IY]
                bit     A, #00000001B
                jr      Z, ctmrContinueSetupHalfway

                ; Setup KRESxxxBuffer with the value to be added to halftime.
                ld      A, #1
                ld      [COREEventArgument], A
                UTLACCELERATION_1SEC
                        
                ; Setup the maximum hours for timer data.
                ld      A, #24h
                ld      [KRESHrRollOverValue], A
                
                ; Update the halftime data.
                ld      HL, #CTMRBackgroundHalfSecond
                KRES_SECMINHR_UPD

ctmrContinueSetupHalfway:

                ;--------------------------------------------------------------
                ; Sets up the pre-warning data.
                ; Input: AReg  - Resource index.
                ;        IYReg - Points to the beginning of the seconds, minutes 
                ;                   and hour structure of the user RAM area.
                ;--------------------------------------------------------------
                pop     BA

                ld      IY, #CTMRBackgroundHalfSecond
                KTMR_SETUP_PREWARNING_HMS

ctmrSetupHalfwayExit:

                ret


                                                    ; **END SUBROUTINE ctmrSetupHalfwayReminder


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrIncField
;
; Description : Increments the user set data or action at end.
;
; Assumptions : None
;
; Input(s)    : HLReg - Address that contains the action at end bits.  This
;                         is only useful when setting the action at end.
;
; Output(s)   : None
;
;               ( Destroyed: HLReg, AReg )
;
; Module Change History:
;
;               12/17/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrIncField()
{

    push    (IYReg);

    switch( CTMRCurrentSetPosition )
    {
        case    CTMRSETMINHOUR:
            //=================================================================
            //
            //                      HOUR AND MINUTE
            //
            //=================================================================
                    
            //-----------------------------------------------------------------
            // Determine the acceleration factor for COREEventArgument and
            //   write factor into buffer.
            // Stores the acceleration data to 'KRESMinBuffer'.
            //-----------------------------------------------------------------
            UTLACCELERATION_1MIN;

            // Setup the roll-over value so that it will automatically update
            //   if hour data exceeds 23.
            KRESHrRollOverValue = 24h;

            // Load HLReg the address of the user set minute.
            HLReg = &CTMRUserSetMinute;     

            // Adds the acceleration data to the user set data.
            KRES_MINHR_UPD;

            break;                                                                 


        case    CTMRSETSECOND:
            //=================================================================
            //
            //                          SECOND
            //
            //=================================================================

            //-----------------------------------------------------------------
            // Determine the acceleration factor for COREEventArgument and
            //   write factor into buffer.
            // Stores the acceleration data to 'KRESSecBuffer'.
            //-----------------------------------------------------------------
            UTLACCELERATION_1SEC;

            // Setup the roll-over value so that it will automatically update
            //   if hour data exceeds 23.
            KRESHrRollOverValue = 24h;

            // Set HLReg to user set second data address.
            HLReg = &CTMRUserSetSecond;     
            
            // Load the acceleration data.
            AReg = KRESSecBuffer;

            UTL_DECIMAL_MATH_MODE;

            // Add the acceleration data to user set seconds and check
            //   if its valid.
            *HLReg += AReg;
            if( *HLReg >= 60 )
            {
                // Subtract 60 from data to make it valid.
                *HLReg -= 60;       
            }

            UTL_BINARY_MATH_MODE;

            break;


        case    CTMRSETACTIONATEND:
            //=================================================================
            //
            //                       ACTION AT END
            //
            //=================================================================
                
            // Get the information pointed by HLReg and mask the bits leaving
            //   only the action at end.
            AReg = *HLReg;
            AReg &= CTMRACTIONATENDMASKBITS;

            // Point to the next action at end.
            AReg++;                           
            if( AReg >= CTMRMAXACTIONATENDITEM )      
            {
                AReg = CTMRFIRSTACTIONATENDITEM;
            }
            
            *HLReg &= @LOW(~CTMRACTIONATENDMASKBITS);
            *HLReg |= AReg;

            break;

    }

    pop     (IYReg);

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrIncField'"

        GLOBAL  ctmrIncField

ctmrIncField:                                       ; **SUBROUTINE ctmrIncField

                push    IY

                ld      A, [CTMRCurrentSetPosition]

                cp      A, #CTMRSETMINHOUR
                jr      Z, ctmrIncFieldHrMin

                cp      A, #CTMRSETSECOND
                jr      Z, ctmrIncFieldSecond

                ;==============================================================
                ;
                ;                       ACTION AT END
                ;
                ;==============================================================
                
                ; Get the information pointed by HLReg and mask the bits leaving
                ;   only the action at end.
                ld      A, [HL]
                and     A, #CTMRACTIONATENDMASKBITS
                
                ; Point to the next action at end.
                inc     A
                cp      A, #CTMRMAXACTIONATENDITEM
                jr      C, ctmrIncFieldLoadNewActionAtEnd

                ld      A, #CTMRFIRSTACTIONATENDITEM

ctmrIncFieldLoadNewActionAtEnd:
                
                ; Erase the previous action at end and load the new ones.
                and     [HL], #@LOW(~CTMRACTIONATENDMASKBITS)
                or      [HL], A

                jr      ctmrIncPopIYAndExit

ctmrIncFieldHrMin:
                ;==============================================================
                ;
                ;                      HOUR AND MINUTE
                ;
                ;==============================================================
                                    
                ;--------------------------------------------------------------
                ; Determine the acceleration factor for COREEventArgument and
                ;   write factor into buffer.
                ; Stores the acceleration data to 'KRESMinBuffer'.
                ;--------------------------------------------------------------
                UTLACCELERATION_1MIN
                
                ; Setup the roll-over value so that it will automatically update
                ;   if hour data exceeds 23.
                ld      A, #24h
                ld      [KRESHrRollOverValue], A
                
                ; Load HLReg the address of the user set minute.
                ld      HL, #CTMRUserSetMinute

                ; Adds the acceleration data to the user set data.
                KRES_MINHR_UPD
                
                jr      ctmrIncPopIYAndExit

ctmrIncFieldSecond:
                ;==============================================================
                ;
                ;                          SECOND
                ;
                ;==============================================================

                ;--------------------------------------------------------------
                ; Determine the acceleration factor for COREEventArgument and
                ;   write factor into buffer.
                ; Stores the acceleration data to 'KRESSecBuffer'.
                ;--------------------------------------------------------------
                UTLACCELERATION_1SEC
                
                ; Setup the roll-over value so that it will automatically update
                ;   if hour data exceeds 23.
                ld      A, #24h
                ld      [KRESHrRollOverValue], A

                ; Set HLReg to user set second data address.
                ld      HL, #CTMRUserSetSecond
            
                ; Load the acceleration data.
                ld      A, [KRESSecBuffer]
                
                UTL_DECIMAL_MATH_MODE

                ; Add the acceleration data to user set seconds and check
                ;   if its valid.
                add     [HL], A
                cp      [HL], #60h
                jr      C, ctmrIncFieldExit  
                
                ; Subtract 60 from data to make it valid.
                sub     [HL], #60h

ctmrIncFieldExit:

                UTL_BINARY_MATH_MODE

ctmrIncPopIYAndExit:

                pop     IY

                ret

                                                    ; **END SUBROUTINE ctmrIncField


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrDecField
;
; Description : Decrements the user set data or action at end.
;
; Assumptions : None
;
; Input(s)    : HLReg - Address that contains the action at end bits.  This
;                         is only useful when setting the action at end.
;
; Output(s)   : None
;
;               ( Destroyed: HLReg, AReg )
;
; Module Change History:
;
;               12/17/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrDecField()
{

    push    (IYReg);

    switch( CTMRCurrentSetPosition )
    {
        case    CTMRSETMINHOUR:
            //=================================================================
            //
            //                      HOUR AND MINUTE
            //
            //=================================================================
                
            //-----------------------------------------------------------------
            // Determine the acceleration factor for COREEventArgument and
            //   write factor into buffer.
            // Stores the acceleration data to 'KRESMinBuffer'.
            //-----------------------------------------------------------------
            UTLACCELERATION_1MIN;
            
            // Setup the roll-over value so that it will automatically update
            //   if hour data exceeds 23.
            KRESHrRollOverValue = #24h;

            // Load HLReg the address of the user set minute.
            HLReg = &CTMRUserSetMinute;     
                
            // Subtracts the acceleration data to the user set data.
            KRES_MINHR_SUB;

            break;                                                                 


        case    CTMRSETSECOND:
            //=================================================================
            //
            //                          SECOND
            //
            //=================================================================

            //-----------------------------------------------------------------
            // Determine the acceleration factor for COREEventArgument and
            //   write factor into buffer.
            // Stores the acceleration data to 'KRESSecBuffer'.
            //-----------------------------------------------------------------
            UTLACCELERATION_1SEC;

            // Setup the roll-over value so that it will automatically update
            //   if hour data exceeds 23.
            KRESHrRollOverValue = #24h;

            // Load HLReg the address of the user set second.
            HLReg = &CTMRUserSetSecond;     

            // Load the acceleration data.
            AReg = KRESSecBuffer;

            UTL_DECIMAL_MATH_MODE;

            // Subtract the acceleration data to user set seconds and check
            //   if its valid.
            *HLReg -= AReg;
            if( bCarryFlag == TRUE )
            {
                // Subtract 40 from seconds to validate the result.
                *HLReg -= 40;
            }

            UTL_BINARY_MATH_MODE;

            break;


        case    CTMRSETACTIONATEND:
            //=================================================================
            //
            //                       ACTION AT END
            //
            //=================================================================

            // Get the information pointed by HLReg and mask the bits leaving
            //   only the action at end.
            AReg = *HLReg;
            AReg &= CTMRACTIONATENDMASKBITS;

            // Point to the next action at end.
            AReg -= 1;                            
            if( bCarryFlag == TRUE )
            {
                AReg = CTMRLASTACTIONATENDITEM;
            }
            
            *HLReg &= @LOW(~CTMRACTIONATENDMASKBITS);
            *HLReg |= AReg;

            break;
    }

    pop     (IYReg);

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrDecField'"

        GLOBAL  ctmrDecField

ctmrDecField:                                       ; **SUBROUTINE ctmrDecField

                push    IY

                ld      A, [CTMRCurrentSetPosition]

                cp      A, #CTMRSETMINHOUR
                jr      Z, ctmrDecFieldHrMin

                cp      A, #CTMRSETSECOND
                jr      Z, ctmrDecFieldSecond

                ;==============================================================
                ;
                ;                       ACTION AT END
                ;
                ;==============================================================
                
                ; Get the information pointed by HLReg and mask the bits leaving
                ;   only the action at end.
                ld      A, [HL]
                and     A, #CTMRACTIONATENDMASKBITS
                
                ; Point to the next action at end.
                sub     A, #1
                jr      NC, ctmrDecFieldLoadNewActionAtEnd

                ld      A, #CTMRLASTACTIONATENDITEM
                            
ctmrDecFieldLoadNewActionAtEnd:

                and     [HL], #@LOW(~CTMRACTIONATENDMASKBITS)
                or      [HL], A

                jr      ctmrDecPopIYAndExit

ctmrDecFieldHrMin:
                ;==============================================================
                ;
                ;                      HOUR AND MINUTE
                ;
                ;==============================================================
                
                ;--------------------------------------------------------------
                ; Determine the acceleration factor for COREEventArgument and
                ;   write factor into buffer.
                ; Stores the acceleration data to 'KRESMinBuffer'.
                ;--------------------------------------------------------------
                UTLACCELERATION_1MIN
                            
                ; Setup the roll-over value so that it will automatically update
                ;   if hour data exceeds 23.
                ld      A, #24h
                ld      [KRESHrRollOverValue], A
                
                ; Load HLReg the address of the user set minute.
                ld      HL, #CTMRUserSetMinute
                                
                ; Subtracts the acceleration data to the user set data.
                KRES_MINHR_SUB

                jr      ctmrDecPopIYAndExit


ctmrDecFieldSecond:
                ;==============================================================
                ;
                ;                          SECOND
                ;
                ;==============================================================

                ;--------------------------------------------------------------
                ; Determine the acceleration factor for COREEventArgument and
                ;   write factor into buffer.
                ; Stores the acceleration data to 'KRESSecBuffer'.
                ;--------------------------------------------------------------
                UTLACCELERATION_1SEC
                
                ; Setup the roll-over value so that it will automatically update
                ;   if hour data exceeds 23.
                ld      A, #24h
                ld      [KRESHrRollOverValue], A

                ; Load HLReg the address of the user set second.
                ld      HL, #CTMRUserSetSecond
                
                ; Load the acceleration data.
                ld      A, [KRESSecBuffer]

                UTL_DECIMAL_MATH_MODE

                ; Subtract the acceleration data to user set seconds and check
                ;   if its valid.
                sub     [HL], A
                jr      NC, ctmrDecFieldSecondExit
                
                ; Subtract 40 from seconds to validate the result.
                sub     [HL], #40h

ctmrDecFieldSecondExit:

                UTL_BINARY_MATH_MODE

ctmrDecPopIYAndExit:

                pop     IY

                ret

                                                    ; **END SUBROUTINE ctmrDecField


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrCopyUserSetToWorkBuffer
;
; Description : Copies the user set data to work buffer.
;
; Assumptions : None
;
; Input(s)    : None
;
; Output(s)   : None
;
;               ( Destroyed: HLReg )
;
; Module Change History:
;
;               03/05/2002  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrCopyUserSetToWorkBuffer'"

        GLOBAL  ctmrCopyUserSetToWorkBuffer

ctmrCopyUserSetToWorkBuffer:                        ; **SUBROUTINE ctmrCopyUserSetToWorkBuffer

                ld      HL, [CTMRUserSetSecond]
                ld      [CTMRWorkSecond], HL
                ld      L, [CTMRUserSetHour]
                ld      [CTMRWorkHour], L
    
                ret

                                                    ; **END SUBROUTINE ctmrCopyUserSetToWorkBuffer



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrReadNextRecord
;
; Description : Reads the next DB record.
;               Resets the active resource.
;               Clears the entire screen.
;
; Assumptions : CORECurrentADDAddress - Absolute address of DB in external memory.
;               CTMRNumberOfRecords   - Number of records in DB.
;               CTMRLastViewedEntry   - Record number to be read.
;               CTMRActiveResource    - Resource to be reset.
;
; Input(s)    : None
;
; Output(s)   : AReg, CTMRLastViewedEntry- Record number that was read.
;               User set data is stored in:
;                   -> CTMRFlags         - Status flags.
;                   -> CTMRUserSetSecond - User set seconds data.
;                   -> CTMRUserSetMinute - User set minute data.
;                   -> CTMRUserSetHour   - User set hour data.
;                   -> CTMRNameX         - 9 char timer name, where x = 0 to 8
;
;               ( Destroyed: BAReg, HLReg )
;
; Module Change History:
;
;               03/05/2002  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrReadNextRecord()
{

    // Load the number of records in TMR database.
    AReg = CTMRNumberOfRecords;

    // Set HLReg where the last viewed entry was saved and increment it.
    // Check also if there is a wraparound.
    HLReg = &CTMRLastViewedEntry;
    *HLReg++;

    if( *HLReg >= AReg )
    {
        // Point to the first record.
        *HLReg = 0;
    }

    // Get the new record and display it.
    ctmrDefaultGetNewData();

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrReadNextRecord'"

        GLOBAL  ctmrReadNextRecord

ctmrReadNextRecord:                                 ; **SUBROUTINE ctmrReadNextRecord

                ; Load the number of records in TMR database.
                ld      A, [CTMRNumberOfRecords]
                
                ; Set HLReg where the last viewed entry was saved and increment it.
                ; Check also if there is a wraparound.
                ld      HL, #CTMRLastViewedEntry
                inc     [HL]

                cp      [HL], A
                jr      C, ctmrDefaultGetNewData
                
                ; Point to the first record.
                ld      [HL], #0
                
                ; Get the new record and display it.
                jr      ctmrDefaultGetNewData       ; **EXTERNAL JUMP


                                                    ; **END SUBROUTINE ctmrReadNextRecord


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrReadPrevRecord
;
; Description : Reads the previous DB record.
;               Resets the active resource.
;               Clears the entire screen.
;
; Assumptions : CORECurrentADDAddress - Absolute address of DB in external memory.
;               CTMRNumberOfRecords   - Number of records in DB.
;               CTMRLastViewedEntry   - Record number to be read.
;               CTMRActiveResource    - Resource to be reset.
;
; Input(s)    : None
;
; Output(s)   : AReg, CTMRLastViewedEntry- Record number that was read.
;               User set data is stored in:
;                   -> CTMRFlags         - Status flags.
;                   -> CTMRUserSetSecond - User set seconds data.
;                   -> CTMRUserSetMinute - User set minute data.
;                   -> CTMRUserSetHour   - User set hour data.
;                   -> CTMRNameX         - 9 char timer name, where x = 0 to 8
;
;               ( Destroyed: BAReg, HLReg )
;
; Module Change History:
;
;               03/05/2002  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `


void ctmrReadPrevRecord()
{
    // Set HLReg where the last viewed entry was saved and decrement it.
    // Check also if there is a wraparound.
    HLReg = &CTMRLastViewedEntry;
    *HLReg -= 1;

    if( bCarryFlag == TRUE )
    {
        // Point to the last record.
        AReg = CTMRNumberOfRecords;
        AReg--;
        *HLReg = AReg;
    }

ctmrDefaultGetNewData:
    //-------------------------------------------------------------------------
    // Reads a record from the timer database.
    // Destroys BAReg, HLReg.
    // Input:  HLReg - Absolute address of the DB file in external memory.
    //         AReg  - Record number to read.
    // Output: CTMRFlags         - Status flags.
    //         CTMRUserSetSecond - User set seconds data.
    //         CTMRUserSetMinute - User set minute data.
    //         CTMRUserSetHour   - User set hour data.
    //         CTMRNameX         - 9 char timer name, where x = 0 to 8
    //-------------------------------------------------------------------------
    AReg = *HLReg;

    push    (BAReg);

    HLReg = CORECurrentADDAddress;
    ctmrReadOneRecord();

    //-------------------------------------------------------------------------
    // Force the timer to be in reset state because when traversing to the 
    //   other entries, the state of timer may be stopped.
    //   Forcing it to be reset would display the new data in reset form.
    // Destroys HLReg.
    // Input: AReg - Resource index.
    //-------------------------------------------------------------------------
    AReg = CTMRActiveResource;
    KTMR_RESET_RESOURCE;

    //-------------------------------------------------------------------------
    // Clears the entire display.
    // Destroys BAReg, HLReg.
    //-------------------------------------------------------------------------
    LCD_CLR_DISPLAY;

    pop     (BAReg);
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrReadPrevRecord'"

        GLOBAL  ctmrReadPrevRecord

ctmrReadPrevRecord:                                 ; **SUBROUTINE ctmrReadPrevRecord

                ; Set HLReg where the last viewed entry was saved and decrement it.
                ; Check also if there is a wraparound.
                ld      HL, #CTMRLastViewedEntry
                sub     [HL], #1

                jr      NC, ctmrDefaultGetNewData
                    
                ; Point to the last record number.
                ld      A, [CTMRNumberOfRecords]
                dec     A
                ld      [HL], A

ctmrDefaultGetNewData:
                ;--------------------------------------------------------------
                ; Reads a record from the timer database.
                ; Destroys BAReg, HLReg.
                ; Input: HLReg - Absolute address of the DB file in external
                ;                  memory.
                ;        AReg  - Record number to read.
                ;--------------------------------------------------------------
                ld      A, [HL]

                push    BA

                ld      HL, [CORECurrentADDAddress]
                car     ctmrReadOneRecord

                ;--------------------------------------------------------------
                ; Force the timer to be in reset state because when traversing
                ;   to the other entries, the state of timer may be stopped.
                ;   Forcing it to be reset would display the new data in reset
                ;   form.
                ; Destroys HLReg.
                ; Input: AReg - Resource index.
                ;--------------------------------------------------------------
                ld      A, [CTMRActiveResource]
                KTMR_RESET_RESOURCE

                ;--------------------------------------------------------------
                ; Clears the entire display.
                ; Destroys BAReg, HLReg.
                ;--------------------------------------------------------------
                LCD_CLR_DISPLAY

                pop     BA

                ret

                                                    ; **END SUBROUTINE ctmrReadPrevRecord


        CALLS   SUBROUTINE, 'ctmrReadOneRecord'


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrReadOneRecord
;
; Description : Reads a record from the timer database.
;
; Assumptions : None
;
; Input(s)    : HLReg - Absolute address of the DB file in external memory.
;               AReg  - Record number to read.
;
; Output(s)   : CTMRNumberOfRecords - Number of records in DB.
;               User set data is stored in:
;                   -> CTMRFlags         - Status flags.
;                   -> CTMRUserSetSecond - User set seconds data.
;                   -> CTMRUserSetMinute - User set minute data.
;                   -> CTMRUserSetHour   - User set hour data.
;                   -> CTMRNameX         - 9 char timer name, where x = 0 to 8
;
;               ( Destroyed: BAReg, HLReg )
;
; Module Change History:
;
;               12/12/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrReadOneRecord()
{

    //-------------------------------------------------------------------------
    // Initialize first the variable needed to read from an external memory.
    // DBRecordNumber - Record number (16-bit).
    //-------------------------------------------------------------------------
    BReg = 0;

    DBRecordNumber = BAReg;

    //-------------------------------------------------------------------------
    // Open the database in external memory.
    // Input:  DBExternalMemoryAddress - Absolute address of the DB file in 
    //                                     external memory (16-bit)
    // Output: DBExternalBaseAddress   - Base address of DB use to convert 
    //                                     offsets to absolute address (16-bit)
    //         DBNumberOfRecords       - Number of records for the current 
    //                                     DB (16-bit)
    //         DBSizePerRecord         - Size per record of database (8-bit)
    //-------------------------------------------------------------------------
    DBExternalMemoryAddress = HLReg;
    DB_OPEN_FILE;

    // Discard the value of BReg since the max entry is only 99.
    BAReg = DBNumberOfRecords;
    CTMRNumberOfRecords = AReg;

    //-------------------------------------------------------------------------
    // Read from a specific record in external memory.
    // Input: DBInternalMemoryAddress - Destination address in internal memory 
    //                                    (16-bit)
    //        DBRecordNumber          - Record number (16-bit)
    //        DBLengthLo              - Number of bytes to read (8-bit)
    //-------------------------------------------------------------------------
    HLReg = &CTMRFlags;
    DBInternalMemoryAddress = HLReg;

    BReg = CTMRRECORDSIZE;
    DBLengthLo = BReg;
    
    DB_READ_RECORD_RANDOMFIX;

    //-------------------------------------------------------------------------
    // Close the file.  Turn off the EEPROM.
    //-------------------------------------------------------------------------
    DB_CLOSE_FILE;

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrReadOneRecord'"

        GLOBAL  ctmrReadOneRecord

ctmrReadOneRecord:                                  ; **SUBROUTINE ctmrReadOneRecord

                ;--------------------------------------------------------------
                ; Initialize first the variable needed to read from an external 
                ;   memory.
                ; DBRecordNumber - Record number (16-bit).
                ;--------------------------------------------------------------
                ld      B, #0
                ld      [DBRecordNumber], BA

                ;--------------------------------------------------------------
                ; Open the database in external memory.
                ; Input:  DBExternalMemoryAddress - Absolute address of the DB 
                ;                                     file in external memory 
                ;                                     (16-bit)
                ; Output: DBExternalBaseAddress   - Base address of DB use to 
                ;                                     convert offsets to absolute 
                ;                                     address (16-bit)
                ;         DBNumberOfRecords       - Number of records for the 
                ;                                     current DB (16-bit)
                ;         DBSizePerRecord         - Size per record of database 
                ;                                     (8-bit)
                ;--------------------------------------------------------------
                ld      [DBExternalMemoryAddress], HL
                DB_OPEN_FILE

                ; Discard the value of BReg since the max entry is only 99.
                ld      BA, [DBNumberOfRecords]
                ld      [CTMRNumberOfRecords], A

                ;--------------------------------------------------------------
                ; Read from a specific record in external memory.
                ; Input: DBInternalMemoryAddress - Destination address in internal 
                ;                                    memory (16-bit)
                ;        DBRecordNumber          - Record number (16-bit)
                ;        DBLengthLo              - Number of bytes to read (8-bit)
                ;--------------------------------------------------------------
                ld      HL, #CTMRFlags
                ld      [DBInternalMemoryAddress], HL
                
                ld      B, #CTMRRECORDSIZE
                ld      [DBLengthLo], B
    
                DB_READ_RECORD_RANDOMFIX

                ;--------------------------------------------------------------
                ; Close the file.  Turn off the EEPROM.
                ;--------------------------------------------------------------
                DB_CLOSE_FILE

                ret

                                                    ; **END SUBROUTINE ctmrReadOneRecord


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrWriteStatusFlagToEEPROM
;
; Description : Writes the status flag to EEPROM
;
; Assumptions : CTMRLastViewedEntry - Record number to write to.
;               CORECurrentMode     - App index of the DB to write to.
;
; Input(s)    : HLReg - Absolute address of DB file in external memory.
;
; Output(s)   : None
;
;               ( Destroyed: HLReg, BAReg )
;
; Module Change History:
;
;               12/17/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrWriteStatusFlagToEEPROM'"

        GLOBAL  ctmrWriteStatusFlagToEEPROM

ctmrWriteStatusFlagToEEPROM:                        ; **SUBROUTINE ctmrWriteStatusFlagToEEPROM
                
                ; Number of bytes to write.
                ld      B, #1
                
                jr      ctmrWriteDataToEEPROMCommon


                                                    ; **END SUBROUTINE ctmrWriteStatusFlagToEEPROM

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrWriteStatusFlagAndHMSDataToEEPROM
;
; Description : Writes the status flag and user set data to EEPROM.
;
; Assumptions : CTMRLastViewedEntry - Record number to write to.
;               CORECurrentMode     - App index of the DB to write to.
;
; Input(s)    : HLReg - Absolute address of DB file in external memory.
;
; Output(s)   : None
;
;               ( Destroyed: HLReg, BAReg )
;
; Module Change History:
;
;               12/17/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrWriteStatusFlagAndHMSDataToEEPROM()
{

    BReg = 4;

ctmrWriteDataToEEPROMCommon:

    //-------------------------------------------------------------------------
    // Initialize variables for writing to an EEPROM.
    // DBRecordNumber - Record number (16-bit)
    // DBLengthLo     - Number of bytes to write (8-bit)
    //-------------------------------------------------------------------------
    DBLengthLo = BReg;

    AReg = CTMRLastViewedEntry;
    BReg = 0;
    DBRecordNumber = BAReg;


    //-------------------------------------------------------------------------
    // Opens the database file in external memory.
    // Input:  DBExternalMemoryAddress - Absolute address of the DB file in
    //                                     external memory (16-bit)
    // Output: DBExternalBaseAddress   - Base address of DB use to convert 
    //                                     offsets to absolute address (16-bit)
    //         DBNumberOfRecords       - Number of records for the current
    //                                     DB (16-bit)
    //         DBSizePerRecord         - Size per record of DB (8-bit)
    //-------------------------------------------------------------------------
    DBExternalMemoryAddress = HLReg;
    DB_OPEN_FILE;

    //-------------------------------------------------------------------------
    // Writes to a specific record in a DB.
    // Input: DBInternalMemoryAddress - Source base address in internal memory 
    //                                    (16-bit)
    //        DBRecordNumber          - Record number (16-bit)
    //        DBLengthLo              - Number of bytes to write (8-bit)
    //-------------------------------------------------------------------------
    HLReg = &CTMRFlags;
    DBInternalMemoryAddress = HLReg;

    DB_WRITE_RECORD_RANDOMFIX;


    //-------------------------------------------------------------------------
    // Close the opened file. Turn off the EEPROM.
    //-------------------------------------------------------------------------
    DB_CLOSE_FILE;

    //-------------------------------------------------------------------------
    // Indicates to an external PC application that an entry in the current 
    //   database was modified by the user since the last PC synchronization.
    //-------------------------------------------------------------------------
    AReg = CORECurrentMode;
    CORE_DATABASE_MODIFIED_BY_USER;

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrWriteStatusFlagAndHMSDataToEEPROM'"

        GLOBAL  ctmrWriteStatusFlagAndHMSDataToEEPROM

ctmrWriteStatusFlagAndHMSDataToEEPROM:              ; **SUBROUTINE ctmrWriteStatusFlagAndHMSDataToEEPROM

                ld      B, #4

ctmrWriteDataToEEPROMCommon:

                ;--------------------------------------------------------------
                ; Initialize variables for writing to an EEPROM.
                ; DBRecordNumber - Record number (16-bit)
                ; DBLengthLo     - Number of bytes to write (8-bit)
                ;--------------------------------------------------------------
                ld      [DBLengthLo], B
                                
                ld      A, [CTMRLastViewedEntry]
                ld      B, #0
                ld      [DBRecordNumber], BA


                ;--------------------------------------------------------------
                ; Opens the database file in external memory.
                ; Input:  DBExternalMemoryAddress - Absolute address of the DB file in
                ;                                     external memory (16-bit)
                ; Output: DBExternalBaseAddress   - Base address of DB use to convert 
                ;                                     offsets to absolute address (16-bit)
                ;         DBNumberOfRecords       - Number of records for the current
                ;                                     DB (16-bit)
                ;         DBSizePerRecord         - Size per record of DB (8-bit)
                ;--------------------------------------------------------------
                ld      [DBExternalMemoryAddress], HL
                DB_OPEN_FILE

                ;--------------------------------------------------------------
                ; Writes to a specific record in a DB.
                ; Input: DBInternalMemoryAddress - Source base address in internal 
                ;                                    memory (16-bit)
                ;        DBRecordNumber          - Record number (16-bit)
                ;        DBLengthLo              - Number of bytes to write (8-bit)
                ;--------------------------------------------------------------
                ld      HL, #CTMRFlags
                ld      [DBInternalMemoryAddress], HL
                DB_WRITE_RECORD_RANDOMFIX

                ;--------------------------------------------------------------
                ; Close the opened file. Turn off the EEPROM.
                ;--------------------------------------------------------------
                DB_CLOSE_FILE

                ;--------------------------------------------------------------
                ; Indicates to an external PC application that an entry in the 
                ;   current database was modified by the user since the last PC 
                ;   synchronization.
                ;--------------------------------------------------------------
                ld      A, [CORECurrentMode]
                CORE_DATABASE_MODIFIED_BY_USER

                ret


                                                    ; **END SUBROUTINE ctmrWriteStatusFlagAndHMSDataToEEPROM


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrUpdateTurnOffResourceFlags
;
; Description : Turns off the LCD resource flags.
;
; Assumptions : None
;               
; Input(s)    : None
;
; Output(s)   : None
;
;               ( Destroyed: HLReg, IXReg )
;
; Module Change History:
;
;               04/16/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrUpdateTurnOffResourceFlags'"

        GLOBAL  ctmrUpdateTurnOffResourceFlags


ctmrUpdateTurnOffResourceFlags:                     ; **SUBROUTINE ctmrUpdateTurnOffResourceFlags

                push    BA

                ld      A, [CORECurrentMode]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  TMR_RSRC_FLAG, FLAG_OFF

                ld      A, [CORECurrentMode]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  CHR_RSRC_FLAG, FLAG_OFF

                ld      A, [CORECurrentMode]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  ARROW_RSRC_FLAG, FLAG_OFF

                ld      A, [CORECurrentMode]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  TAIL_RSRC_FLAG, FLAG_OFF

                pop     BA

                ret

                                                    ; **END SUBROUTINE ctmrUpdateTurnOffResourceFlags




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : ctmrUpdateResourceFlags
;
; Description : Updates the LCD resource flags.
;
; Assumptions : None
;               
; Input(s)    : BReg - Contains the flag values that contains the action at end.
;
; Output(s)   : None
;
;               ( Destroyed: HLReg, BAReg, IXReg )
;
; Module Change History:
;
;               04/16/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void ctmrUpdateResourceFlags()
{
    push    (BAReg);

    // Turns off all timer related flags.
    ctmrUpdateTurnOffResourceFlags();

    AReg = CORECurrentMode;
    LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  TMR_RSRC_FLAG, FLAG_ON;

    pop     (BAReg);

    // Clear the other value leaving only the action at end bits.
    BReg &= CTMRACTIONATENDMASKBITS;

    // Check if its repeat at end.
    if( BReg == CTMRREPEATATEND )
    {
        AReg = CORECurrentMode;
        LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  TAIL_RSRC_FLAG, FLAG_ON;
    }

    // Check if its chrono at end.
    elseif( BReg == CTMRCHRONOATEND )
    {
        AReg = CORECurrentMode;
        LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  CHR_RSRC_FLAG, FLAG_ON;

        AReg = CORECurrentMode;
        LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  ARROW_RSRC_FLAG, FLAG_ON;
    }
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'ctmrUpdateResourceFlags'"

        GLOBAL  ctmrUpdateResourceFlags

ctmrUpdateResourceFlags:                            ; **SUBROUTINE ctmrUpdateResourceFlags

                push    BA
                
                ; Turns off all timer related flags.
                car     ctmrUpdateTurnOffResourceFlags
                
                ld      A, [CORECurrentMode]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  TMR_RSRC_FLAG, FLAG_ON

                pop     BA
                
                ; Clear the other value leaving only the action at end bits.
                and     B, #CTMRACTIONATENDMASKBITS

                ; Check if its repeat at end.
                cp      B, #CTMRREPEATATEND
                jr      Z, ctmrUpdateResourceFlagRepeatAtEnd

                ; Check if its chrono at end.
                cp      B, #CTMRCHRONOATEND
                jr      NZ, ctmrUpdateResourceFlagExit

                ld      A, [CORECurrentMode]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  CHR_RSRC_FLAG, FLAG_ON

                ld      A, [CORECurrentMode]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  ARROW_RSRC_FLAG, FLAG_ON

                ret

ctmrUpdateResourceFlagRepeatAtEnd:

                ld      A, [CORECurrentMode]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  TAIL_RSRC_FLAG, FLAG_ON

ctmrUpdateResourceFlagExit:

                ret





                                                    ; **END SUBROUTINE ctmrUpdateResourceFlags



;**END FILE ctmrutil.asm


        END











