;**FILE tmrdisp.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2000 - 2002 TIMEX Corporation.
; 
; This document is the property of, and contains information proprietary to
; TIMEX Corporation.  It is to be returned upon request.  Reproduction by any
; means, disclosure, or use of its contents for other than the expressed
; purpose for which it is loaned, without the written consent of TIMEX
; Corporation, is strictly prohibited.
;
; File Name    : tmrdisp.asm
;
; Module Names : tmrChkAndDispRepeatNumber - Check if repeat at end is the action 
;                                              and repeat number needs to be updated 
;                                              before the display would commence.
;
;                tmrDisplayEditField       - Displays the data of the field to be edited.
;
;
; Purpose      : Countdown Timer display routines.
;
; File Change History:
;
;               12/12/2001  - Created for M851 development.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

$CASE ON
$LIST OFF

        INCLUDE '..\..\CORE\H\MODEL.H'
        INCLUDE '..\..\HW\H\HW88349.H'
        INCLUDE '..\..\HW\H\HWI2CEE.H'
        INCLUDE '..\..\HW\H\HWBOND.H'
        INCLUDE '..\..\HW\H\HWKBD.H'
        INCLUDE '..\..\HW\H\HWMACRO.H'
        INCLUDE '..\..\HW\H\HWMAIN.H'
        INCLUDE '..\..\HW\H\HWRESET.H'
        INCLUDE '..\..\HW\H\HWVARS.H'
        INCLUDE '..\..\HW\H\HWSERMAC.H'
        INCLUDE '..\..\CORE\H\CORE.H'
        INCLUDE '..\..\CORE\H\COREMAC.H'
        INCLUDE '..\..\CORE\H\CORESVMC.H'
        INCLUDE '..\..\COMM\H\COMM.H'
        INCLUDE '..\..\LCD\H\LCD.H'
        INCLUDE '..\..\LCD\H\LCDMACRO.H'
        INCLUDE '..\..\LCD\H\LCDRAM.H'
        INCLUDE '..\..\DATABASE\H\DB.H'
        INCLUDE '..\..\DATABASE\H\DBMACRO.H'
        INCLUDE '..\..\AUDIO\H\AUD.H'
        INCLUDE '..\..\AUDIO\H\AUDMAC.H'
        INCLUDE '..\..\UTIL\H\UTLMATH.H'
        INCLUDE '..\..\UTIL\H\UTLMTHMC.H'
        INCLUDE '..\..\UTIL\H\UTLEDT.H'
        INCLUDE '..\..\UTIL\H\UTLEDTMC.H'
        INCLUDE '..\..\UTIL\H\UTLDSP.H'
        INCLUDE '..\..\UTIL\H\UTLDSPMC.H'
        INCLUDE '..\..\UTIL\H\UTLACCMC.H'
        INCLUDE '..\..\UTIL\H\UTLMSC.H'
        INCLUDE '..\..\UTIL\H\UTLMSCMC.H'
        INCLUDE '..\..\UTIL\H\UTLVARS.H'
        INCLUDE '..\..\KRES\H\KRES.H'
        INCLUDE '..\..\KRES\H\KRESMAC.H'
        INCLUDE '..\..\KRES\H\KSTP.H'
        INCLUDE '..\..\KRES\H\KSTPMAC.H'
        INCLUDE '..\..\KRES\H\KSYN.H'
        INCLUDE '..\..\KRES\H\KSYNMAC.H'
        INCLUDE '..\..\KRES\H\KTMR.H'
        INCLUDE '..\..\KRES\H\KTMRMAC.H'
        INCLUDE '..\..\KRES\H\KTOD.H'
        INCLUDE '..\..\KRES\H\KTODMAC.H'
        INCLUDE '..\..\KRES\H\KTZC.H'
        INCLUDE '..\..\KRES\H\KTZCMAC.H'
        INCLUDE '..\..\KRES\H\KBCK.H'
        INCLUDE '..\..\KRES\H\KBCKMAC.H'
        INCLUDE '..\..\USAGE\H\USG.H'
        INCLUDE '..\..\USAGE\H\USGVARS.H'
        INCLUDE '..\..\USAGE\H\USGMAC.H'
        INCLUDE '..\..\CORE\H\SECTION.H'

        INCLUDE '..\..\TOD\H\TOD.H'
        INCLUDE '..\..\TOD\H\TODVARS.H'
        
        INCLUDE '..\..\CTMR\H\CTMR.H'
        INCLUDE '..\..\CTMR\H\CTMRVARS.H'

        INCLUDE '..\..\TMR\H\TMRVARS.H'

$LIST ON

        SECT    ".text_tmr"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : tmrChkAndDispRepeatNumber
;
; Description : Check if repeat at end is the action and repeat number needs
;                 to be updated before the display would commence.
;
; Assumptions : None
;               
; Input(s)    : None
;
; Output(s)   : None
;
;               ( Destroyed: HLReg, IXReg )
;
; Module Change History:
;
;               11/14/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT `

void tmrChkAndDispRepeatNumber()
{
    push    (BAReg);

    // Mask the unwanted bits leaving only the action at end.
    AReg = CTMRFlags;
    AReg &= CTMRACTIONATENDMASKBITS;

    // Check if repeat at end is the action at end.  If not, then don't display repeat number.
    if( AReg == CTMRREPEATATEND )
    {
        // Set HLReg to timer flag offset.    
        HLReg = CORECurrentASDAddress;
        HLReg += TMRFLAGS2OFFSET;

        // Check if repeat number display needs to be updated.
        if( *HLReg.bTMRUpdRepeatNumDisplay == TRUE )
        {
            // Clear the bit to indicate that the repeat number display is already updated.
            *HLReg &= ~bTMRUpdRepeatNumDisplay;

            //-----------------------------------------------------------------
            // Get the running repeat counter of the resource specified by AReg.
            // Destroys HLReg.
            // Input:  AReg - Resource index
            // Output: BReg - Running repeat counter (HEX value)
            //-----------------------------------------------------------------
            AReg = CTMRActiveResource;  
            KTMR_GET_RUNNING_REPEAT_COUNTER;

            //-----------------------------------------------------------------
            // Displays the repeat number at upper DM.
            // Input: LReg - Repeat counter (HEX value)
            //-----------------------------------------------------------------
            LReg = BReg;
            chrDisplayLapNumberInSmallDM();

        }
    }

    pop     (BAReg);

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'tmrChkAndDispRepeatNumber'"

        GLOBAL  tmrChkAndDispRepeatNumber

tmrChkAndDispRepeatNumber:                          ; **SUBROUTINE tmrChkAndDispRepeatNumber

                push    BA
                
                ; Mask the unwanted bits leaving only the action at end.
                ld      A, [CTMRFlags]
                and     A, #CTMRACTIONATENDMASKBITS

                ; Check if repeat at end is the action at end.  If not, then don't 
                ;   display repeat number.
                cp      A, #CTMRREPEATATEND
                jr      NZ, tmrChkAndDispRepNumberExit
                
                ; Set HLReg to timer flag offset.  Check if repeat number display needs to be updated.
                ld      HL, [CORECurrentASDAddress]
                add     HL, #TMRFLAGS2OFFSET
                bit     [HL], #bTMRUpdRepeatNumDisplay
                jr      Z, tmrChkAndDispRepNumberExit
                 
                ; Clear the bit to indicate that the repeat number display is already updated.
                and     [HL], #@LOW(~bTMRUpdRepeatNumDisplay)

                ;--------------------------------------------------------------
                ; Get the running repeat counter of the resource specified by 
                ;   AReg.
                ; Destroys HLReg.
                ; Input:  AReg - Resource index
                ; Output: BReg - Running repeat counter (HEX value)
                ;--------------------------------------------------------------
                ld      A, [CTMRActiveResource]
                KTMR_GET_RUNNING_REPEAT_COUNTER

                ;--------------------------------------------------------------
                ; Displays the repeat number at upper DM.
                ; Input: LReg - Repeat counter (HEX value)
                ;--------------------------------------------------------------
                ld      L, B
                car     chrDisplayLapNumberInSmallDM

tmrChkAndDispRepNumberExit:
                
                pop     BA

                ret


                                                    ; **END SUBROUTINE tmrChkAndDispRepeatNumber


        CALLS   SUBROUTINE, 'chrDisplayLapNumberInSmallDM'


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : tmrDisplayEditField
;
; Description : Displays the data of the field to be edited.
;
; Assumptions : None
;               
; Input(s)    : None
;
; Output(s)   : None
;
;               ( Destroyed: IYReg, BAReg )
;
; Module Change History:
;
;               12/17/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void tmrDisplayEditField()
{
    switch( CTMRCurrentSetPosition )
    {
        case    CTMRSETMINHOUR:
            //=================================================================
            //
            //                      HOUR AND MINUTE
            //
            //=================================================================

            //-----------------------------------------------------------------
            // Display user set hour and minute data.
            // Destroys BAReg, HLReg, IXReg.
            //-----------------------------------------------------------------         
            ctmrDisplayUserSetHourMinuteAtBDM();
                                                                                                    
            break;                                                                 

        case    CTMRSETSECOND:
            //=================================================================
            //
            //                          SECOND
            //
            //=================================================================

            //-----------------------------------------------------------------         
            // Display user set seconds data.
            // Destroys BAReg, HLReg, IXReg.
            //-----------------------------------------------------------------         
            ctmrDisplayUserSetSecondAtBDM();

            break;

        case    CTMRSETACTIONATEND:
            //=================================================================
            //
            //                       ACTION AT END
            //
            //=================================================================

            // Display action at end flags.
            BReg = CTMRFlags;
            ctmrDisplayActionAtEndFlags();

tmrDisplayActionAtEndMsg:

            // Display action at end message.
            BReg = CTMRFlags;
            ctmrDisplayActionAtEndMsg();


            break;

    } 

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'tmrDisplayEditField'"

        GLOBAL  tmrDisplayEditField
        GLOBAL  tmrDisplayActionAtEndMsg

tmrDisplayEditField:                                ; **SUBROUTINE tmrDisplayEditField

                ld      A, [CTMRCurrentSetPosition]

                cp      A, #CTMRSETMINHOUR
                jr      Z, ctmrDisplayUserSetHourMinuteAtBDM    ; **EXTERNAL JUMP

                cp      A, #CTMRSETSECOND
                jr      Z, ctmrDisplayUserSetSecondAtBDM        ; **EXTERNAL JUMP

                ;==============================================================
                ;
                ;                       ACTION AT END
                ;
                ;==============================================================
                
                ; Display action at end flags.
                ld      B, [CTMRFlags]
                car     ctmrDisplayActionAtEndFlags

tmrDisplayActionAtEndMsg:
                
                ; Display action at end message.
                ld      B, [CTMRFlags]
                jr      ctmrDisplayActionAtEndMsg   ; **EXTERNAL JUMP


                                                    ; **END SUBROUTINE tmrDisplayEditField



        CALLS   SUBROUTINE, 'ctmrDisplayActionAtEndFlags'



;**END FILE tmrdisp.asm


        END
