; **FILE tmrstmgr.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2000 - 2002 TIMEX Corporation.
; 
; This document is the property of, and contains information proprietary to
; TIMEX Corporation.  It is to be returned upon request.  Reproduction by any
; means, disclosure, or use of its contents for other than the expressed
; purpose for which it is loaned, without the written consent of TIMEX
; Corporation, is strictly prohibited.
;
; File Name    : tmrstmgr.asm
;
; Module Names : tmrStateManager
;                   
; Purpose      : Dispatches and executes the current state handler for event processing.
;
; File Change History:
;
;                08/03/2000  - Created tmrstmgr.asm for m841
;
;   M851 Development:
;
;                12/17/2001  - Modified for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

$CASE ON
$LIST OFF

        INCLUDE '..\..\CORE\H\MODEL.H'
        INCLUDE '..\..\HW\H\HW88349.H'
        INCLUDE '..\..\HW\H\HWI2CEE.H'
        INCLUDE '..\..\HW\H\HWBOND.H'
        INCLUDE '..\..\HW\H\HWKBD.H'
        INCLUDE '..\..\HW\H\HWMACRO.H'
        INCLUDE '..\..\HW\H\HWMAIN.H'
        INCLUDE '..\..\HW\H\HWRESET.H'
        INCLUDE '..\..\HW\H\HWVARS.H'
        INCLUDE '..\..\HW\H\HWSERMAC.H'
        INCLUDE '..\..\CORE\H\CORE.H'
        INCLUDE '..\..\CORE\H\COREMAC.H'
        INCLUDE '..\..\CORE\H\CORESVMC.H'
        INCLUDE '..\..\COMM\H\COMM.H'
        INCLUDE '..\..\LCD\H\LCD.H'
        INCLUDE '..\..\LCD\H\LCDMACRO.H'
        INCLUDE '..\..\LCD\H\LCDRAM.H'
        INCLUDE '..\..\DATABASE\H\DB.H'
        INCLUDE '..\..\DATABASE\H\DBMACRO.H'
        INCLUDE '..\..\AUDIO\H\AUD.H'
        INCLUDE '..\..\AUDIO\H\AUDMAC.H'
        INCLUDE '..\..\UTIL\H\UTLMATH.H'
        INCLUDE '..\..\UTIL\H\UTLMTHMC.H'
        INCLUDE '..\..\UTIL\H\UTLEDT.H'
        INCLUDE '..\..\UTIL\H\UTLEDTMC.H'
        INCLUDE '..\..\UTIL\H\UTLDSP.H'
        INCLUDE '..\..\UTIL\H\UTLDSPMC.H'
        INCLUDE '..\..\UTIL\H\UTLACCMC.H'
        INCLUDE '..\..\UTIL\H\UTLMSC.H'
        INCLUDE '..\..\UTIL\H\UTLMSCMC.H'
        INCLUDE '..\..\UTIL\H\UTLVARS.H'
        INCLUDE '..\..\KRES\H\KRES.H'
        INCLUDE '..\..\KRES\H\KRESMAC.H'
        INCLUDE '..\..\KRES\H\KSTP.H'
        INCLUDE '..\..\KRES\H\KSTPMAC.H'
        INCLUDE '..\..\KRES\H\KSYN.H'
        INCLUDE '..\..\KRES\H\KSYNMAC.H'
        INCLUDE '..\..\KRES\H\KTMR.H'
        INCLUDE '..\..\KRES\H\KTMRMAC.H'
        INCLUDE '..\..\KRES\H\KTOD.H'
        INCLUDE '..\..\KRES\H\KTODMAC.H'
        INCLUDE '..\..\KRES\H\KTZC.H'
        INCLUDE '..\..\KRES\H\KTZCMAC.H'
        INCLUDE '..\..\KRES\H\KBCK.H'
        INCLUDE '..\..\KRES\H\KBCKMAC.H'
        INCLUDE '..\..\USAGE\H\USG.H'
        INCLUDE '..\..\USAGE\H\USGVARS.H'
        INCLUDE '..\..\USAGE\H\USGMAC.H'
        INCLUDE '..\..\CORE\H\SECTION.H'

        INCLUDE '..\..\TOD\H\TOD.H'
        INCLUDE '..\..\TOD\H\TODVARS.H'
        
        INCLUDE '..\..\CTMR\H\CTMR.H'
        INCLUDE '..\..\CTMR\H\CTMRVARS.H'

        INCLUDE '..\..\TMR\H\TMRVARS.H'
        
$LIST ON


        SECT    ".text_tmr"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : tmrStateManager
;
; Description : TMR Application State Manager
;
; Assumptions : None
;
; Input(s)    : CORECurrentState    - state to process the event
;
; Output(s)   : None
;
; Module Change History:
;
;               08/03/2000  - Created module for M841     
;
;   M851 Development:
;
;               12/17/2001  - Modified for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void tmrStateManager()
{

    switch( CORECurrentState )
    {

        case    CTMRBANNERSTATE:
            //*****************************************************************
            //
            //                      BANNER STATE
            //
            //*****************************************************************

            coreCommonBannerStateHandler();

            break;


        case    CTMRDEFAULTSTATE:
            //*****************************************************************
            //
            //                      DEFAULT STATE
            //
            //*****************************************************************

            tmrDefaultStateManager();

            break;


        case    CTMRSETBANNERSTATE:
            //*****************************************************************
            //
            //                      SET BANNER STATE
            //
            //*****************************************************************

            ctmrSetBannerStateManager();

            break;


        case    CTMRSETSTATE:
            //*****************************************************************
            //
            //                         SET STATE
            //
            //*****************************************************************

            tmrSetStateManager();

            break;


        case    CTMRPOPUPSTATE:
            //*****************************************************************
            //
            //                          POPUP STATE
            //
            //*****************************************************************

            tmrPopupStateManager();

            break;


        case    CTMRUNUSEDPOPUPSTATE:
            //*****************************************************************
            //
            //                      UNUSED POPUP STATE
            //
            //*****************************************************************

            utlYouRockStateManager();

            break;

    }

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF   SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'tmrStateManager'"

        GLOBAL  tmrStateManager


tmrStateManager:                                    ;**SUBROUTINE tmrStateManager

                ;--------------------------------------------------------------
                ; This routine will work only for states that are in sequence.
                ;--------------------------------------------------------------

                ld      HL, #tmrTableStateManagerAddress

                ; This will jump to the address pointed by the index in AReg.
                ; The address to jump is found in the table specified in HLReg.                
                ; After a desire task has been executed, it will return back to
                ;   the one that called this routine.
                jr      utlProcessStateManagerTable ; **EXTERNAL JUMP

tmrTableStateManagerAddress:

                dw      coreCommonBannerStateHandler
                dw      tmrDefaultStateManager
                dw      ctmrSetBannerStateManager
                dw      tmrSetStateManager
                dw      tmrPopupStateManager
                dw      utlYouRockStateManager



                                                    ;**END SUBROUTINE tmrStateManager

        CALLS   SUBROUTINE, 'coreCommonBannerStateHandler'
        CALLS   SUBROUTINE, 'tmrDefaultStateManager'
        CALLS   SUBROUTINE, 'ctmrSetBannerStateManager'
        CALLS   SUBROUTINE, 'tmrSetStateManager'
        CALLS   SUBROUTINE, 'tmrPopupStateManager'
        CALLS   SUBROUTINE, 'utlYouRockStateManager'




;**END FILE tmrstmgr.asm


        END
        