; **FILE tmrBckHd.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2000 - 2002 TIMEX Corporation.
; 
; This document is the property of, and contains information proprietary to
; TIMEX Corporation.  It is to be returned upon request.  Reproduction by any
; means, disclosure, or use of its contents for other than the expressed
; purpose for which it is loaned, without the written consent of TIMEX
; Corporation, is strictly prohibited.
;
; File Name    : tmrBckHd.asm
;
; Module Names : tmrBackgroundHandler       - Handles application initialization 
;                                               and refresh resource handlers.
;
;                tmrInitialDataTable        - Table for the initial values of timer 
;                                               data during POR.
;               
;                tmrDBHeaderInitialValue    - Table for the initial values of DB 
;                                               header during POR.
;               
;                tmrInitialTimerDataAndName - Initial timer data and name.
;
;                
; Purpose      : Handles the following application specific functions:
;                   - application initialization
;                   - resource refresh
;
; File Change History:
;
;               12/18/2001  - Created for M851.
;             
;               06/24/2002  - ECR# 851-20020621-368
;               06/26/2002  - ECR# 851-20020625-376
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

$CASE ON
$LIST OFF

        INCLUDE '..\..\CORE\H\MODEL.H'
        INCLUDE '..\..\HW\H\HW88349.H'
        INCLUDE '..\..\HW\H\HWI2CEE.H'
        INCLUDE '..\..\HW\H\HWBOND.H'
        INCLUDE '..\..\HW\H\HWKBD.H'
        INCLUDE '..\..\HW\H\HWMACRO.H'
        INCLUDE '..\..\HW\H\HWMAIN.H'
        INCLUDE '..\..\HW\H\HWRESET.H'
        INCLUDE '..\..\HW\H\HWVARS.H'
        INCLUDE '..\..\HW\H\HWSERMAC.H'
        INCLUDE '..\..\CORE\H\CORE.H'
        INCLUDE '..\..\CORE\H\COREMAC.H'
        INCLUDE '..\..\CORE\H\CORESVMC.H'
        INCLUDE '..\..\COMM\H\COMM.H'
        INCLUDE '..\..\LCD\H\LCD.H'
        INCLUDE '..\..\LCD\H\LCDMACRO.H'
        INCLUDE '..\..\LCD\H\LCDRAM.H'
        INCLUDE '..\..\DATABASE\H\DB.H'
        INCLUDE '..\..\DATABASE\H\DBMACRO.H'
        INCLUDE '..\..\AUDIO\H\AUD.H'
        INCLUDE '..\..\AUDIO\H\AUDMAC.H'
        INCLUDE '..\..\UTIL\H\UTLMATH.H'
        INCLUDE '..\..\UTIL\H\UTLMTHMC.H'
        INCLUDE '..\..\UTIL\H\UTLEDT.H'
        INCLUDE '..\..\UTIL\H\UTLEDTMC.H'
        INCLUDE '..\..\UTIL\H\UTLDSP.H'
        INCLUDE '..\..\UTIL\H\UTLDSPMC.H'
        INCLUDE '..\..\UTIL\H\UTLACCMC.H'
        INCLUDE '..\..\UTIL\H\UTLMSC.H'
        INCLUDE '..\..\UTIL\H\UTLMSCMC.H'
        INCLUDE '..\..\UTIL\H\UTLVARS.H'
        INCLUDE '..\..\KRES\H\KRES.H'
        INCLUDE '..\..\KRES\H\KRESMAC.H'
        INCLUDE '..\..\KRES\H\KSTP.H'
        INCLUDE '..\..\KRES\H\KSTPMAC.H'
        INCLUDE '..\..\KRES\H\KSYN.H'
        INCLUDE '..\..\KRES\H\KSYNMAC.H'
        INCLUDE '..\..\KRES\H\KTMR.H'
        INCLUDE '..\..\KRES\H\KTMRMAC.H'
        INCLUDE '..\..\KRES\H\KTOD.H'
        INCLUDE '..\..\KRES\H\KTODMAC.H'
        INCLUDE '..\..\KRES\H\KTZC.H'
        INCLUDE '..\..\KRES\H\KTZCMAC.H'
        INCLUDE '..\..\KRES\H\KBCK.H'
        INCLUDE '..\..\KRES\H\KBCKMAC.H'
        INCLUDE '..\..\USAGE\H\USG.H'
        INCLUDE '..\..\USAGE\H\USGVARS.H'
        INCLUDE '..\..\USAGE\H\USGMAC.H'
        INCLUDE '..\..\CORE\H\SECTION.H'

        INCLUDE '..\..\TOD\H\TOD.H'
        INCLUDE '..\..\TOD\H\TODVARS.H'
        
        INCLUDE '..\..\CTMR\H\CTMR.H'
        INCLUDE '..\..\CTMR\H\CTMRVARS.H'

        INCLUDE '..\..\TMR\H\TMRVARS.H'

$LIST ON


        SECT    ".text_tmr"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : tmrBackgroundHandler
;
; Description : Handles application initialization and refresh resource handlers.
;
; Assumptions : COREInitializationASDAddress is already set by kernel
;               COREInitializationADDAddress is already set by kernel
;
; Input(s)    : COREBackgroundEvent - system event to be processed
;
; Output(s)   : None
;
;               ( Destroyed: ALL registers )
;
; Module Change History:
;
;               12/18/2001  - Created for M851.
;
;               06/24/2002  - ECR# 851-20020621-368
;               06/26/2002  - ECR# 851-20020625-376
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void tmrBackgroundHandler()
{

    // Initialize IYReg to point to the ASD address.
    IYReg = COREBackgroundASDAddress;

    switch( COREBackgroundEvent )
    {

        case    COREEVENT_PORINIT:
            //*****************************************************************
            //
            //              POWER ON RESET INITIALIZATION
            //
            //*****************************************************************

            push    (IYReg);

            //=================================================================
            // Initialize DB generic header and app specific header.
            //=================================================================

            //-----------------------------------------------------------------
            // Writes the DB header information.  Turning on and off of the 
            //   EEPROM is also included.
            // Destroyes nothing.
            // Input: DBExternalMemoryAddress - Absolute address of DB in 
            //                                    external memory (16-bit)
            //        DBInternalMemoryAddress - Source address in internal
            //                                    memory (16-bit)
            //        DBLengthLo - Number of bytes to write (8-bit) 
            //-----------------------------------------------------------------
            BReg = (TMRADDGENERICHEADERSIZE + TMRAPPSPECIFICHEADERSIZE);
            DBLengthLo = BReg;
            IXReg = COREInitializationADDAddress;
            DBExternalMemoryAddress = IXReg;
            IXReg = &tmrDBHeaderInitialValue;
            DBInternalMemoryAddress = IXReg;
            DB_WRITE_DATABASE_HEADER;


            //=================================================================
            // Initialize 10 timer entries.
            //=================================================================
            
            //-----------------------------------------------------------------
            // Initialize variable needed to write to an EEPROM.
            // DBInternalMemoryAddress - Source base address in internal 
            //                             memory (16-bit)
            // DBLengthLo              - Number of bytes to write (8-bit) 
            // DBRecordNumber          - Record number to write (16-bit)
            //                 
            // NOTE: DBRecordNumber can be separated into DBRecordNumberLo
            //         and DBRecordNumberHi.  Since the initial number of
            //         records is 10, DBRecordNumberHi is always 0.
            //-----------------------------------------------------------------
            BReg = 0;
            DBRecordNumberHi = BReg;
            HLReg = &CTMRFlags;
            DBInternalMemoryAddress = HLReg;
            BReg = TMRRECORDSIZE;
            DBLengthLo = BReg;

            //-----------------------------------------------------------------
            // Copies the initial status flag, HMS data and name.  The HM data
            //   is insignificant.
            // HLReg and BReg are already loaded with the correct data when
            //   initializing the variables to write to EEPROM.
            // Destroys BReg, HLReg, IYReg.
            // Input: BReg  - Number of bytes to copy.
            //        IYReg - Lowest address of the source structure.
            //        HLReg - Lowest address of the destination structure.
            //-----------------------------------------------------------------
            IYReg = &tmrInitialTimerDataAndName;
            UTL_COPY_IYREG_TO_HLREG;

            // Load the address of the 7th character name.  This is where the 
            //   LSD of timer number is stored.
            HLReg = &CTMRName6;

            // Set IYReg the address of the initial data table.
            IYReg = &tmrInitialDataTable;

            //-----------------------------------------------------------------
            // Opens a database file in external memory for read/write access.
            // Destroys nothing.
            // Input:  DBExternalMemoryAddress - Absolute address of DB in
            //                                     external memory (16-bit).
            // Output: DBExternalBaseAddress   - Base address of DB use to 
            //                                     convert offsets to absolute
            //                                     address (16-bit)
            //         DBNumberOfRecords       - Number of records for the 
            //                                     current DB (16-bit)
            //         DBSizePerRecord         - Size per record of DB (8-bit)
            //-----------------------------------------------------------------
            IXReg = COREInitializationADDAddress;
            DBExternalMemoryAddress = IXReg;
            DB_OPEN_FILE;

            for( BReg = 0 ; BReg < TMRNUMBEROFRECORDS ; BReg++ )
            {
                // Get the initial timer data from the table and store it
                //   to a buffer before writing it to EEPROM.
                IXReg = *IYReg;
                CTMRUserSetMinute = IXReg;

                // Increment the timer number.  Check if its the 10th timer entry.
                *HLReg++;
                if( *HLReg >= 10 )
                {
                    *HLReg = DM5_1;                 // Set the 6th position value to 1.
                    HLReg++;
                    *HLReg = DM5_0;                 // Set the 7th position value to 0.
                }

                //-------------------------------------------------------------
                // Writes to the specific record in EEPROM.
                // Destroys BReg.
                // DBInternalMemoryAddress and DBLengthLo have already been 
                //   initialized.
                // Input: DBRecordNumber - Record number to write (16-bit)
                //                 
                // NOTE: DBRecordNumber can be separated into DBRecordNumberLo
                //         and DBRecordNumberHi.  Since the initial number of
                //         records is 10, DBRecordNumberHi is always 0.  Only
                //         DBRecordNumberLo will be changed.
                //-------------------------------------------------------------
                DBRecordNumberLo = BReg;
                push    (BAReg);
                DB_WRITE_RECORD_RANDOMFIX;
                pop     (BAReg);

                // Point to the next data.
                IYReg += 2;
                
            }

            //-----------------------------------------------------------------
            // Close the opened database and power down the EEPROM.
            //-----------------------------------------------------------------
            DB_CLOSE_FILE;
            
            pop     (IYReg);

//-----------------------------------------------------------------------------
//
// W A R N I N G !!! This is a fall through.  Do not rearrange.
//
//-----------------------------------------------------------------------------

        case    COREEVENT_INIT:
            //*****************************************************************
            //
            //                  APPLICATION INITIALIZATION
            //
            //*****************************************************************

            // Initial the last entry viewed to be the first record.
            BReg = 0;
            *(IYReg + TMRLASTVIEWEDENTRYOFFSET) = BReg;

            goto    tmrBackgroundTurnOffResourceFlag;


        case    COREEVENT_UPDATEDATABASEHEADER:
            //*****************************************************************
            //
            //                  UPDATE DATABASE HEADER
            //
            //*****************************************************************

            //#----------------------------------------------------------------
            //# ECR# 851-20020625-376
            //#   Clear 'bTMRNewTimerData' to indicate that no interval timer
            //#     has started.
            //#   Clear 'bTMRCameFromPopup' to indicate that default state
            //#     entry came from a mode change. 
            //#----------------------------------------------------------------
            AReg = *(IYReg + TMRFLAGS2OFFSET);
            AReg &= ~(bTMRNewTimerData | bTMRCameFromPopup);
            *(IYReg + TMRFLAGS2OFFSET) = AReg;

            // Reset the resource used by this app.
            AReg = *(IYReg + TMRRESOURCEINDEX0OFFSET);
            KTMR_RESET_RESOURCE;

            // Turn off resource flags used by this app.
            goto    tmrBackgroundTurnOffResourceFlag;


        case    COREEVENT_TASKEXIT:
            //*****************************************************************
            //
            //              APPLICATION TASK EXIT EVENT
            //
            //*****************************************************************

            //-----------------------------------------------------------------
            // This is a core generic bit.  The timer will set this bit when it
            //   enters the set state and clears it when going back to default.
            //   The core will issue a task exit event when there is a transition
            //   from set to default.  Since this bit will be cleared, it would
            //   not update the EEPROM data.  If there was another event occured,
            //   for example, USB is plugged-in causing it to go to COMM mode.
            //   Since this bit is still set, it will save the current setting to
            //   the EEPROM.
            //-----------------------------------------------------------------

            // Check if we are still in set mode.
            HLReg = &CORESystemFlag3;
            if( *HLReg.bCORESetMode == TRUE )
            {
                // Clear the bit to indicate that we are finish saving the current
                //   data to EEPROM.
                CORE_DISABLE_SET_MODE;

                //-------------------------------------------------------------
                //  Updates the individual bits according to the following:
                //      - Less than 1 minute
                //      - Less than 15 seconds
                //      - Zero data.
                //  Disables the halfway reminder alert if user set is less than 
                //    1 minute.
                //  Destroys IXReg.
                //-------------------------------------------------------------
                ctmrUpdateFlagStatus();

                //-------------------------------------------------------------
                // Writes the status flag and user set data.
                // Input: AReg  - Record number to write to.
                //        HLReg - Absolute address of DB file in external memory.
                //-------------------------------------------------------------
                AReg = *(IYReg + TMRLASTVIEWEDENTRYOFFSET);
                HLReg = COREBackgroundADDAddress;
                ctmrWriteStatusFlagAndHMSDataToEEPROM();

            }

            break;


        case    COREEVENT_TIMERHALFWAYACHIEVED:
            //*****************************************************************
            //
            //              HALFWAY TIME HAVE BEEN REACHED
            //
            //*****************************************************************

            // Request halfway alert melody and don't pass a melody event.
            AUDSTART_SYSTEM_MELODY  AUDHALFTIMERMELODY, AUDNOMELODYDONEEVENT

            // Increment usage counter indicating that a halfway reminder
            //   is being tracked and done.
            USG_INCREMENT_EEPROM_COUNTERS   USGHALFWAYREMINDERS;

            break;

        case    COREEVENT_STP_FORCIBLY_STARTED:
            //*****************************************************************
            //
            //              CHRONO HAS SUCCESSFULLY STARTED
            //
            //*****************************************************************

            // This event means that a chrono is started when the timer
            //   has completed its countdown.

            // Increment the usage counter.
            USG_INCREMENT_EEPROM_COUNTERS   USGTMRPOPUPCHRONOATEND;

tmrBackgroundTurnOffResourceFlag:

            // Turn off timer related flags.
            AReg = COREBackgroundAppIndex;
            LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  TMR_RSRC_FLAG, FLAG_OFF

            AReg = COREBackgroundAppIndex;
            LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  CHR_RSRC_FLAG, FLAG_OFF

            AReg = COREBackgroundAppIndex;
            LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  ARROW_RSRC_FLAG, FLAG_OFF

            //#----------------------------------------------------------------
            //# ECR# 851-20020621-368 
            //#----------------------------------------------------------------
            AReg = COREBackgroundAppIndex;
            LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  TAIL_RSRC_FLAG, FLAG_OFF;

            break;

        case    COREEVENT_TIMERFINISHED:
            //*****************************************************************
            //
            //              TIMER COUNTDOWN IS COMPLETE
            //
            //*****************************************************************

            // Get the status flag and clear other bits leaving only the action
            //   at end.
            AReg = *(IYReg + TMRFLAGS1OFFSET);
            AReg &= CTMRACTIONATENDMASKBITS;

            switch( AReg )
            {

                case    CTMRSTOPATEND:
                case    CTMRCHRONOATEND:
                    //=========================================================
                    //
                    //                  STOP AT END
                    //         CHRONO AT END (if failed to start chrono)
                    //
                    //=========================================================
                    
                    AReg = CTMRSTOPATENDALERT;
                    *(IYReg + TMRPOPUPTYPEOFFSET) = AReg;
                                    
                    //-----------------------------------------------------------------
                    // Request a timer popup.
                    // Destroys nothing.
                    // Input: AReg - App index
                    //        LReg - Resource index.
                    //-----------------------------------------------------------------
                    LReg = *(IYReg + TMRRESOURCEINDEX0OFFSET);
                    AReg = COREBackgroundAppIndex;
                    CORETASK_POPUP_REQUEST_TMR;
                    
                    break;

                   
                case    CTMRREPEATATEND:
                    //=========================================================
                    //
                    //                  REPEAT AT END
                    //
                    //=========================================================

                    // Set to indicate to update the repeat number display and 
                    //   a new countdown has started.
                    LReg = *(IYReg + TMRFLAGS2OFFSET);
                    LReg |= bTMRUpdRepeatNumDisplay;
                            
                    AReg = COREBackgroundAppIndex;
                    if( AReg == CORECurrentMode )
                    {
                        LReg |= bTMRNewTimerData;
                    }

                    *(IYReg + TMRFLAGS2OFFSET) = LReg;

                    //-----------------------------------------------------
                    // Get the number of repetitions.  Set repeat counter
                    //   to '0' if it reaches 200d.
                    // Destroys HLReg.
                    // Input:  AReg - Resource index.
                    // Output: BReg - Running repeat counter (HEX value)
                    //-----------------------------------------------------
                    AReg = *(IYReg + TMRRESOURCEINDEX0OFFSET);
                    KTMR_GET_RUNNING_REPEAT_COUNTER;
                    if( BReg == CTMRMAXREPEATCOUNT )
                    {
                        //-------------------------------------------------
                        // Change the value of the repeat counter.
                        // Destroys nothing.
                        // Input: AReg - Resource index
                        //        BReg - Starting repetition counter (HEX)
                        //-------------------------------------------------
                        BReg = 0;
                        KTMR_SET_RUNNING_REPEAT_COUNTER;
                    }

                    // Check if the data is less than 15 seconds.
                    HLReg = IYReg;
                    HLReg += TMRFLAGS1OFFSET;
                    if( *HLReg.bTMRLess15Sec == TRUE )
                    {
                        //-----------------------------------------------------
                        // Request a confirmation tone for the 'user set data 
                        //   is less than 15 seconds' in the background handler 
                        //   so that even if the crown is in set state, the user 
                        //   would still know that it has completed a countdown 
                        //   and a new countdown will be started.
                        //-----------------------------------------------------

                        // Request a confirmation tone.                     
                        AUDSTART_SYSTEM_MELODY AUDSWBEEPMELODY, AUDNOMELODYDONEEVENT
                    }

                    else
                    {
                        // Data is not less than 15 seconds.  Setup to indicate that
                        //   a repeat at end popup is requested.
                        AReg = CTMRREPEATATENDALERT;
                        *(IYReg + TMRPOPUPTYPEOFFSET) = AReg;

                        //-----------------------------------------------------------------
                        // Request a timer popup.
                        // Destroys IXReg.
                        // Input: AReg - App index
                        //        LReg - Resource index.
                        //-----------------------------------------------------------------
                        LReg = *(IYReg + TMRRESOURCEINDEX0OFFSET);
                        AReg = COREBackgroundAppIndex;
                        CORETASK_POPUP_REQUEST_TMR;
                    }
                    
                    break;

            }
    }
}


ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'tmrBackgroundHandler'"

        GLOBAL  tmrBackgroundHandler

tmrBackgroundHandler:                               ; **SUBROUTINE tmrBackgroundHandler

                ; Initialize IYReg to point to the ASD address.
                ld      IY, [COREBackgroundASDAddress]

                ; Load the type of event to be checked.
                ld      A, [COREBackgroundEvent]

                cp      A, #COREEVENT_TASKEXIT
                jr      Z, tmrBackgroundTaskExitEvent

                cp      A, #COREEVENT_STP_FORCIBLY_STARTED
                jr      Z, tmrBackgroundStopwatchStartedEvent

                cp      A, #COREEVENT_TIMERHALFWAYACHIEVED
                jr      Z, tmrBackgroundTimerHalfwayAchievedEvent

                cp      A, #COREEVENT_TIMERFINISHED
                jr      Z, tmrBackgroundTimerFinishedEvent

                cp      A, #COREEVENT_UPDATEDATABASEHEADER
                jr      Z, tmrBckHdUpdDBHeaderEvent

                cp      A, #COREEVENT_INIT
                jr      Z, tmrBackgroundInitEvent

                cp      A, #COREEVENT_PORINIT
                jr      NZ, tmrBackgroundExit

                ;**************************************************************
                ;
                ;              POWER ON RESET INITIALIZATION
                ;
                ;**************************************************************

                push    IY

                ;==============================================================
                ; Initialize DB generic header and app specific header.
                ;==============================================================
                
                ;--------------------------------------------------------------
                ; Writes the DB header information.  Turning on and off of the 
                ;   EEPROM is also included.
                ; Destroyes nothing.
                ; Input: DBExternalMemoryAddress - Absolute address of DB in 
                ;                                    external memory (16-bit)
                ;        DBInternalMemoryAddress - Source address in internal
                ;                                    memory (16-bit)
                ;        DBLengthLo - Number of bytes to write (8-bit) 
                ;--------------------------------------------------------------
                ld      B, #(TMRADDGENERICHEADERSIZE + TMRAPPSPECIFICHEADERSIZE)
                ld      [DBLengthLo], B
                ld      IX, [COREInitializationADDAddress]
                ld      [DBExternalMemoryAddress], IX
                ld      IX, #tmrDBHeaderInitialValue
                ld      [DBInternalMemoryAddress], IX
                DB_WRITE_DATABASE_HEADER


                ;==============================================================
                ; Initialize 10 timer entries.
                ;==============================================================
            

                ;--------------------------------------------------------------
                ; Initialize variable needed to write to an EEPROM.
                ; DBInternalMemoryAddress - Source base address in internal 
                ;                             memory (16-bit)
                ; DBLengthLo              - Number of bytes to write (8-bit) 
                ; DBRecordNumber          - Record number to write (16-bit)
                ;                 
                ; NOTE: DBRecordNumber can be separated into DBRecordNumberLo
                ;         and DBRecordNumberHi.  Since the initial number of
                ;         records is 10, DBRecordNumberHi is always 0.
                ;--------------------------------------------------------------
                ld      B, #0
                ld      [DBRecordNumberHi], B
                ld      HL, #CTMRFlags
                ld      [DBInternalMemoryAddress], HL
                ld      B, #TMRRECORDSIZE
                ld      [DBLengthLo], B

                ;--------------------------------------------------------------
                ; Copies the initial status flag, HMS data and name.  The HM 
                ;   data is insignificant.
                ; HLReg and BReg are already loaded with the correct data when
                ;   initializing the variables to write to EEPROM.
                ; Destroys BReg, HLReg, IYReg.
                ; Input: BReg  - Number of bytes to copy.
                ;        IYReg - Lowest address of the source structure.
                ;        HLReg - Lowest address of the destination structure.
                ;--------------------------------------------------------------
                ld      IY, #tmrInitialTimerDataAndName
                UTL_COPY_IYREG_TO_HLREG
                
                ; Load the address of the 7th character name.  This is where the 
                ;   LSD of timer number is stored.
                ld      HL, #CTMRName6
                
                ; Set IYReg the address of the initial data table.
                ld      IY, #tmrInitialDataTable

                ;--------------------------------------------------------------
                ; Opens a database file in external memory for read/write access.
                ; Destroys nothing.
                ; Input:  DBExternalMemoryAddress - Absolute address of DB in
                ;                                     external memory (16-bit).
                ; Output: DBExternalBaseAddress   - Base address of DB use to 
                ;                                     convert offsets to absolute
                ;                                     address (16-bit)
                ;         DBNumberOfRecords       - Number of records for the 
                ;                                     current DB (16-bit)
                ;         DBSizePerRecord         - Size per record of DB (8-bit)
                ;--------------------------------------------------------------
                ld      IX, [COREInitializationADDAddress]
                ld      [DBExternalMemoryAddress], IX
                DB_OPEN_FILE

                ld      B, #0

tmrWriteInitDataToEEPROMLoop:

                ; Get the initial timer data from the table and store it
                ;   to a buffer before writing it to EEPROM.
                ld      IX, [IY]
                ld      [CTMRUserSetMinute], IX
                
                ; Increment the timer number.  Check if its the 10th timer entry.
                inc     [HL]
                cp      [HL], #10
                jr      C, tmrWriteToEEPROM
                
                ; Set the 6th position value to 1.
                ld      [HL], #DM5_1

                inc     HL

                ;Set the 7th position value to 0.
                ld      [HL], #DM5_0

tmrWriteToEEPROM:

                ;--------------------------------------------------------------
                ; Writes to the specific record in EEPROM.
                ; Destroys BReg.
                ; DBInternalMemoryAddress and DBLengthLo have already been 
                ;   initialized.
                ; Input: DBRecordNumber - Record number to write (16-bit)
                ;                 
                ; NOTE: DBRecordNumber can be separated into DBRecordNumberLo
                ;         and DBRecordNumberHi.  Since the initial number of
                ;         records is 10, DBRecordNumberHi is always 0.  Only
                ;         DBRecordNumberLo will be changed.
                ;--------------------------------------------------------------
                ld      [DBRecordNumberLo], B
                push    BA
                DB_WRITE_RECORD_RANDOMFIX
                pop     BA
                                
                ; Point to the next data.
                add     IY, #2

                ; Point to the next record.
                inc     B

                ; Check if all of the 10 records have been initialized.
                cp      B, #TMRNUMBEROFRECORDS
                jr      C, tmrWriteInitDataToEEPROMLoop
                
                ;--------------------------------------------------------------
                ; Close the opened database and power down the EEPROM.
                ;--------------------------------------------------------------
                DB_CLOSE_FILE
            
                pop     IY

;------------------------------------------------------------------------------
;
; W A R N I N G !!! This is a fall through.  Do not rearrange.
;
;------------------------------------------------------------------------------

tmrBackgroundInitEvent:
                ;**************************************************************
                ;
                ;              APPLICATION INITIALIZATION
                ;
                ;**************************************************************
                
                ; Initial the last entry viewed to be the first record.
                ld      B, #0
                ld      [IY + TMRLASTVIEWEDENTRYOFFSET], B

                jr      tmrBackgroundTurnOffResourceFlag




tmrBckHdUpdDBHeaderEvent:
                ;**************************************************************
                ;
                ;                  UPDATE DATABASE HEADER
                ;
                ;**************************************************************

                ;#-------------------------------------------------------------
                ;# ECR# 851-20020625-376
                ;#   Clear 'bTMRNewTimerData' to indicate that no interval timer
                ;#     has started.
                ;#   Clear 'bTMRCameFromPopup' to indicate that default state
                ;#     entry came from a mode change. 
                ;#-------------------------------------------------------------
                ld      A, [IY + TMRFLAGS2OFFSET];
                and     A, #@LOW(~(bTMRNewTimerData | bTMRCameFromPopup));
                ld      [IY + TMRFLAGS2OFFSET], A

                ; Reset the resource used by this app.
                ld      A, [IY + TMRRESOURCEINDEX0OFFSET]
                KTMR_RESET_RESOURCE
                
                ; Turn off resource flags used by this app.
                jr      tmrBackgroundTurnOffResourceFlag


tmrBackgroundTaskExitEvent:
                ;**************************************************************
                ;
                ;              APPLICATION TASK EXIT EVENT
                ;
                ;**************************************************************
                
                ;==============================================================
                ; This is a core generic bit.  The timer will set this bit when 
                ;   it enters the set state and clears it when going back to 
                ;   default.  The core will issue a task exit event when there 
                ;   is a transition from set to default.  Since this bit will 
                ;   be cleared, it would not update the EEPROM data.  If there 
                ;   was another event occured, for example, USB is plugged-in 
                ;   causing it to go to COMM mode.  Since this bit is still set, 
                ;   it will save the current setting to the EEPROM.
                ;==============================================================
                                
                ; Check if we are still in set mode.
                ld      HL, #CORESystemFlag3
                bit     [HL], #bCORESetMode
                jr      Z, tmrBackgroundExit

                ; Clear the bit to indicate that we are finish saving the current
                ;   data to EEPROM.
                CORE_DISABLE_SET_MODE
                
                ;--------------------------------------------------------------
                ;   Updates the individual bits according to the following:
                ;       - Less than 1 minute
                ;       - Less than 15 seconds
                ;       - Zero data.
                ;   Disables the halfway reminder alert if user set is less than 
                ;    1 minute.
                ;  Destroys IXReg.
                ;--------------------------------------------------------------
                car     ctmrUpdateFlagStatus

                ;--------------------------------------------------------------
                ; Writes the status flag and user set data.
                ; Input: AReg  - Record number to write to.
                ;        HLReg - Absolute address of DB file in external memory.
                ;--------------------------------------------------------------
                ld      A, [IY + TMRLASTVIEWEDENTRYOFFSET]
                ld      HL, [COREBackgroundADDAddress]
                jr      ctmrWriteStatusFlagAndHMSDataToEEPROM   ; **EXTERNAL JUMP

                
tmrBackgroundTimerHalfwayAchievedEvent:
                ;**************************************************************
                ;
                ;              HALFWAY TIME HAVE BEEN REACHED
                ;
                ;**************************************************************
                
                ; Request halfway alert melody and don't pass a melody event.
                AUDSTART_SYSTEM_MELODY  AUDHALFTIMERMELODY, AUDNOMELODYDONEEVENT

                ; Increment usage counter indicating that a halfway reminder
                ;   is being tracked and done.
                USG_INCREMENT_EEPROM_COUNTERS   USGHALFWAYREMINDERS

tmrBackgroundExit:

                ret


tmrBackgroundStopwatchStartedEvent:
                ;**************************************************************
                ;
                ;              CHRONO HAS SUCCESSFULLY STARTED
                ;
                ;**************************************************************

                ; This event means that a chrono is started when the timer
                ;   has completed its countdown.

                ; Increment the usage counter.
                USG_INCREMENT_EEPROM_COUNTERS   USGTMRPOPUPCHRONOATEND
                    
tmrBackgroundTurnOffResourceFlag:

                ; Turn off timer related flags.
                ld      A, [COREBackgroundAppIndex]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  TMR_RSRC_FLAG, FLAG_OFF

                ld      A, [COREBackgroundAppIndex]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  CHR_RSRC_FLAG, FLAG_OFF

                ld      A, [COREBackgroundAppIndex]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  ARROW_RSRC_FLAG, FLAG_OFF
                
                ;#-------------------------------------------------------------
                ;# ECR# 851-20020621-368 
                ;#-------------------------------------------------------------
                ld      A, [COREBackgroundAppIndex]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  TAIL_RSRC_FLAG, FLAG_OFF

                ret


tmrBackgroundTimerFinishedEvent:
                ;**************************************************************
                ;
                ;              TIMER COUNTDOWN IS COMPLETE
                ;
                ;**************************************************************
                
                ; Get the status flag and clear other bits leaving only the action
                ;   at end.
                ld      A, [IY + TMRFLAGS1OFFSET]
                and     A, #CTMRACTIONATENDMASKBITS

                ; Check if stop at end.
                cp      A, #CTMRSTOPATEND
                jr      Z, tmrBackgroundStopAtEnd

                cp      A, #CTMRCHRONOATEND
                jr      Z, tmrBackgroundStopAtEnd

                ;==============================================================
                ;
                ;                  REPEAT AT END
                ;
                ;==============================================================
                                
                ; Set to indicate to update the repeat number display and 
                ;   a new countdown has started.
                ld      L, [IY + TMRFLAGS2OFFSET]
                or      L, #bTMRUpdRepeatNumDisplay

                ld      A, [COREBackgroundAppIndex]
                cp      A, [CORECurrentMode]
                jr      NZ, tmrBackgroundRepeatAtEndContinue

                or      L, #bTMRNewTimerData

tmrBackgroundRepeatAtEndContinue:

                ld      [IY + TMRFLAGS2OFFSET], L

                ;--------------------------------------------------------------
                ; Get the number of repetitions.
                ; Destroys HLReg.
                ; Input:  AReg - Resource index.
                ; Output: BReg - Running repeat counter (HEX value)
                ;--------------------------------------------------------------
                ld      A, [IY + TMRRESOURCEINDEX0OFFSET]
                KTMR_GET_RUNNING_REPEAT_COUNTER

                ; Check if repeat count reaches its maximum (200d).
                ;   If yes, then set it back to '0'.
                cp      B, #CTMRMAXREPEATCOUNT
                jr      NZ, tmrBackgroundChkTimerData

                ;--------------------------------------------------------------
                ; Change the value of the repeat counter.
                ; Input: AReg - Resource index
                ;        BReg - Starting repetition counter (HEX)
                ;--------------------------------------------------------------
                ld      B, #0
                KTMR_SET_RUNNING_REPEAT_COUNTER


tmrBackgroundChkTimerData:

                ; Check if the data is less than 15 seconds.
                ld      HL, IY
                add     HL, #TMRFLAGS1OFFSET
                bit     [HL], #bTMRLess15Sec
                jr      Z, tmrBackgroundProcessNotLess15

                ;--------------------------------------------------------------
                ; Request a confirmation tone for the 'user set data is less 
                ;   than 15 seconds' in the background handler so that even if 
                ;   the crown is in set state, the user would still know that 
                ;   it has completed a countdown and a new countdown will be 
                ;   started.
                ;--------------------------------------------------------------
                AUDSTART_SYSTEM_MELODY  AUDSWBEEPMELODY, AUDNOMELODYDONEEVENT

                ret

tmrBackgroundProcessNotLess15:

                ; Data is not less than 15 seconds.  Setup to indicate that
                ;   a repeat at end popup is requested.
                ld      A, #CTMRREPEATATENDALERT
                ld      [IY + TMRPOPUPTYPEOFFSET], A

tmrBackgroundReqPopup:

                ;--------------------------------------------------------------
                ; Request a timer popup.
                ; Destroys nothing.
                ; Input: AReg - App index
                ;        LReg - Resource index.
                ;--------------------------------------------------------------
                ld      L, [IY + TMRRESOURCEINDEX0OFFSET]
                ld      A, [COREBackgroundAppIndex]
                CORETASK_POPUP_REQUEST_TMR

                ret

tmrBackgroundStopAtEnd:
                ;==============================================================
                ;
                ;                  STOP AT END
                ;         CHRONO AT END (if failed to start chrono)
                ;
                ;==============================================================
                                    
                ld      A, #CTMRSTOPATENDALERT
                ld      [IY + TMRPOPUPTYPEOFFSET], A
                                     
                ; Request a popup.
                jr      tmrBackgroundReqPopup

                    
                



                                                    ; **END SUBROUTINE tmrBackgroundHandler

        CALLS   SUBROUTINE, 'ctmrUpdateFlagStatus'


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : tmrInitialDataTable
;
; Description : Table for the initial values of timer data during POR.
;
; Assumptions : None
;
; Input(s)    : None
;
; Output(s)   : None
;
; Module Change History:
;
;               12/18/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

            GLOBAL  tmrInitialDataTable

tmrInitialDataTable:                                ; **SUBROUTINE tmrInitialDataTable

            ; This table only holds the hour and minute definition since seconds data during
            ;   POR is 0.
            
            dw      0001h                           ; 1 Minute
            dw      0002h                           ; 2 Minutes
            dw      0003h                           ; 3 Minutes
            dw      0005h                           ; 5 Minutes
            dw      0010h                           ; 10 Minutes
            dw      0015h                           ; 15 Minutes
            dw      0020h                           ; 20 Minutes
            dw      0030h                           ; 30 Minutes
            dw      0045h                           ; 45 Minutes
            dw      0100h                           ; 1 hour

                                                    ; **END SUBROUTINE tmrInitialDataTable

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : tmrDBHeaderInitialValue
;
; Description : Table for the initial values of DB header during POR.
;
; Assumptions : None
;
; Input(s)    : None
;
; Output(s)   : None
;
; Module Change History:
;
;               03/01/2002  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


            GLOBAL  tmrDBHeaderInitialValue

tmrDBHeaderInitialValue:                            ; **SUBROUTINE tmrDBHeaderInitialValue

            dw      TMREEPROMDBALLOCATIONSIZE       ; Allocation size
            dw      TMREEPROMDATABASEDATASIZE       ; DB size
            db      TMRAPPSPECIFICHEADERSIZE        ; App specific header size
            dw      TMRNUMBEROFRECORDS              ; Number of records
            db      TMRRECORDSIZE                   ; Record size

                                                    ; **END SUBROUTINE tmrDBHeaderInitialValue

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : tmrInitialTimerDataAndName
;
; Description : Initial timer data and name.
;
; Assumptions : None
;
; Input(s)    : None
;
; Output(s)   : None
;
; Module Change History:
;
;               03/01/2002  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        GLOBAL  tmrInitialTimerDataAndName

tmrInitialTimerDataAndName:                         ; **SUBROUTINE tmrInitialTimerDataAndName
                             
                db      0                           ; Flag status
                db      0                           ; Seconds
                db      0                           ; Minute
                db      0                           ; Hour
                db      DM5_T                       ; Name0
                db      DM5_I                       ; Name1
                db      DM5_M                       ; Name2
                db      DM5_E                       ; Name3
                db      DM5_R                       ; Name4
                db      DM5_SPACE                   ; Name5
                db      DM5_0                       ; Name6
                db      DM5_SPACE                   ; Name7
                db      DM5_SPACE                   ; Name8


                                                    ; **END SUBROUTINE tmrInitialTimerDataAndName



; **END FILE tmrBckHd.asm

        END