;**FILE tmrdef.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2000 - 2002 TIMEX Corporation.
; 
; This document is the property of, and contains information proprietary to
; TIMEX Corporation.  It is to be returned upon request.  Reproduction by any
; means, disclosure, or use of its contents for other than the expressed
; purpose for which it is loaned, without the written consent of TIMEX
; Corporation, is strictly prohibited.
;
; File Name    : tmrDef.asm
;
; Module Names : tmrDefaultStateManager
;                   
; Purpose      : Countdown Timer Application Default State Manager
;
; File Change History:
;
;                12/12/2001  - Created tmrdef.asm for m851
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

$CASE ON
$LIST OFF

        INCLUDE '..\..\CORE\H\MODEL.H'
        INCLUDE '..\..\HW\H\HW88349.H'
        INCLUDE '..\..\HW\H\HWI2CEE.H'
        INCLUDE '..\..\HW\H\HWBOND.H'
        INCLUDE '..\..\HW\H\HWKBD.H'
        INCLUDE '..\..\HW\H\HWMACRO.H'
        INCLUDE '..\..\HW\H\HWMAIN.H'
        INCLUDE '..\..\HW\H\HWRESET.H'
        INCLUDE '..\..\HW\H\HWVARS.H'
        INCLUDE '..\..\HW\H\HWSERMAC.H'
        INCLUDE '..\..\CORE\H\CORE.H'
        INCLUDE '..\..\CORE\H\COREMAC.H'
        INCLUDE '..\..\CORE\H\CORESVMC.H'
        INCLUDE '..\..\COMM\H\COMM.H'
        INCLUDE '..\..\LCD\H\LCD.H'
        INCLUDE '..\..\LCD\H\LCDMACRO.H'
        INCLUDE '..\..\LCD\H\LCDRAM.H'
        INCLUDE '..\..\DATABASE\H\DB.H'
        INCLUDE '..\..\DATABASE\H\DBMACRO.H'
        INCLUDE '..\..\AUDIO\H\AUD.H'
        INCLUDE '..\..\AUDIO\H\AUDMAC.H'
        INCLUDE '..\..\UTIL\H\UTLMATH.H'
        INCLUDE '..\..\UTIL\H\UTLMTHMC.H'
        INCLUDE '..\..\UTIL\H\UTLEDT.H'
        INCLUDE '..\..\UTIL\H\UTLEDTMC.H'
        INCLUDE '..\..\UTIL\H\UTLDSP.H'
        INCLUDE '..\..\UTIL\H\UTLDSPMC.H'
        INCLUDE '..\..\UTIL\H\UTLACCMC.H'
        INCLUDE '..\..\UTIL\H\UTLMSC.H'
        INCLUDE '..\..\UTIL\H\UTLMSCMC.H'
        INCLUDE '..\..\UTIL\H\UTLVARS.H'
        INCLUDE '..\..\KRES\H\KRES.H'
        INCLUDE '..\..\KRES\H\KRESMAC.H'
        INCLUDE '..\..\KRES\H\KSTP.H'
        INCLUDE '..\..\KRES\H\KSTPMAC.H'
        INCLUDE '..\..\KRES\H\KSYN.H'
        INCLUDE '..\..\KRES\H\KSYNMAC.H'
        INCLUDE '..\..\KRES\H\KTMR.H'
        INCLUDE '..\..\KRES\H\KTMRMAC.H'
        INCLUDE '..\..\KRES\H\KTOD.H'
        INCLUDE '..\..\KRES\H\KTODMAC.H'
        INCLUDE '..\..\KRES\H\KTZC.H'
        INCLUDE '..\..\KRES\H\KTZCMAC.H'
        INCLUDE '..\..\KRES\H\KBCK.H'
        INCLUDE '..\..\KRES\H\KBCKMAC.H'
        INCLUDE '..\..\USAGE\H\USG.H'
        INCLUDE '..\..\USAGE\H\USGVARS.H'
        INCLUDE '..\..\USAGE\H\USGMAC.H'
        INCLUDE '..\..\CORE\H\SECTION.H'

        INCLUDE '..\..\TOD\H\TOD.H'
        INCLUDE '..\..\TOD\H\TODVARS.H'
        
        INCLUDE '..\..\CTMR\H\CTMR.H'
        INCLUDE '..\..\CTMR\H\CTMRVARS.H'

        INCLUDE '..\..\TMR\H\TMRVARS.H'

$LIST ON

        SECT    ".text_tmr"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : tmrDefaultStateManager
;
; Description : Timer Application Default State Manager.
;
; Assumptions : Display is cleared on first time entry into the state.
;
; Input(s)    : CORECurrentEvent  - system event to be processed
;               COREEventArgument - event extra information
;
; Output(s)   : None
;
;               ( Destroyed: ALL registers )
;
; Module Change History:
;
;               12/12/2001  - Created module for M851     
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void tmrDefaultStateManager()
{
    
    // Set IYReg to point to the ASD address.
    IYReg = CORECurrentASDAddress;

    switch( CORECurrentEvent )
    {

        case    CTMR_STATEENTRY:
            //*****************************************************************
            //
            //                      STATE ENTRY EVENT
            //
            //*****************************************************************

            CORE_ENABLE_SWITCH_RELEASE;

            //-----------------------------------------------------------------
            // Check if it came from a countdown popup entry.  if yes, then
            //   don't reset the counter.
            //-----------------------------------------------------------------
            HLReg = CORECurrentASDAddress;
            HLReg += TMRFLAGS2OFFSET;
            if( *HLReg.bTMRCameFromPopup == FALSE )
            {
                CTMR10SecondsCounter = 0;
            }
            else
            {
                *HLReg &= ~bTMRCameFromPopup;
    
                if( CTMR10SecondsCounter != 0 )
                {
                    ctmrDisplayTimerNameAtLine2();
                }
            }



tmrDefaultPopupEntry:

            //-----------------------------------------------------------------
            // Copies the ASD data to a common foreground buffer to be used 
            //    for foreground processing.
            // Copies the entry specified in last viewed entry to a common 
            //    foreground buffer.
            //-----------------------------------------------------------------
            push    (IYReg);
            tmrCopyDataToCommonBuffer();
            pop     (IYReg);
                        
tmrDefaultDisplayAll:

            //-----------------------------------------------------------------
            // Set the update repeat number display so that it would refresh the 
            //   repeat number displayed on upper DM.
            //-----------------------------------------------------------------
            LReg = *(IYReg + TMRFLAGS2OFFSET);
            LReg |= bTMRUpdRepeatNumDisplay;
            *(IYReg + TMRFLAGS2OFFSET) = LReg;

            //-----------------------------------------------------------------
            // Enable ring events to be passed to this state assuming that 
            //   timer is in reset or stop state so that we can tranverse 
            //   other entries.
            // Destroys HLReg.
            //-----------------------------------------------------------------
            CORE_ENABLE_RING_EVENTS;              

            //-----------------------------------------------------------------
            // Disable display update event to be passed to this state assuming
            //   that timer is in reset or stop state.
            // Destroys BReg.
            // Input: AReg - Resource index.
            //-----------------------------------------------------------------
            AReg = CTMRActiveResource;
            KTMR_DISABLE_DISP_UPD_EVENT;

            //-----------------------------------------------------------------
            // Displays the action at end flags.
            // Destroys HLReg, IXReg.
            // Input: BReg - Contains the flag values that contains the action
            //                 at end.
            //-----------------------------------------------------------------
            BReg = CTMRFlags;
            ctmrDisplayActionAtEndFlags();

            //-----------------------------------------------------------------
            // Get the status of the timer.
            // Destroys nothing.
            // Input:  AReg - Resource index
            // Output: BReg - Status of the timer.
            //-----------------------------------------------------------------
            KTMR_GET_STATUS;

            // Check the status of the timer and process the procedures accordingly.
            switch( BReg )
            {
                case    KTMRSTATUSRESET:
                    //=========================================================
                    //
                    //                  RESET STATE
                    //
                    //=========================================================

                    //---------------------------------------------------------
                    // Display "1/2" at upper DM if halfway reminder is enabled.
                    // The halfway reminder bit should be in CTMRFlags.bCTMRHalfwayEnable
                    // Destroys BAReg, HLReg, IXReg.
                    //---------------------------------------------------------
                    ctmrDisplayOneHalfAtSDM();                                    

                    //---------------------------------------------------------
                    // Displays timer name at line 2.
                    // The name should be on the CTMRNameX where X = 0 to 8
                    // Destroys BReg, LReg, IXReg.
                    //---------------------------------------------------------
                    ctmrDisplayTimerNameAtLine2();

                    //---------------------------------------------------------
                    // Copies the user set data to work buffer.
                    // Destroys HLReg.
                    //---------------------------------------------------------
                    ctmrCopyUserSetToWorkBuffer();

                    //---------------------------------------------------------
                    // Displays the user set data on line 1.
                    // The user set data should be in CTMRWorkSecond, CTMRWorkMinute
                    //   and CTMRWorkHour.
                    // Destroys BAReg, HLReg, IXReg, IYReg.
                    // Input: BReg - Contains the bits of the data to be displayed.
                    //---------------------------------------------------------
                    BReg = CTMRUPDATEHMSDATA;
                    ctmrDisplayWorkHMSAtLine1();

                    break;


                case    KTMRSTATUSRUNNING:
                    //=========================================================
                    //
                    //                  RUNNING STATE
                    //
                    //=========================================================

                    //---------------------------------------------------------
                    // Enable display update events to be passed to this state.
                    // Destroys BReg.
                    // Input: AReg - Resource index
                    //---------------------------------------------------------
                    KTMR_ENABLE_DISP_UPD_EVENT;

                    //---------------------------------------------------------
                    // Suspend the ring events if timer is running so that the 
                    //   ring events would not be passed to the application.
                    //   Suspending the ring events would disable traversing
                    //   to other entries.
                    // Destroys HLReg.
                    //---------------------------------------------------------
                    CORE_SUSPEND_RING_EVENTS;

                    //---------------------------------------------------------
                    // Request timer flag to blink.
                    // Destroys BAReg.
                    //---------------------------------------------------------
                    CORE_REQ_BLINK_2HZ  TMR_FLAG;

//-----------------------------------------------------------------------------
//
// W A R N I N G !!! This is a fall through.  Do not rearrange.
//
//-----------------------------------------------------------------------------

                case    KTMRSTATUSSTOPPED:
                    //=========================================================
                    //
                    //                  STOP STATE
                    //
                    //=========================================================

                    //---------------------------------------------------------
                    // Display user set HMS at 9 segment.
                    // Destroys BAReg, HLReg, IXReg, IYReg.
                    //---------------------------------------------------------
                    ctmrDisplayUserSetHMSAtSegment();

                    //---------------------------------------------------------
                    // Displays working HMS and repeat number.
                    // Input: BReg - Contains the bits of the data to be displayed.
                    //---------------------------------------------------------
                    BReg = CTMRUPDATEHMSDATA;
                    goto    tmrDefaultDispWorkDataWithoutUpdBits;

            }

            break;


        case    CTMR_DISPUPDTMRRES:
            //*****************************************************************
            //
            //                  DISPLAY UPDATE EVENT
            //
            //*****************************************************************

tmrDefaultDisplayWorkHMS:
            //=================================================================
            // The display update event will only be called when there is a
            //   second update.
            //=================================================================

            //-----------------------------------------------------------------
            // Get the update bits.
            // Destroys HLReg.
            // Input:  AReg - Resource index.
            // Output: BReg - Contains the bits of which data was updated.
            //-----------------------------------------------------------------
            AReg = CTMRActiveResource;
            KTMR_GET_UPDATE_BITS;

            //-----------------------------------------------------------------
            // Check if the 10 seconds timer has expired from the time it was
            //   started and is now preparing to display the data in large font.
            //-----------------------------------------------------------------
            if( CTMR10SecondsCounter != 0 )
            {
                // Decrement counter.
                CTMR10SecondsCounter--;

                //-------------------------------------------------------------
                // Check if the counter has just expired.  If yes, then clear 
                //   the main DM and for the update bits to update the entire
                //   display.
                //-------------------------------------------------------------
                if( CTMR10SecondsCounter == 0 )
                {
                    // Clear main DM.
                    // Destroys AReg, IXReg.
                    LCD_CLEAR_MAIN_DM;

                    // Set BReg to update the entire display.
                    BReg = CTMRUPDATEHMSDATA;
                }
            }

tmrDefaultDispWorkDataWithoutUpdBits:
            //=================================================================
            // BReg should be set to the update bits when using this as an
            //   entry point.
            //=================================================================

            // Display new timer entry if a new timer has started.
            push    (BAReg);

            HLReg = CORECurrentASDAddress;
            HLReg += TMRFLAGS2OFFSET;
            if( *HLReg.bTMRNewTimerData == TRUE )
            {
                *HLReg &= ~bTMRNewTimerData;

                BReg = 10;
                CTMR10SecondsCounter = BReg;

                // Display timer name.
                LCD_CLR_MAIN_DM_LINE2;
                ctmrDisplayTimerNameAtLine2();
            }

            pop     (BAReg);

            //-----------------------------------------------------------------
            // Check if repeat at end and repeat number display needs to be
            //   updated before the display will commence.
            // Destroys HLReg, IXReg
            //-----------------------------------------------------------------
            tmrChkAndDispRepeatNumber();

            //-----------------------------------------------------------------
            // Display working data to either BDM or L1.
            // Input: CTMRActiveResource   - Resource index of the working data 
            //                                 to be displayed.
            //        CTMR10SecondsCounter - Counter to indicate whether to 
            //                                 display large or small font.
            //                               If 0, use large font.
            //                               If not 0, use small font.
            //        BReg.bKTMRSecondUpd  - Update seconds data.
            //            .bKTMRMinuteUpd  - Update minute data.  
            //            .bKTMRHourUpd    - Update hour data.
            //
            //-----------------------------------------------------------------
            ctmrDisplayWorkHMSAtBDMOrL1();

            break;


        case    CTMR_CWEDGE:
            //*****************************************************************
            //
            //                      CW EDGE EVENT
            //
            //*****************************************************************

            //-----------------------------------------------------------------
            // Reads the next DB record.
            // Resets the active resource.
            // Clears the entire screen.
            // Destroys BAReg, HLReg.
            // Assumptions : CORECurrentADDAddress - Absolute address of DB in external memory.
            //               CTMRNumberOfRecords   - Number of records in DB.
            //               CTMRLastViewedEntry   - Record number to be read.
            //               CTMRActiveResource    - Resource to be reset.
            //
            // Output(s)   : AReg, CTMRLastViewedEntry- Record number that was read.
            //               User set data is stored in:
            //                   -> CTMRFlags         - Status flags.
            //                   -> CTMRUserSetSecond - User set seconds data.
            //                   -> CTMRUserSetMinute - User set minute data.
            //                   -> CTMRUserSetHour   - User set hour data.
            //                   -> CTMRNameX         - 9 char timer name, where x = 0 to 8
            //-----------------------------------------------------------------
            ctmrReadNextRecord();
     
            // Save the last viewed entry.
            *(IYReg + TMRLASTVIEWEDENTRYOFFSET) = AReg;
            
            // Refresh the entire display.
            goto    tmrDefaultDisplayAll;


        case    CTMR_CCWEDGE:
            //*****************************************************************
            //
            //                      CCW EDGE EVENT
            //
            //*****************************************************************

            //-----------------------------------------------------------------
            // Reads the previous DB record.
            // Resets the active resource.
            // Clears the entire screen.
            // Destroys BAReg, HLReg.
            // Assumptions : CORECurrentADDAddress - Absolute address of DB in external memory.
            //               CTMRNumberOfRecords   - Number of records in DB.
            //               CTMRLastViewedEntry   - Record number to be read.
            //               CTMRActiveResource    - Resource to be reset.
            //
            // Output(s)   : AReg, CTMRLastViewedEntry- Record number that was read.
            //               User set data is stored in:
            //                   -> CTMRFlags         - Status flags.
            //                   -> CTMRUserSetSecond - User set seconds data.
            //                   -> CTMRUserSetMinute - User set minute data.
            //                   -> CTMRUserSetHour   - User set hour data.
            //                   -> CTMRNameX         - 9 char timer name, where x = 0 to 8
            //-----------------------------------------------------------------
            ctmrReadPrevRecord();

            // Save the last viewed entry.
            *(IYReg + TMRLASTVIEWEDENTRYOFFSET) = AReg;

            // Refresh the entire display.
            goto    tmrDefaultDisplayAll;
            

        case    CTMR_STARTSPLITDEPRESS:
            //*****************************************************************
            //
            //              START/SPLIT SWITCH DEPRESS EVENT
            //
            //*****************************************************************

            // Cancel the current switch release event coz it is not needed.
            HW_KBD_CANCEL_CURRENT_SWITCH_RELEASE;

            //-----------------------------------------------------------------
            // Get the status of the timer.
            // Destroys nothing.
            // Input:  AReg - Resource index
            // Output: BReg - Status of the timer.
            //-----------------------------------------------------------------
            AReg = CTMRActiveResource;
            KTMR_GET_STATUS;

            switch( BReg )
            {
                case    KTMRSTATUSRESET:
                    //=========================================================
                    //
                    //                  RESET STATE
                    //
                    //=========================================================

                    // Check if user set data is zero
                    HLReg = &CTMRFlags;
                    if( *HLReg.bCTMRDataIsZero == FALSE )
                    {
                        //-----------------------------------------------------
                        // Set the type of display counter to 10.  This means 
                        //   that to display a small font for 10 seconds.
                        //-----------------------------------------------------
                        BReg = 10;
                        CTMR10SecondsCounter = BReg;

                        //-----------------------------------------------------
                        // Save the status flag from the common buffer to ASD.
                        //-----------------------------------------------------
                        BReg = CTMRFlags;
                        *(IYReg + TMRFLAGS1OFFSET) = BReg;

                        //-----------------------------------------------------
                        // Sets up the resources user set and working HMS data.
                        // Counting direction have already been set to countdown.
                        // Destroys nothing.
                        // Input: AReg  - Resource index.
                        //        IYReg - Points to the beginning of the HMS
                        //                  structure of the user RAM area.
                        //-----------------------------------------------------
                        IYReg = &CTMRUserSetSecond;
                        KTMR_SETUP_USER_AND_WORK_HMS;

                        //-----------------------------------------------------
                        // Check if halfway reminder is enabled.  If yes, then
                        //   set it up.
                        // Destroys IYReg, IXReg, HLReg.
                        // Input: AReg - Resource index.
                        //        BReg - Contains the halfway reminder bit.
                        //        IYReg- Seconds address of the user set data.
                        //-----------------------------------------------------
                        BReg = CTMRFlags;
                        ctmrSetupHalfwayReminder();

                        //-----------------------------------------------------
                        // Sets up the resource action at end.  
                        //   Stop At End   - Cancel all links. Clear repeat at
                        //                     end bit.
                        //   Repeat At End - Set repeat at end bit.  Set the
                        //                     repeat count to 1.  Set the upd
                        //                     repeat number display.
                        //   Chrono At End - Create a link to STP.
                        // Destroys BAReg, IYReg, IXReg, HLReg.
                        //-----------------------------------------------------
                        tmrSetupActionAtEnd();

                        // Clear upper DM data in case "1/2" is displayed.
                        LCD_CLEAR_UPPER_DM;

                        //-----------------------------------------------------
                        // Start the resource and refresh the display.
                        // Input: AReg - Resource index.
                        //-----------------------------------------------------
                        AReg = CTMRActiveResource;
                        goto    tmrDefaultStartResourceCommon;

                    }

                    break;
                    
                case    KTMRSTATUSSTOPPED:
                    //=========================================================
                    //
                    //                  STOP STATE
                    //
                    //=========================================================

                    //---------------------------------------------------------
                    // Check if working HMS data is zero.
                    // Destroys nothing.
                    // Input:  AReg - Resource index
                    // Output: bZeroFlag = 0 if data is non zero.
                    //                   = 1 if data is zero.
                    //---------------------------------------------------------
                    KTMR_CHECK_WORK_HMS_IF_ZERO;
                    if( bZeroFlag == FALSE )
                    {

tmrDefaultStartResourceCommon:
                        //=====================================================
                        // AReg should be set to the resource index when using
                        //   this as an entry point.
                        //=====================================================

                        //-----------------------------------------------------
                        // Starts the timer resource.
                        // Destroys nothing.
                        // Input: AReg - Resource index
                        //-----------------------------------------------------
                        KTMR_START_RESOURCE;

                        AUDSTART_SYSTEM_MELODY AUDSWBEEPMELODY, AUDNOMELODYDONEEVENT;

                        //-----------------------------------------------------
                        // Update LCD resource flags.
                        // Destroys BAReg, HLReg, IXReg.
                        // Input: BReg - Contains the flag values that contains
                        //                 the action at end.
                        //-----------------------------------------------------
                        BReg = CTMRFlags;
                        ctmrUpdateResourceFlags();

                        //-----------------------------------------------------
                        // Refresh the entire display.
                        // Ring events are suspended.
                        // Request timer flag blinking.
                        // Display update is enabled.
                        // Update the primary icon resource.
                        //-----------------------------------------------------
                        goto    tmrDefaultDisplayAll;

                    }

                    break;

            }

            break;


        case    CTMR_STOPRESETDEPRESS:
            //*****************************************************************
            //
            //              STOP/RESET SWITCH DEPRESS EVENT
            //
            //*****************************************************************

            //-----------------------------------------------------------------
            // Get the status of the timer.
            // Destroys nothing.
            // Input:  AReg - Resource index
            // Output: BReg - Status of the timer.
            //-----------------------------------------------------------------
            AReg = CTMRActiveResource;
            KTMR_GET_STATUS;

            switch( BReg )
            {
                case    KTMRSTATUSRESET:
                    //=========================================================
                    //
                    //                  RESET STATE
                    //
                    //=========================================================

                    // Cancel the current switch release to be passed coz it is
                    //   not needed.
                    HW_KBD_CANCEL_CURRENT_SWITCH_RELEASE;

                    // Check if data is less than 1 minute, if not then toggle
                    //   halfway reminder bit.
                    HLReg = &CTMRFlags;
                    if( *HLReg.bCTMRLess1Min == FALSE )
                    {
                        // Toggle the halfway reminder bit.
                        *HLReg ^= bCTMRHalfwayEnable;

                        //-----------------------------------------------------
                        // Display "1/2" at upper DM if halfway reminder is 
                        //   enabled.
                        // The halfway reminder bit should be in CTMRFlags.bCTMRHalfwayEnable
                        // Destroys BAReg, HLReg, IXReg.
                        //-----------------------------------------------------
                        ctmrDisplayOneHalfAtSDM();                                    
                            
                        //-----------------------------------------------------
                        // Write the status flag to the EEPROM.
                        // Destroys BAReg, HLReg.
                        // Input: AReg  - Record number to write to.
                        //        HLReg - Absolute address of DB file in
                        //                  external memory.
                        //        CTMRLastViewedEntry - Record number to write to.
                        //-----------------------------------------------------
                        HLReg = CORECurrentADDAddress;
                        ctmrWriteStatusFlagToEEPROM();
                    }

                    break;


                case    KTMRSTATUSRUNNING:
                    //=========================================================
                    //
                    //                  RUNNING STATE
                    //
                    //=========================================================

                    //---------------------------------------------------------
                    // Stops the timer resource.  This will update the timer data.
                    // Input: AReg - Resource index.
                    //---------------------------------------------------------
                    KTMR_STOP_RESOURCE;

                    //---------------------------------------------------------
                    // Set the update repeat number display so that it would 
                    //   refresh the repeat number displayed on upper DM.
                    //---------------------------------------------------------
                    LReg = *(IYReg + TMRFLAGS2OFFSET);
                    LReg |= bTMRUpdRepeatNumDisplay;
                    *(IYReg + TMRFLAGS2OFFSET) = LReg;

                    // Turns off the LCD resource flags.
                    ctmrUpdateTurnOffResourceFlags();

                    // Cancel timer flag blinking.
                    CORE_CANCEL_BLINK_2HZ   TMR_FLAG;

                    // Cancels the current switch release to be passed coz it is 
                    //   not needed.
                    HW_KBD_CANCEL_CURRENT_SWITCH_RELEASE;

                    AUDSTART_SYSTEM_MELODY AUDSWBEEPMELODY, AUDNOMELODYDONEEVENT;

                    //---------------------------------------------------------
                    // Refresh the entire display.
                    // Ring events are enabled.
                    // Display update is disabled.
                    // Updates the primary icon resource
                    //---------------------------------------------------------
                    goto    tmrDefaultDisplayAll;


                case    KTMRSTATUSSTOPPED:
                    //=========================================================
                    //
                    //                  STOP STATE
                    //
                    //=========================================================

                    //---------------------------------------------------------
                    // Check if working HMS data is zero.
                    // Destroys nothing.
                    // Input:  AReg - Resource index
                    // Output: bZeroFlag = 0 if data is non zero.
                    //                   = 1 if data is zero.
                    //---------------------------------------------------------
                    KTMR_CHECK_WORK_HMS_IF_ZERO;
                    if( bZeroFlag == TRUE )
                    {
                        //-----------------------------------------------------
                        // Reset timer resource.
                        // Generate a button beep.
                        // Refresh the entire display.
                        //-----------------------------------------------------
                        tmrDefaultResetTimerResource();
                    }

                    else
                    {
                        //-----------------------------------------------------
                        // Set the type of display counter to 0.  This means 
                        //   that to display a large font data.
                        //-----------------------------------------------------
                        BReg = 0;
                        CTMR10SecondsCounter = BReg;

                        // Request a 2 second delay.
                        CORE_REQ_TIMEOUT_HIRES  CTMRHIRESTO_2SECONDS;

                        // Clear the entire display.
                        LCD_CLR_DISPLAY;

                        LCD_DISP_SMALL_DM_MSG_HOLD_TO_RESET;
                    }

                    break;

            }

            break;

        
        case    CTMR_MODEDEPRESS:
            //*****************************************************************
            //
            //                  MODE SWITCH DEPRESS EVENT
            //
            //*****************************************************************

            CORE_REQ_MODE_CHANGE_NEXT;

            break;


        case    CTMR_TMOUTDONEHIRES:
            //*****************************************************************
            //
            //              HIGH RESOLUTION TIMEOUT DONE EVENT
            //
            //*****************************************************************

tmrDefaultResetTimerResource:

            HW_KBD_CANCEL_CURRENT_SWITCH_RELEASE;

            AUDSTART_SYSTEM_MELODY AUDSWBEEPMELODY, AUDNOMELODYDONEEVENT;

            //-----------------------------------------------------------------
            // Reset the timer resource.
            // Destroys HLReg.
            // Input: AReg - Resource index.
            //-----------------------------------------------------------------
            AReg = CTMRActiveResource;
            KTMR_RESET_RESOURCE;

//-----------------------------------------------------------------------------
// 
// W A R N I N G !!! This is a fall through.  Do not rearrange.
//
//-----------------------------------------------------------------------------


        case    CTMR_STOPRESETRELEASE:
            //*****************************************************************
            //
            //              STOP/RESET SWITCH RELEASE EVENT
            //
            //*****************************************************************

            // Clears the entire LCD.
            LCD_CLR_DISPLAY;

            //-----------------------------------------------------------------
            // Refresh the entire display.
            // Ring events are enabled.
            // Display update is disabled.
            //-----------------------------------------------------------------
            goto    tmrDefaultDisplayAll;


        case    CTMR_CROWNSET:
            //*****************************************************************
            //
            //                      CROWN SET EVENT
            //
            //*****************************************************************

            BReg = CTMRSETBANNERSTATE;
            CORE_REQ_STATE_CHANGE;

            break;

    }

}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF   SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'tmrDefaultStateManager'"

        GLOBAL  tmrDefaultStateManager
        GLOBAL  tmrDefaultDisplayWorkHMS
        GLOBAL  tmrDefaultPopupEntry

tmrDefaultStateManager:                             ; **SUBROUTINE tmrDefaultStateManager
                
                ; Set IYReg to point to the ASD address.
                ld      IY, [CORECurrentASDAddress]

                ; Load the type of event to be processed.
                ld      A, [CORECurrentEvent]

                ; Check if display update event.
                cp      A, #CTMR_DISPUPDTMRRES
                jr      Z, tmrDefaultDispUpdEvent

                ; Check if CW edge event.
                cp      A, #CTMR_CWEDGE
                jr      Z, tmrDefaultCWEdgeEvent
                        
                ; Check if CCW edge event.
                cp      A, #CTMR_CCWEDGE
                jr      Z, tmrDefaultCCWEdgeEvent
                            
                ; Check if start/split depress event.
                cp      A, #CTMR_STARTSPLITDEPRESS
                jr      Z, tmrDefaultStartSplitDepressEvent

                ; Check if stop/reset depress event.
                cp      A, #CTMR_STOPRESETDEPRESS
                jr      Z, tmrDefaultStopResetDepressEvent

                ; Check if stop/reset release event.
                cp      A, #CTMR_STOPRESETRELEASE
                jr      Z, tmrDefaultStopResetReleaseEvent
                
                ; Check if timeout done hi res event.
                cp      A, #CTMR_TMOUTDONEHIRES
                jr      Z, tmrDefaultTmoutDoneHiResEvent

                ; Check if mode depress event.
                cp      A, #CTMR_MODEDEPRESS
                jr      Z, tmrDefaultModeDepressEvent

                ; Check if state entry event.
                cp      A, #CTMR_STATEENTRY
                jr      Z, tmrDefaultStateEntryEvent

                ; Check if crown set event.
                cp      A, #CTMR_CROWNSET
                jr      NZ, tmrDefaultStateExit
                
                ;**************************************************************
                ;
                ;                     CROWN SET EVENT
                ;
                ;**************************************************************

                ld      B, #CTMRSETBANNERSTATE
                CORE_REQ_STATE_CHANGE

tmrDefaultStateExit:

                ret



tmrDefaultDispUpdEvent:
tmrDefaultDisplayWorkHMS:
                ;**************************************************************
                ;
                ;                   DISPLAY UPDATE EVENT
                ;
                ;**************************************************************

                ;==============================================================
                ; The display update event will only be called when there is a
                ;   second update.
                ;==============================================================
                
                ;--------------------------------------------------------------
                ; Get the update bits.
                ; Destroys HLReg.
                ; Input:  AReg - Resource index.
                ; Output: BReg - Contains the bits of which data was updated.
                ;--------------------------------------------------------------
                ld      A, [CTMRActiveResource]
                KTMR_GET_UPDATE_BITS

                ;--------------------------------------------------------------
                ; Check if the 10 seconds timer has expired from the time it 
                ;   was started and is now preparing to display the data in 
                ;   large font.
                ;--------------------------------------------------------------
                ld      HL, #CTMR10SecondsCounter
                cp      [HL], #0
                jr      Z, tmrDefaultDisplayData
                
                ; Decrement counter.
                sub     [HL], #1

                ;--------------------------------------------------------------
                ; Check if the counter has just expired.  If yes, then clear 
                ;   the main DM and for the update bits to update the entire
                ;   display.
                ;--------------------------------------------------------------
                jr      NZ, tmrDefaultDisplayData

                ; Clear main DM.
                ; Destroys AReg, IXReg.
                LCD_CLEAR_MAIN_DM
                
                ; Set BReg to update the entire display.
                ld      B, #CTMRUPDATEHMSDATA


tmrDefaultDispWorkDataWithoutUpdBits:
tmrDefaultDisplayData:
                ;==============================================================
                ; BReg should be set to the update bits when using this as an
                ;   entry point.
                ;==============================================================

                ; Display new timer entry if a new timer has started.
                push    BA
                
                ld      HL, [CORECurrentASDAddress]
                add     HL, #TMRFLAGS2OFFSET
                bit     [HL], #bTMRNewTimerData
                jr      Z, tmrDefaultDisplayDataContinue

                and     [HL], #@LOW(~bTMRNewTimerData)

                ; Refresh the 10second counter display of data in DM line 1.
                ld      B, #10
                ld      [CTMR10SecondsCounter], B
                
                ; Display timer name.
                LCD_CLR_MAIN_DM_LINE2
                car     ctmrDisplayTimerNameAtLine2

tmrDefaultDisplayDataContinue:

                pop     BA


                ;--------------------------------------------------------------
                ; Check if repeat at end and repeat number display needs to be
                ;   updated before the display will commence.
                ; Destroys HLReg, IXReg
                ;--------------------------------------------------------------
                car     tmrChkAndDispRepeatNumber

                ;--------------------------------------------------------------
                ; Display working data to either BDM or L1.
                ; Input: CTMRActiveResource   - Resource index of the working data 
                ;                                 to be displayed.
                ;        CTMR10SecondsCounter - Counter to indicate whether to 
                ;                                 display large or small font.
                ;                               If 0, use large font.
                ;                               If not 0, use small font.
                ;        BReg.bKTMRSecondUpd  - Update seconds data.
                ;            .bKTMRMinuteUpd  - Update minute data.  
                ;            .bKTMRHourUpd    - Update hour data.
                ;
                ;--------------------------------------------------------------
                jr      ctmrDisplayWorkHMSAtBDMOrL1 ; **EXTERNAL JUMP
                

tmrDefaultCWEdgeEvent:
                ;**************************************************************
                ;
                ;                      CW EDGE EVENT
                ;
                ;**************************************************************

                ;--------------------------------------------------------------
                ; Reads the next DB record.
                ; Resets the active resource.
                ; Clears the entire screen.
                ; Destroys BAReg, HLReg.
                ; Assumptions : CORECurrentADDAddress - Absolute address of DB in external memory.
                ;               CTMRNumberOfRecords   - Number of records in DB.
                ;               CTMRLastViewedEntry   - Record number to be read.
                ;               CTMRActiveResource    - Resource to be reset.
                ;
                ; Output(s)   : AReg, CTMRLastViewedEntry- Record number that was read.
                ;               User set data is stored in:
                ;                   -> CTMRFlags         - Status flags.
                ;                   -> CTMRUserSetSecond - User set seconds data.
                ;                   -> CTMRUserSetMinute - User set minute data.
                ;                   -> CTMRUserSetHour   - User set hour data.
                ;                   -> CTMRNameX         - 9 char timer name, where x = 0 to 8
                ;--------------------------------------------------------------
                car     ctmrReadNextRecord
                          
                ; Save the last viewed entry.
                ld      [IY + TMRLASTVIEWEDENTRYOFFSET], A
                            
                ; Refresh the entire display.
                jr      tmrDefaultDisplayAll


tmrDefaultCCWEdgeEvent:
                ;**************************************************************
                ;
                ;                      CCW EDGE EVENT
                ;
                ;**************************************************************


                ;--------------------------------------------------------------
                ; Reads the previous DB record.
                ; Resets the active resource.
                ; Clears the entire screen.
                ; Destroys BAReg, HLReg.
                ; Assumptions : CORECurrentADDAddress - Absolute address of DB in external memory.
                ;               CTMRNumberOfRecords   - Number of records in DB.
                ;               CTMRLastViewedEntry   - Record number to be read.
                ;               CTMRActiveResource    - Resource to be reset.
                ;
                ; Output(s)   : AReg, CTMRLastViewedEntry- Record number that was read.
                ;               User set data is stored in:
                ;                   -> CTMRFlags         - Status flags.
                ;                   -> CTMRUserSetSecond - User set seconds data.
                ;                   -> CTMRUserSetMinute - User set minute data.
                ;                   -> CTMRUserSetHour   - User set hour data.
                ;                   -> CTMRNameX         - 9 char timer name, where x = 0 to 8
                ;--------------------------------------------------------------
                car     ctmrReadPrevRecord
                
                ; Save the last viewed entry.
                ld      [IY + TMRLASTVIEWEDENTRYOFFSET], A

                ; Refresh the entire display.
                jr      tmrDefaultDisplayAll


tmrDefaultStartSplitDepressEvent:
                ;**************************************************************
                ;
                ;              START/SPLIT SWITCH DEPRESS EVENT
                ;
                ;**************************************************************
                
                ; Cancel the current switch release event coz it is not needed.
                HW_KBD_CANCEL_CURRENT_SWITCH_RELEASE

                ;--------------------------------------------------------------
                ; Get the status of the timer.
                ; Destroys nothing.
                ; Input:  AReg - Resource index
                ; Output: BReg - Status of the timer.
                ;--------------------------------------------------------------
                ld      A, [CTMRActiveResource]
                KTMR_GET_STATUS

                ; Check if in reset state.
                cp      B, #KTMRSTATUSRUNNING
                jr      Z, tmrDefaultStateExit

                cp      B, #KTMRSTATUSSTOPPED
                jr      Z, tmrDefaultStartSplitDepressStopState

                ;==============================================================
                ;
                ;                       RESET STATE
                ;
                ;==============================================================
                
                ; Check if user set data is zero
                ld      HL, #CTMRFlags
                bit     [HL], #bCTMRDataIsZero
                jr      NZ, tmrDefaultStateExit

                ;--------------------------------------------------------------
                ; Set the type of display counter to 10.  This means that to 
                ;   display a small font for 10 seconds.
                ;--------------------------------------------------------------
                ld      B, #10
                ld      [CTMR10SecondsCounter], B

                ;--------------------------------------------------------------
                ; Save the status flag and last viewed entry from the common 
                ;   buffer to ASD.
                ;--------------------------------------------------------------
                ld      B, [CTMRFlags]
                ld      [IY + TMRFLAGS1OFFSET], B

                ;--------------------------------------------------------------
                ; Sets up the resources user set and working HMS data.
                ; Counting direction have already been set to countdown.
                ; Destroys nothing.
                ; Input: AReg  - Resource index.
                ;        IYReg - Points to the beginning of the HMS
                ;                  structure of the user RAM area.
                ;--------------------------------------------------------------
                ld      IY, #CTMRUserSetSecond
                KTMR_SETUP_USER_AND_WORK_HMS

                ;--------------------------------------------------------------
                ; Check if halfway reminder is enabled.  If yes, then set it up.
                ; Destroys BAReg, IYReg, IXReg, HLReg.
                ; Input: AReg - Resource index.
                ;        BReg - Contains the halfway reminder bit.
                ;        IYReg- Seconds address of the user set data.
                ;--------------------------------------------------------------
                ld      B, [CTMRFlags]
                car     ctmrSetupHalfwayReminder

                ;--------------------------------------------------------------
                ; Sets up the resource action at end.  
                ;   Stop At End   - Cancel all links. Clear repeat at end bit.
                ;   Repeat At End - Set repeat at end bit.  Set the repeat count 
                ;                     to 1.  Set the upd repeat number display.
                ;   Chrono At End - Create a link to STP.
                ; Destroys BAReg, IYReg, IXReg, HLReg.
                ;--------------------------------------------------------------
                car     tmrSetupActionAtEnd
                                
                ; Clear upper DM data in case "1/2" is displayed.
                LCD_CLEAR_UPPER_DM

                ;--------------------------------------------------------------
                ; Start the resource and refresh the display.
                ; Input: AReg - Resource index.
                ;--------------------------------------------------------------
                ld      A, [CTMRActiveResource]
                jr      tmrDefaultStartResourceCommon

                    
tmrDefaultStartSplitDepressStopState:
                ;==============================================================
                ;
                ;                       STOP STATE
                ;
                ;==============================================================

                ;--------------------------------------------------------------
                ; Check if working HMS data is zero.
                ; Destroys nothing.
                ; Input:  AReg - Resource index
                ; Output: bZeroFlag = 0 if data is non zero.
                ;                   = 1 if data is zero.
                ;--------------------------------------------------------------
                KTMR_CHECK_WORK_HMS_IF_ZERO
                jr      Z, tmrDefaultStateExit

tmrDefaultStartResourceCommon:
                ;==============================================================
                ; AReg should be set to the resource index when using this as 
                ;   an entry point.
                ;==============================================================

                ;--------------------------------------------------------------
                ; Starts the timer resource.
                ; Destroys nothing.
                ; Input: AReg - Resource index
                ;--------------------------------------------------------------
                KTMR_START_RESOURCE

                AUDSTART_SYSTEM_MELODY AUDSWBEEPMELODY, AUDNOMELODYDONEEVENT

                ;--------------------------------------------------------------
                ; Update LCD resource flags.
                ; Destroys BAReg, HLReg, IXReg.
                ; Input: BReg - Contains the flag values that contains
                ;                 the action at end.
                ;--------------------------------------------------------------
                ld      B, [CTMRFlags]
                car     ctmrUpdateResourceFlags

                ;--------------------------------------------------------------
                ; Refresh the entire display.
                ; Ring events are suspended.
                ; Request timer flag blinking.
                ; Display update is enabled.
                ; Update the primary icon resource.
                ;--------------------------------------------------------------
                jr      tmrDefaultDisplayAll


tmrDefaultStopResetDepressEvent:
                ;**************************************************************
                ;
                ;              STOP/RESET SWITCH DEPRESS EVENT
                ;
                ;**************************************************************

                ;--------------------------------------------------------------
                ; Get the status of the timer.
                ; Destroys nothing.
                ; Input:  AReg - Resource index
                ; Output: BReg - Status of the timer.
                ;--------------------------------------------------------------
                ld      A, [CTMRActiveResource]
                KTMR_GET_STATUS

                ; Check if in running state.
                cp      B, #KTMRSTATUSRUNNING
                jr      Z, tmrDefaultStopResetDepressRunningState
                
                cp      B, #KTMRSTATUSSTOPPED
                jr      Z, tmrDefaultStopResetDepressStopState

                ;==============================================================
                ;
                ;                       RESET STATE
                ;
                ;==============================================================

                ; Cancel the current switch release to be passed coz it is
                ;   not needed.
                HW_KBD_CANCEL_CURRENT_SWITCH_RELEASE

                ; Check if data is less than 1 minute, if not then toggle
                ;   halfway reminder bit.
                ld      HL, #CTMRFlags
                bit     [HL], #bCTMRLess1Min
                jr      NZ, tmrDefaultStateExit 
                
                ; Toggle the halfway reminder bit.
                xor     [HL], #bCTMRHalfwayEnable

                ;--------------------------------------------------------------
                ; Display "1/2" at upper DM if halfway reminder is enabled.
                ; The halfway reminder bit should be in CTMRFlags.bCTMRHalfwayEnable
                ; Destroys BAReg, HLReg, IXReg.
                ;--------------------------------------------------------------
                car     ctmrDisplayOneHalfAtSDM
                    
                ;-------------------------------------------------------------
                ; Write the status flag to the EEPROM.
                ; Destroys BAReg, HLReg.
                ; Input: AReg  - Record number to write to.
                ;        HLReg - Absolute address of DB file in external memory.
                ;        CTMRLastViewedEntry - Record number to write to.
                ;-------------------------------------------------------------
                ld      HL, [CORECurrentADDAddress]
                jr      ctmrWriteStatusFlagToEEPROM ; **EXTERNAL JUMP

                
tmrDefaultStopResetDepressRunningState:
                ;==============================================================
                ;
                ;                       RUNNING STATE
                ;
                ;==============================================================

                ;--------------------------------------------------------------
                ; Stops the timer resource.  This will update the timer data.
                ; Input: AReg - Resource index.
                ;--------------------------------------------------------------
                KTMR_STOP_RESOURCE

                ;--------------------------------------------------------------
                ; Set the update repeat number display so that it would 
                ;   refresh the repeat number displayed on upper DM.
                ;--------------------------------------------------------------
                ld      L, [IY + TMRFLAGS2OFFSET]
                or      L, #bTMRUpdRepeatNumDisplay
                ld      [IY + TMRFLAGS2OFFSET], L

                ; Turns off the LCD resource flags.
                car     ctmrUpdateTurnOffResourceFlags

                ; Cancel timer flag blinking.
                CORE_CANCEL_BLINK_2HZ   TMR_FLAG
                
                ; Cancels the current switch release to be passed coz it is 
                ;   not needed.
                HW_KBD_CANCEL_CURRENT_SWITCH_RELEASE

                AUDSTART_SYSTEM_MELODY AUDSWBEEPMELODY, AUDNOMELODYDONEEVENT
                
                ;--------------------------------------------------------------
                ; Refresh the entire display.
                ; Ring events are enabled.
                ; Display update is disabled.
                ; Updates the primary icon resource
                ;--------------------------------------------------------------
                jr      tmrDefaultDisplayAll


tmrDefaultStopResetDepressStopState:
                ;==============================================================
                ;
                ;                      STOP STATE
                ;
                ;==============================================================

                ;--------------------------------------------------------------
                ; Check if working HMS data is zero.
                ; Destroys nothing.
                ; Input:  AReg - Resource index
                ; Output: bZeroFlag = 0 if data is non zero.
                ;                   = 1 if data is zero.
                ;--------------------------------------------------------------
                KTMR_CHECK_WORK_HMS_IF_ZERO

                ;--------------------------------------------------------------
                ; Reset timer resource.
                ; Generate a button beep.
                ; Refresh the entire display.
                ;--------------------------------------------------------------
                jr      Z, tmrDefaultResetTimerResource
                                
                ;--------------------------------------------------------------
                ; Set the type of display counter to 0.  This means that to 
                ;   display a large font data.
                ;--------------------------------------------------------------
                ld      B, #0
                ld      [CTMR10SecondsCounter], B

                ; Request a 2 second delay.
                CORE_REQ_TIMEOUT_HIRES  CTMRHIRESTO_2SECONDS
                
                ; Clear the entire display.
                LCD_CLR_DISPLAY
                
                LCD_DISP_SMALL_DM_MSG_HOLD_TO_RESET

                ret


tmrDefaultTmoutDoneHiResEvent:
tmrDefaultResetTimerResource:
                ;**************************************************************
                ;
                ;               HIGH RESOLUTION TIMEOUT DONE EVENT
                ;
                ;**************************************************************

                HW_KBD_CANCEL_CURRENT_SWITCH_RELEASE

                AUDSTART_SYSTEM_MELODY AUDSWBEEPMELODY, AUDNOMELODYDONEEVENT

                ;--------------------------------------------------------------
                ; Reset the timer resource.
                ; Destroys HLReg.
                ; Input: AReg - Resource index.
                ;--------------------------------------------------------------
                ld      A, [CTMRActiveResource]
                KTMR_RESET_RESOURCE

;------------------------------------------------------------------------------
; 
; W A R N I N G !!! This is a fall through.  Do not rearrange.
;
;------------------------------------------------------------------------------

tmrDefaultStopResetReleaseEvent:
                ;**************************************************************
                ;
                ;               STOP/RESET SWITCH RELEASE EVENT
                ;
                ;**************************************************************

                ; Clears the entire LCD.
                LCD_CLR_DISPLAY

                ;--------------------------------------------------------------
                ; Refresh the entire display.
                ; Ring events are enabled.
                ; Display update is disabled.
                ;--------------------------------------------------------------
                jr      tmrDefaultDisplayAll


tmrDefaultModeDepressEvent:
                ;**************************************************************
                ;
                ;                  MODE SWITCH DEPRESS EVENT
                ;
                ;**************************************************************

                CORE_REQ_MODE_CHANGE_NEXT

                ret

tmrDefaultStateEntryEvent:
                ;**************************************************************
                ;
                ;                       STATE ENTRY EVENT
                ;
                ;**************************************************************

                CORE_ENABLE_SWITCH_RELEASE

                ;--------------------------------------------------------------
                ; Check if it came from a countdown popup entry.  if yes, then
                ;   don't reset the counter.
                ;--------------------------------------------------------------
                ld      HL, [CORECurrentASDAddress]
                add     HL, #TMRFLAGS2OFFSET
                bit     [HL], #bTMRCameFromPopup
                jr      NZ, tmrDefaultStateEntryEventContinue

                ld      B, #0
                ld      [CTMR10SecondsCounter], B

                jr      tmrDefaultPopupEntry

tmrDefaultStateEntryEventContinue:

                and     [HL], #@LOW(~bTMRCameFromPopup)

                ld      A, [CTMR10SecondsCounter]
                cp      A, #0
                jr      Z, tmrDefaultPopupEntry

                car     ctmrDisplayTimerNameAtLine2

tmrDefaultPopupEntry:

                ;--------------------------------------------------------------
                ; Copies the ASD data to a common foreground buffer to be used 
                ;    for foreground processing.
                ; Copies the entry specified in last viewed entry to a common 
                ;    foreground buffer.
                ;--------------------------------------------------------------
                push    IY
                car     tmrCopyDataToCommonBuffer
                pop     IY
                                
tmrDefaultDisplayAll:
                ;--------------------------------------------------------------
                ; Set the update repeat number display so that it would 
                ;   refresh the repeat number displayed on upper DM.
                ;--------------------------------------------------------------
                ld      HL, [CORECurrentASDAddress]
                add     HL, #TMRFLAGS2OFFSET
                or      [HL], #bTMRUpdRepeatNumDisplay

                ;--------------------------------------------------------------
                ; Enable ring events to be passed to this state assuming that 
                ;   timer is in reset or stop state so that we can tranverse 
                ;   other entries.
                ; Destroys HLReg.
                ;--------------------------------------------------------------
                CORE_ENABLE_RING_EVENTS
                
                ;--------------------------------------------------------------
                ; Disable display update event to be passed to this state 
                ;   assuming that timer is in reset or stop state.
                ; Destroys BReg.
                ; Input: AReg - Resource index.
                ;--------------------------------------------------------------
                ld      A, [CTMRActiveResource]
                KTMR_DISABLE_DISP_UPD_EVENT

                ;--------------------------------------------------------------
                ; Displays the action at end flags.
                ; Destroys HLReg, IXReg.
                ; Input: BReg - Contains the flag values that contains the 
                ;                 action at end.
                ;--------------------------------------------------------------
                ld      B, [CTMRFlags]
                car     ctmrDisplayActionAtEndFlags

                ;--------------------------------------------------------------
                ; Get the status of the timer.
                ; Destroys nothing.
                ; Input:  AReg - Resource index
                ; Output: BReg - Status of the timer.
                ;--------------------------------------------------------------
                KTMR_GET_STATUS

                ; Check the status of the timer and process the procedures accordingly.
                cp      B, #KTMRSTATUSRESET
                jr      Z, tmrDefaultStateEntryResetState

                cp      B, #KTMRSTATUSSTOPPED
                jr      Z, tmrDefaultStateEntryStopState

                ;==============================================================
                ;
                ;                      RUNNING STATE
                ;
                ;==============================================================

                ;--------------------------------------------------------------
                ; Enable display update events to be passed to this state.
                ; Destroys BReg.
                ; Input: AReg - Resource index
                ;--------------------------------------------------------------
                KTMR_ENABLE_DISP_UPD_EVENT

                ;--------------------------------------------------------------
                ; Suspend the ring events if timer is running so that the ring 
                ;   events would not be passed to the application.  Suspending 
                ;   the ring events would disable traversing to other entries.
                ; Destroys HLReg.
                ;--------------------------------------------------------------
                CORE_SUSPEND_RING_EVENTS

                ;--------------------------------------------------------------
                ; Request timer flag to blink.
                ; Destroys BAReg.
                ;--------------------------------------------------------------
                CORE_REQ_BLINK_2HZ  TMR_FLAG

;------------------------------------------------------------------------------
;
; W A R N I N G !!! This is a fall through.  Do not rearrange.
;
;------------------------------------------------------------------------------

tmrDefaultStateEntryStopState:
                ;==============================================================
                ;
                ;                       STOP STATE
                ;
                ;==============================================================

                ;--------------------------------------------------------------
                ; Display user set HMS at 9 segment.
                ; Destroys BAReg, HLReg, IXReg, IYReg.
                ;--------------------------------------------------------------
                car     ctmrDisplayUserSetHMSAtSegment

                ;--------------------------------------------------------------
                ; Displays working HMS and repeat number.
                ; Input: BReg - Contains the bits of the data to be displayed.
                ;--------------------------------------------------------------
                ld      B, #CTMRUPDATEHMSDATA
                jr      tmrDefaultDispWorkDataWithoutUpdBits


tmrDefaultStateEntryResetState:
                ;==============================================================
                ;
                ;                       RESET STATE
                ;
                ;==============================================================

                ;--------------------------------------------------------------
                ; Display "1/2" at upper DM if halfway reminder is enabled.
                ; The halfway reminder bit should be in CTMRFlags.bCTMRHalfwayEnable
                ; Destroys BAReg, HLReg, IXReg.
                ;--------------------------------------------------------------
                car     ctmrDisplayOneHalfAtSDM

                ;--------------------------------------------------------------
                ; Displays timer name at line 2.
                ; The name should be on the CTMRNameX where X = 0 to 8
                ; Destroys BReg, LReg, IXReg.
                ;--------------------------------------------------------------
                car     ctmrDisplayTimerNameAtLine2 

                ;--------------------------------------------------------------
                ; Copies the user set data to work buffer.
                ; Destroys HLReg.
                ;--------------------------------------------------------------
                car     ctmrCopyUserSetToWorkBuffer
                
                ;--------------------------------------------------------------
                ; Displays the user set data on line 1.
                ; The user set data should be in CTMRWorkSecond, CTMRWorkMinute
                ;   and CTMRWorkHour.
                ; Destroys BAReg, HLReg, IXReg, IYReg.
                ; Input: BReg - Contains the bits of the data to be displayed.
                ;--------------------------------------------------------------
                ld      B, #CTMRUPDATEHMSDATA
                jr      ctmrDisplayWorkHMSAtLine1   ; **EXTERNAL JUMP
                

                                                    ; **END SUBROUTINE tmrDefaultStateManager


        CALLS   SUBROUTINE, 'tmrChkAndDispRepeatNumber'
        CALLS   SUBROUTINE, 'ctmrReadOneRecord'
        CALLS   SUBROUTINE, 'ctmrSetupHalfwayReminder'
        CALLS   SUBROUTINE, 'tmrSetupActionAtEnd'
        CALLS   SUBROUTINE, 'ctmrDisplayOneHalfAtSDM'
        CALLS   SUBROUTINE, 'tmrCopyDataToCommonBuffer'
        CALLS   SUBROUTINE, 'ctmrDisplayActionAtEndFlags'
        CALLS   SUBROUTINE, 'ctmrDisplayUserSetHMSAtSegment'
        CALLS   SUBROUTINE, 'ctmrDisplayOneHalfAtSDM'
        CALLS   SUBROUTINE, 'ctmrDisplayTimerNameAtLine2'
        CALLS   SUBROUTINE, 'ctmrCopyUserSetToWorkBuffer'
        CALLS   SUBROUTINE, 'ctmrReadNextRecord'
        CALLS   SUBROUTINE, 'ctmrReadPrevRecord'
        CALLS   SUBROUTINE, 'ctmrUpdateResourceFlags'
        CALLS   SUBROUTINE, 'ctmrUpdateTurnOffResourceFlags'

;**END FILE tmrdef.asm

        
        END

















