;**FILE tmrpopup.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2000 - 2002 TIMEX Corporation.
; 
; This document is the property of, and contains information proprietary to
; TIMEX Corporation.  It is to be returned upon request.  Reproduction by any
; means, disclosure, or use of its contents for other than the expressed
; purpose for which it is loaned, without the written consent of TIMEX
; Corporation, is strictly prohibited.
;
; File Name    : tmrpopup.asm
;
; Module Names : tmrPopupStateManager
;                   
; Purpose      : Countdown Timer Application Popup State Manager
;
; File Change History:
;
;                12/17/2001  - Created tmrpopup.asm for m851
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

$CASE ON
$LIST OFF

        INCLUDE '..\..\CORE\H\MODEL.H'
        INCLUDE '..\..\HW\H\HW88349.H'
        INCLUDE '..\..\HW\H\HWI2CEE.H'
        INCLUDE '..\..\HW\H\HWBOND.H'
        INCLUDE '..\..\HW\H\HWKBD.H'
        INCLUDE '..\..\HW\H\HWMACRO.H'
        INCLUDE '..\..\HW\H\HWMAIN.H'
        INCLUDE '..\..\HW\H\HWRESET.H'
        INCLUDE '..\..\HW\H\HWVARS.H'
        INCLUDE '..\..\HW\H\HWSERMAC.H'
        INCLUDE '..\..\CORE\H\CORE.H'
        INCLUDE '..\..\CORE\H\COREMAC.H'
        INCLUDE '..\..\CORE\H\CORESVMC.H'
        INCLUDE '..\..\COMM\H\COMM.H'
        INCLUDE '..\..\LCD\H\LCD.H'
        INCLUDE '..\..\LCD\H\LCDMACRO.H'
        INCLUDE '..\..\LCD\H\LCDRAM.H'
        INCLUDE '..\..\DATABASE\H\DB.H'
        INCLUDE '..\..\DATABASE\H\DBMACRO.H'
        INCLUDE '..\..\AUDIO\H\AUD.H'
        INCLUDE '..\..\AUDIO\H\AUDMAC.H'
        INCLUDE '..\..\UTIL\H\UTLMATH.H'
        INCLUDE '..\..\UTIL\H\UTLMTHMC.H'
        INCLUDE '..\..\UTIL\H\UTLEDT.H'
        INCLUDE '..\..\UTIL\H\UTLEDTMC.H'
        INCLUDE '..\..\UTIL\H\UTLDSP.H'
        INCLUDE '..\..\UTIL\H\UTLDSPMC.H'
        INCLUDE '..\..\UTIL\H\UTLACCMC.H'
        INCLUDE '..\..\UTIL\H\UTLMSC.H'
        INCLUDE '..\..\UTIL\H\UTLMSCMC.H'
        INCLUDE '..\..\UTIL\H\UTLVARS.H'
        INCLUDE '..\..\KRES\H\KRES.H'
        INCLUDE '..\..\KRES\H\KRESMAC.H'
        INCLUDE '..\..\KRES\H\KSTP.H'
        INCLUDE '..\..\KRES\H\KSTPMAC.H'
        INCLUDE '..\..\KRES\H\KSYN.H'
        INCLUDE '..\..\KRES\H\KSYNMAC.H'
        INCLUDE '..\..\KRES\H\KTMR.H'
        INCLUDE '..\..\KRES\H\KTMRMAC.H'
        INCLUDE '..\..\KRES\H\KTOD.H'
        INCLUDE '..\..\KRES\H\KTODMAC.H'
        INCLUDE '..\..\KRES\H\KTZC.H'
        INCLUDE '..\..\KRES\H\KTZCMAC.H'
        INCLUDE '..\..\KRES\H\KBCK.H'
        INCLUDE '..\..\KRES\H\KBCKMAC.H'
        INCLUDE '..\..\USAGE\H\USG.H'
        INCLUDE '..\..\USAGE\H\USGVARS.H'
        INCLUDE '..\..\USAGE\H\USGMAC.H'
        INCLUDE '..\..\CORE\H\SECTION.H'

        INCLUDE '..\..\TOD\H\TOD.H'
        INCLUDE '..\..\TOD\H\TODVARS.H'
        
        INCLUDE '..\..\CTMR\H\CTMR.H'
        INCLUDE '..\..\CTMR\H\CTMRVARS.H'

        INCLUDE '..\..\TMR\H\TMRVARS.H'

$LIST ON

        SECT    ".text_tmr"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : tmrPopupStateManager
;
; Description : Timer Application Popup State Manager
;
; Assumptions : Display is cleared on first time entry into the state
;
; Input(s)    : CORECurrentEvent    - system event to be processed
;               COREEventArgument   - event extra information
;
; Output(s)   : None
;
; Module Change History:
;
;               12/17/2001  - Created module for M851
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void tmrPopupStateManager()
{

    IYReg = CORECurrentASDAddress;

    switch( CORECurrentEvent )
    {

        case    CTMR_STATEENTRY:
            //*****************************************************************
            //
            //                      STATE ENTRY EVENT
            //
            //*****************************************************************

            // Check the type of action at end to be processed.
            AReg = *(IYReg + TMRPOPUPTYPEOFFSET);

            push    (BAReg);

            switch( AReg )
            {
                case    CTMRSTOPATENDALERT:
                    //=========================================================
                    //
                    //                  STOP AT END
                    //
                    //=========================================================
                    
                    // Turn off timer related resource flags.
                    ctmrUpdateTurnOffResourceFlags();

                    //---------------------------------------------------------
                    // Clear the variable that will be used as counter to 
                    //   determine if timer has already exceeded the 10 
                    //   seconds countdown.  This will also determine if to 
                    //   display large or small font.
                    //---------------------------------------------------------
                    CTMR10SecondsCounter = 0;

                    USG_INCREMENT_EEPROM_COUNTERS   USGTMRPOPUPSTOPATEND;

                    break;

                case    CTMRREPEATATENDALERT:
                    //=========================================================
                    //
                    //                  REPEAT AT END
                    //
                    //=========================================================

                    // Indicates that we are in a popup and action is repeat at
                    //   end.
                    AReg = *(IYReg + TMRFLAGS2OFFSET);
                    AReg |= bTMRCameFromPopup;
                    *(IYReg + TMRFLAGS2OFFSET) = AReg;

                    //---------------------------------------------------------
                    // Set the type of display counter to 10.  This means that
                    //   to display a small font for 10 seconds.
                    //---------------------------------------------------------
                    CTMR10SecondsCounter = 10;

                    USG_INCREMENT_EEPROM_COUNTERS   USGTMRPOPUPREPEATATEND;

                    break;

            }            

            // Stop current melody if there is any.
            AUDSTOP_MELODY;

            // Alternate the lamp and buzzer during the duration of the popup.
            HW_LAMP_POPUP_REQUEST;

            // Request timer alert and melody done event to be passed.
            AUDSTART_SYSTEM_MELODY AUDTIMERMELODY, AUDSENDMELODYDONEEVENT

            // Request the system to pass a generic switch depress event for 
            //   application processing.
            CORE_ENABLE_GENERIC_SWITCH_EVENT;

            // Request the system to pass a generic event for application
            //   processing while the melody is still active.
            CORE_REQUEST_MELODY_POPUP_CANCEL;

            //-----------------------------------------------------------------
            // Refresh the entire display.
            // Display action at end flags.
            // Display user set data at 9 segment.
            // If action is stop at end:
            //  - Ring events are enabled.
            //  - Display update event is disabled.
            //  - Display timer complete data using large font.
            // If action is repeat at end.
            //  - Ring events are disabled.
            //  - Display update event is enabled.
            //  - Request TMR flag blinking.
            //  - Display timer data at line 1.
            //  - Display repeat number.
            //-----------------------------------------------------------------
            tmrDefaultPopupEntry();

            // Enable ring events coz it is disabled when action is repeat at
            //   end.
            CORE_ENABLE_RING_EVENTS;

            pop     (BAReg);

            // Check if repeat at end.  If yes, then display timer name at 
            //   line 2.  The variables are already setup upon calling the 
            //   'tmrDefaultPopupEntry'.
            if( AReg == CTMRREPEATATENDALERT )
            {
                //-------------------------------------------------------------
                // Displays timer name at line 2.
                // The name should be on the CTMRNameX where X = 0 to 8
                // Destroys BReg, LReg, IXReg.
                //-------------------------------------------------------------
                ctmrDisplayTimerNameAtLine2();
            }

            break;          


        case    CTMR_POPUPCANCEL:
            //*****************************************************************
            // 
            //                      POPUP CANCEL EVENT
            //
            //*****************************************************************
            
            // Check if the pop up cancel event was caused by pulling the crown
            //   to set position.
            if( COREEventArgument == CTMR_CROWNSET )
            {
                BReg = CTMRUNUSEDPOPUPSTATE;
                CORE_REQ_STATE_CHANGE;
            }

            else
            {
                goto    tmrPopupMelodyDoneEvent;
            }

            break;

        case    CTMR_MELODYDONE:
            //*****************************************************************
            // 
            //                      MELODY DONE EVENT
            //
            //*****************************************************************

tmrPopupMelodyDoneEvent:

            // Check if the interrupted mode is the popup mode.
            if( CORECurrentMode == COREPopupdownMode )
            {
                // Yup.  Return to its default state.
                CORE_REQ_POPDOWN;
            }
            
            else
            {

                // Clear the bit to indicate that countdown timer is not the current
                //   mode.
                AReg = *(IYReg + TMRFLAGS2OFFSET);
                AReg &= ~bTMRCameFromPopup;
                *(IYReg + TMRFLAGS2OFFSET) = AReg;

                CORE_REQ_TIMEOUT_HIRES  CTMRHIRESTO_3SECONDS;
            }

            break;


        case    CTMR_CWEDGE:
        case    CTMR_CCWEDGE:
        case    CTMR_TMOUTDONEHIRES:
        case    CTMR_ANYSWITCHDEPRESS:
            //*****************************************************************
            // 
            //                      CW EDGE EVENT
            //                      CCW EDGE EVENT
            //               HIGH RESOLUTION TIMEOUT DONE EVENT
            //                  ANY SWITCH DEPRESSION EVENT
            //
            //*****************************************************************

            CORE_REQ_POPDOWN;

            break;


        case    CTMR_DISPUPDTMRRES:
            //*****************************************************************
            //
            //                      DISPLAY UPDATE EVENT
            //
            //*****************************************************************

            // Displays timer data.
            tmrDefaultDisplayWorkHMS();

            break;


        case    CTMR_CROWNSET:
            //*****************************************************************
            //
            //                          CROWN EVENT
            //
            //*****************************************************************

            BReg = CTMRUNUSEDPOPUPSTATE;
            CORE_REQ_STATE_CHANGE;

            break;
    
    }
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF   SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'tmrPopupStateManager'"

        GLOBAL  tmrPopupStateManager


tmrPopupStateManager:                               ; **SUBROUTINE tmrPopupStateManager
                
                ld      IY, [CORECurrentASDAddress]

                ; Load the event to be processed.
                ld      A, [CORECurrentEvent]

                ; Check if display update event.
                cp      A, #CTMR_DISPUPDTMRRES
                jr      Z, tmrDefaultDisplayWorkHMS ; **EXTERNAL JUMP

                ; Check if popup cancel event.
                cp      A, #CTMR_POPUPCANCEL
                jr      Z, tmrPopupCancelEvent

                ; Check if melody done event.
                cp      A, #CTMR_MELODYDONE
                jr      Z, tmrPopupMelodyDoneEvent

                ; Check if hi-res timeout done event.
                cp      A, #CTMR_TMOUTDONEHIRES
                jr      Z, tmrPopupHiResTmoutDoneEvent

                ; Check if any switch depress event.
                cp      A, #CTMR_ANYSWITCHDEPRESS
                jr      Z, tmrPopupAnySwitchDepressEvent

                ; Check CW edge event.
                cp      A, #CTMR_CWEDGE
                jr      Z, tmrPopupCWEdgeEvent
                        
                ; Check CCW edge event.
                cp      A, #CTMR_CCWEDGE
                jr      Z, tmrPopupCCWEdgeEvent

                ; Check if state entry event.
                cp      A, #CTMR_STATEENTRY
                jr      Z, tmrPopupStateEntryEvent

                ; Check if crown set event.
                cp      A, #CTMR_CROWNSET
                jr      Z, tmrPopupCrownSetEvent
                

tmrPopupStateExit:

                ret


tmrPopupCancelEvent:
                ;**************************************************************
                ; 
                ;                   POPUP CANCEL EVENT
                ;
                ;**************************************************************
                            
                ; Check if the pop up cancel event was caused by pulling the 
                ;   crown to set position.
                ld      A, [COREEventArgument]
                cp      A, #CTMR_CROWNSET
                jr      NZ, tmrPopupMelodyDoneEvent

;------------------------------------------------------------------------------
;
; W A R N I N G !!! This is a fall through.  Do not rearrange.
;
;------------------------------------------------------------------------------

tmrPopupCrownSetEvent:
                ;**************************************************************
                ;
                ;                      CROWN EVENT
                ;
                ;**************************************************************

                ld      B, #CTMRUNUSEDPOPUPSTATE
                CORE_REQ_STATE_CHANGE

                ret

tmrPopupMelodyDoneEvent:
                ;**************************************************************
                ; 
                ;                   MELODY DONE EVENT
                ;
                ;**************************************************************
                
                ; Check if the interrupted mode is the popup mode.
                ld      A, [CORECurrentMode]
                cp      A, [COREPopupdownMode]
                jr      NZ, tmrPopupMelodyDoneReqTmout
                
;------------------------------------------------------------------------------
;
; W A R N I N G !!! This is a fall through.  Do not rearrange.
;
;------------------------------------------------------------------------------

tmrPopupCWEdgeEvent:
tmrPopupCCWEdgeEvent:
tmrPopupHiResTmoutDoneEvent:
tmrPopupAnySwitchDepressEvent:
                ;**************************************************************
                ; 
                ;                       CW EDGE EVENT
                ;                       CCW EDGE EVENT
                ;              HIGH RESOLUTION TIMEOUT DONE EVENT
                ;                 ANY SWITCH DEPRESSION EVENT
                ;
                ;**************************************************************
                    
                ; Return to its default state.
                CORE_REQ_POPDOWN

                ret
            
tmrPopupMelodyDoneReqTmout:
                
                ; Clear the bit to indicate that countdown timer is not the current
                ;   mode.
                ld      A, [IY + TMRFLAGS2OFFSET]
                and     A, #@LOW(~bTMRCameFromPopup)
                ld      [IY + TMRFLAGS2OFFSET], A

                CORE_REQ_TIMEOUT_HIRES  CTMRHIRESTO_3SECONDS

                ret


               
tmrPopupStateEntryEvent:
                ;**************************************************************
                ;
                ;                      STATE ENTRY EVENT
                ;
                ;**************************************************************
                
                ; Check the type of action at end to be processed.
                ld      A, [IY + TMRPOPUPTYPEOFFSET]

                push    BA

                cp      A, #CTMRREPEATATENDALERT
                jr      Z, tmrPopupStateEntryRepeatAtEnd

                ;==============================================================
                ;
                ;                       STOP AT END
                ; 
                ;==============================================================
                            
                ; Turn off timer related resource flags.
                car     ctmrUpdateTurnOffResourceFlags

                ;--------------------------------------------------------------
                ; Clear the variable that will be used as counter to 
                ;   determine if timer has already exceeded the 10 
                ;   seconds countdown.  This will also determine if to 
                ;   display large or small font.
                ;--------------------------------------------------------------
                ld      B, #0
                ld      [CTMR10SecondsCounter], B

                USG_INCREMENT_EEPROM_COUNTERS   USGTMRPOPUPSTOPATEND

                jr      tmrPopupStateEntryCommonProcessing

tmrPopupStateEntryRepeatAtEnd:
                ;==============================================================
                ;
                ;                       REPEAT AT END
                ;
                ;==============================================================
                
                ; Indicates that we are in a popup and action is repeat at end.
                ld      A, [IY + TMRFLAGS2OFFSET]
                or      A, #bTMRCameFromPopup
                ld      [IY + TMRFLAGS2OFFSET], A

                ;--------------------------------------------------------------
                ; Set the type of display counter to 10.  This means that
                ;   to display a small font for 10 seconds.
                ;--------------------------------------------------------------
                ld      B, #10
                ld      [CTMR10SecondsCounter], B

                USG_INCREMENT_EEPROM_COUNTERS   USGTMRPOPUPREPEATATEND


tmrPopupStateEntryCommonProcessing:
                
                ; Stop current melody if there is any.
                AUDSTOP_MELODY

                ; Alternate the lamp and buzzer during the duration of the popup.
                HW_LAMP_POPUP_REQUEST
                
                ; Request timer alert and melody done event to be passed.
                AUDSTART_SYSTEM_MELODY AUDTIMERMELODY, AUDSENDMELODYDONEEVENT

                ; Request the system to pass a generic switch depress event for 
                ;   application processing.
                CORE_ENABLE_GENERIC_SWITCH_EVENT
                
                ; Request the system to pass a generic event for application
                ;   processing while the melody is still active.
                CORE_REQUEST_MELODY_POPUP_CANCEL

                ;--------------------------------------------------------------
                ; Refresh the entire display.
                ; Display action at end flags.
                ; Display user set data at 9 segment.
                ; If action is stop at end:
                ;  - Ring events are enabled.
                ;  - Display update event is disabled.
                ;  - Display timer complete data using large font.
                ; If action is repeat at end.
                ;  - Ring events are disabled.
                ;  - Display update event is enabled.
                ;  - Request TMR flag blinking.
                ;  - Display timer data at line 1.
                ;  - Display repeat number.
                ;--------------------------------------------------------------
                car     tmrDefaultPopupEntry

                ; Enable ring events coz it is disabled when action is repeat at
                ;   end.
                CORE_ENABLE_RING_EVENTS

                pop     BA

                ; Check if repeat at end.  If yes, then display timer name at 
                ;   line 2.  The variables are already setup upon calling the 
                ;   'tmrDefaultPopupEntry'.
                cp      A, #CTMRREPEATATENDALERT
                jr      NZ, tmrPopupStateExit
                
                ;--------------------------------------------------------------
                ; Displays timer name at line 2.
                ; The name should be on the CTMRNameX where X = 0 to 8
                ; Destroys BReg, LReg, IXReg.
                ;--------------------------------------------------------------
                jr      ctmrDisplayTimerNameAtLine2 ; **EXTERNAL JUMP
                                
                        
                


                                                    ; **END SUBROUTINE tmrPopupStateManager


        CALLS   SUBROUTINE, 'tmrDefaultPopupEntry'
        CALLS   SUBROUTINE, 'ctmrUpdateTurnOffResourceFlags'



;**END FILE tmrpopup.asm


        END
