;**FILE tmrpor.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2000 - 2002 TIMEX Corporation.
; 
; This document is the property of, and contains information proprietary to
; TIMEX Corporation.  It is to be returned upon request.  Reproduction by any
; means, disclosure, or use of its contents for other than the expressed
; purpose for which it is loaned, without the written consent of TIMEX
; Corporation, is strictly prohibited.
;
; File Name   : tmrpor.asm
;
; Purpose     : Countdown Timer application POR initialization parameters.
;
; File Change History:
;
;               08/03/2000  - Created tmrpor.asm for M841
;
;   M851 Development:
;
;               12/11/2001  - Modified for M851    
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

$CASE ON
$LIST OFF

        INCLUDE '..\..\CORE\H\MODEL.H'
        INCLUDE '..\..\HW\H\HW88349.H'
        INCLUDE '..\..\HW\H\HWI2CEE.H'
        INCLUDE '..\..\HW\H\HWBOND.H'
        INCLUDE '..\..\HW\H\HWKBD.H'
        INCLUDE '..\..\HW\H\HWMACRO.H'
        INCLUDE '..\..\HW\H\HWMAIN.H'
        INCLUDE '..\..\HW\H\HWRESET.H'
        INCLUDE '..\..\HW\H\HWVARS.H'
        INCLUDE '..\..\HW\H\HWSERMAC.H'
        INCLUDE '..\..\CORE\H\CORE.H'
        INCLUDE '..\..\CORE\H\COREMAC.H'
        INCLUDE '..\..\CORE\H\CORESVMC.H'
        INCLUDE '..\..\COMM\H\COMM.H'
        INCLUDE '..\..\LCD\H\LCD.H'
        INCLUDE '..\..\LCD\H\LCDMACRO.H'
        INCLUDE '..\..\LCD\H\LCDRAM.H'
        INCLUDE '..\..\DATABASE\H\DB.H'
        INCLUDE '..\..\DATABASE\H\DBMACRO.H'
        INCLUDE '..\..\AUDIO\H\AUD.H'
        INCLUDE '..\..\AUDIO\H\AUDMAC.H'
        INCLUDE '..\..\UTIL\H\UTLMATH.H'
        INCLUDE '..\..\UTIL\H\UTLMTHMC.H'
        INCLUDE '..\..\UTIL\H\UTLEDT.H'
        INCLUDE '..\..\UTIL\H\UTLEDTMC.H'
        INCLUDE '..\..\UTIL\H\UTLDSP.H'
        INCLUDE '..\..\UTIL\H\UTLDSPMC.H'
        INCLUDE '..\..\UTIL\H\UTLACCMC.H'
        INCLUDE '..\..\UTIL\H\UTLMSC.H'
        INCLUDE '..\..\UTIL\H\UTLMSCMC.H'
        INCLUDE '..\..\UTIL\H\UTLVARS.H'
        INCLUDE '..\..\KRES\H\KRES.H'
        INCLUDE '..\..\KRES\H\KRESMAC.H'
        INCLUDE '..\..\KRES\H\KSTP.H'
        INCLUDE '..\..\KRES\H\KSTPMAC.H'
        INCLUDE '..\..\KRES\H\KSYN.H'
        INCLUDE '..\..\KRES\H\KSYNMAC.H'
        INCLUDE '..\..\KRES\H\KTMR.H'
        INCLUDE '..\..\KRES\H\KTMRMAC.H'
        INCLUDE '..\..\KRES\H\KTOD.H'
        INCLUDE '..\..\KRES\H\KTODMAC.H'
        INCLUDE '..\..\KRES\H\KTZC.H'
        INCLUDE '..\..\KRES\H\KTZCMAC.H'
        INCLUDE '..\..\KRES\H\KBCK.H'
        INCLUDE '..\..\KRES\H\KBCKMAC.H'
        INCLUDE '..\..\USAGE\H\USG.H'
        INCLUDE '..\..\USAGE\H\USGVARS.H'
        INCLUDE '..\..\USAGE\H\USGMAC.H'
        INCLUDE '..\..\CORE\H\SECTION.H'
    
        INCLUDE '..\..\TOD\H\TOD.H'
        INCLUDE '..\..\TOD\H\TODVARS.H'
        
        INCLUDE '..\..\CTMR\H\CTMR.H'
        INCLUDE '..\..\CTMR\H\CTMRVARS.H'

        INCLUDE '..\..\TMR\H\TMRVARS.H'
        

$LIST ON

        SECT ".text_tmr"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : tmrPORInitializationParameters
;
; Description : Default Power-Up Data Parameters for the TIMER Application
;
; Assumptions : None
;
; Input(s)    : None
;
; Output(s)   : None
;
; Module Change History:
;
;               08/03/2000  - Created module for M841 
;
;   M851 Development:
;
;               12/11/2001  - Modified for M851    
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        GLOBAL  tmrPORInitializationParameters

tmrPORInitializationParameters:

                ;==============================================================
                ; ACB offset mask.
                ;==============================================================

                ; Application System Data is located in heap.
                ; Other ACB entries are located either in ROM or EEPROM.
                db      bCOREAppSystemDataOffset

                ;==============================================================
                ; Number of resources required.
                ;==============================================================
    
                db      0                           ; TOD
                db      0                           ; Backup
                db      0                           ; Time Zone Check
                db      1                           ; Timer
                db      0                           ; Stopwatch
                db      0                           ; Synchro

                ;==============================================================
                ; Flag(s) ownership.
                ;==============================================================

                db      (bCOREAppFlag_HOURGLASS | bCOREAppFlag_RING | bCOREAppFlag_ARROW)
                db      bCOREAppFlag_STP   

                ;==============================================================
                ; Heap size requirements.
                ;==============================================================

                dw      0                           ; Code
                dw      TMRSYSTEMDATASIZE           ; ASD
                dw      TMREEPROMDBALLOCATIONSIZE   ; ADD

                ;==============================================================
                ; Application Configuration Data Byte.
                ;==============================================================

                db      COREACDROMAPPDBINEEPROM     ; Application is in ROM.
                                                    ; DB is in EEPROM.

                ;==============================================================
                ; Application Unique ID.
                ;==============================================================

                db      COREAPPTYPETIMER            ; App type.
                db      0                           ; App instance.

                ;==============================================================
                ; ACB Parameters.
                ;==============================================================

                dw      TMRSYSTEMDATASTARTOFFSET    ; ASD address offset.
                dw      0                           ; ADD address offset.  There is no database in RAM.
                dw      tmrStateManager             ; App state manager address (Absolute Address).
                dw      tmrBackgroundHandler        ; App background handler address (Absolute Address).
                dw      lcdBannerMsg_COUNTDN_TIMER  ; App mode name function address (Absolute Address).




;**END FILE tmrpor.asm


        END

