;**FILE tmrset.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2000 -2002 TIMEX Corporation.
; 
; This document is the property of, and contains information proprietary to
; TIMEX Corporation.  It is to be returned upon request.  Reproduction by any
; means, disclosure, or use of its contents for other than the expressed
; purpose for which it is loaned, without the written consent of TIMEX
; Corporation, is strictly prohibited.
;
; File Name    : tmrset.asm
;
; Module Names : tmrSetStateManager
;                   
; Purpose      : Countdown Timer Application Set State Manager
;
; File Change History:
;
;               12/17/2001  - Modified for M851 development.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

$CASE ON
$LIST OFF

        INCLUDE '..\..\CORE\H\MODEL.H'
        INCLUDE '..\..\HW\H\HW88349.H'
        INCLUDE '..\..\HW\H\HWI2CEE.H'
        INCLUDE '..\..\HW\H\HWBOND.H'
        INCLUDE '..\..\HW\H\HWKBD.H'
        INCLUDE '..\..\HW\H\HWMACRO.H'
        INCLUDE '..\..\HW\H\HWMAIN.H'
        INCLUDE '..\..\HW\H\HWRESET.H'
        INCLUDE '..\..\HW\H\HWVARS.H'
        INCLUDE '..\..\HW\H\HWSERMAC.H'
        INCLUDE '..\..\CORE\H\CORE.H'
        INCLUDE '..\..\CORE\H\COREMAC.H'
        INCLUDE '..\..\CORE\H\CORESVMC.H'
        INCLUDE '..\..\COMM\H\COMM.H'
        INCLUDE '..\..\LCD\H\LCD.H'
        INCLUDE '..\..\LCD\H\LCDMACRO.H'
        INCLUDE '..\..\LCD\H\LCDRAM.H'
        INCLUDE '..\..\DATABASE\H\DB.H'
        INCLUDE '..\..\DATABASE\H\DBMACRO.H'
        INCLUDE '..\..\AUDIO\H\AUD.H'
        INCLUDE '..\..\AUDIO\H\AUDMAC.H'
        INCLUDE '..\..\UTIL\H\UTLMATH.H'
        INCLUDE '..\..\UTIL\H\UTLMTHMC.H'
        INCLUDE '..\..\UTIL\H\UTLEDT.H'
        INCLUDE '..\..\UTIL\H\UTLEDTMC.H'
        INCLUDE '..\..\UTIL\H\UTLDSP.H'
        INCLUDE '..\..\UTIL\H\UTLDSPMC.H'
        INCLUDE '..\..\UTIL\H\UTLACCMC.H'
        INCLUDE '..\..\UTIL\H\UTLMSC.H'
        INCLUDE '..\..\UTIL\H\UTLMSCMC.H'
        INCLUDE '..\..\UTIL\H\UTLVARS.H'
        INCLUDE '..\..\KRES\H\KRES.H'
        INCLUDE '..\..\KRES\H\KRESMAC.H'
        INCLUDE '..\..\KRES\H\KSTP.H'
        INCLUDE '..\..\KRES\H\KSTPMAC.H'
        INCLUDE '..\..\KRES\H\KSYN.H'
        INCLUDE '..\..\KRES\H\KSYNMAC.H'
        INCLUDE '..\..\KRES\H\KTMR.H'
        INCLUDE '..\..\KRES\H\KTMRMAC.H'
        INCLUDE '..\..\KRES\H\KTOD.H'
        INCLUDE '..\..\KRES\H\KTODMAC.H'
        INCLUDE '..\..\KRES\H\KTZC.H'
        INCLUDE '..\..\KRES\H\KTZCMAC.H'
        INCLUDE '..\..\KRES\H\KBCK.H'
        INCLUDE '..\..\KRES\H\KBCKMAC.H'
        INCLUDE '..\..\USAGE\H\USG.H'
        INCLUDE '..\..\USAGE\H\USGVARS.H'
        INCLUDE '..\..\USAGE\H\USGMAC.H'
        INCLUDE '..\..\CORE\H\SECTION.H'

        INCLUDE '..\..\TOD\H\TOD.H'
        INCLUDE '..\..\TOD\H\TODVARS.H'
        
        INCLUDE '..\..\CTMR\H\CTMR.H'
        INCLUDE '..\..\CTMR\H\CTMRVARS.H'

        INCLUDE '..\..\TMR\H\TMRVARS.H'

$LIST ON

        SECT    ".text_tmr"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : tmrSetStateManager
;
; Description : Timer Application Set State Manager
;
; Assumptions : Display is cleared on first time entry into the state
;
; Input(s)    : CORECurrentEvent
;
; Output(s)   : None
;
; Module Change History:
;
;               12/17/2001  - Modified for M851 development.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void tmrSetStateManager()
{
    switch( CORECurrentEvent )
    {

        case    CTMR_STATEENTRY:
            //*****************************************************************
            //
            //                      STATE ENTRY EVENT
            //
            //*****************************************************************

            USG_INCREMENT_EEPROM_COUNTERS   USGENTRANCESTOANYTMRSETSTATE;

            //-----------------------------------------------------------------
            // Don't pass start/split switch depress and release event to this
            //   state.
            //-----------------------------------------------------------------
            CORE_MASK_KEYS  CTMRSWITCHMASK_STARTSPLIT;

            // Enable pulse mode so that accelerated data can be used.
            CORE_ENABLE_PULSE_MODE;

            CORE_ENABLE_SWITCH_RELEASE;

            //-----------------------------------------------------------------
            // Enable set mode.  Setting this bit would indicate that timer has
            //   entered set state.  This bit will be checked in the background
            //   handler task exit event.
            //-----------------------------------------------------------------
            CORE_ENABLE_SET_MODE;

            //-----------------------------------------------------------------
            // Copies the ASD data to a common foreground buffer to be used 
            //    for foreground processing.
            // Copies the entry specified in last viewed entry to a common 
            //    foreground buffer.
            //-----------------------------------------------------------------
            tmrCopyDataToCommonBuffer();

            // Reset the timer resource.
            AReg = CTMRActiveResource;
            KTMR_RESET_RESOURCE;

            // Cancels any pending countdown timer popup.
            // Destroys AReg, IYReg, HLReg.
            LReg = AReg;
            CORETASK_POPUP_CANCEL_TMR;

            // Turns off the LCD resource flags.
            // Destroys IXReg, HLReg.
            ctmrUpdateTurnOffResourceFlags();

            // Hour and minute is the first set field.
            CTMRCurrentSetPosition = CTMRSETMINHOUR;
            
            //-----------------------------------------------------------------
            // Refresh the entire display.
            // Request blinking on the edit field.
            //-----------------------------------------------------------------
            goto    tmrSetUpdDispAndReqBlink;


        case    CTMR_MODEDEPRESS:
            //*****************************************************************
            //
            //                  MODE SWITCH DEPRESS EVENT
            //
            //*****************************************************************
            
            //-----------------------------------------------------------------
            // Copy the event passed by the core.  This will be used during 
            //   fast selection of data to edit.        
            //-----------------------------------------------------------------
            AReg = CORECurrentEvent;
            CTMRKeyDepressed = AReg;

            // Go to the next set item.
            ctmrGoToNextSetItem();

            //-----------------------------------------------------------------
            // Refresh entire display.
            // Request blinking on the edit field.
            // Request 1sec timeout for the automatic changing of edit field
            //   while this switch is being held.
            //-----------------------------------------------------------------
            goto    tmrSetUpdDispAndReqBlinkAndTmout


        case    CTMR_STOPRESETDEPRESS:
            //*****************************************************************
            //
            //              STOP/RESET SWITCH DEPRESS EVENT
            //
            //*****************************************************************
                
            //-----------------------------------------------------------------
            // Copy the event passed by the core.  This will be used during 
            //  fast selection of data to edit.     
            //-----------------------------------------------------------------
            AReg = CORECurrentEvent;
            CTMRKeyDepressed = AReg;

            // Go to the previous set item.
            ctmrGoToPrevSetItem();

tmrSetUpdDispAndReqBlinkAndTmout:

            //-----------------------------------------------------------------
            // Request for 1 second timeout before the fast edit field 
            //   selection is started.
            //-----------------------------------------------------------------
            CTMRTimeoutArgument = CTMRTMOUTDELAYBEFORESCANNING;
            CORE_REQ_TIMEOUT_HIRES CTMRHIRESTO_1SECOND;
                    
tmrSetUpdDispAndReqBlink:

            //-----------------------------------------------------------------
            // Refresh the entire display depending on the field to set.  Only 
            //   those who are not editable on that field.
            // Input: BReg - Variable that contains the action at end bits.
            //-----------------------------------------------------------------
            BReg = CTMRFlags;
            ctmrStaticDisplayOfField();     

            // Display the editable field.
            tmrDisplayEditField();          

//-----------------------------------------------------------------------------
//
// W A R N I N G !!! This is a fall through.  Do not rearrange.
//
//-----------------------------------------------------------------------------

        case    CTMR_STOPRESETRELEASE:
        case    CTMR_MODERELEASE:
            //*****************************************************************
            //
            //                 MODE SWITCH RELEASE EVENT
            //              STOP/RESET SWITCH RELEASE EVENT
            //
            //*****************************************************************

            // Request blinking on the editable field.
            tmrRequestBlinking();

            break;

        case    CTMR_CWPULSES:
            //*****************************************************************
            // 
            //                      CW PULSE EVENT
            //
            //*****************************************************************

            //-----------------------------------------------------------------
            // Increments a value of the selected field.
            // Destroys AReg, HLReg.
            // Input: HLReg - Address that contains the action at end bits.
            //                  This is only useful when setting the action
            //                  at end.
            //-----------------------------------------------------------------
            HLReg = &CTMRFlags;
            ctmrIncField();                         

            // Display the editable field.
            tmrDisplayEditField();                  

            // Request blinking on the editable field.
            tmrRequestBlinking();

            // Display the new data.            
    		LCD_DISP_4HZ_DATA_FIRST;

            break;

        case    CTMR_CCWPULSES:
            //*****************************************************************
            // 
            //                      CCW PULSE EVENT
            //
            //*****************************************************************
            
            //-----------------------------------------------------------------
            // Decrements a value of the selected field.
            // Destroys AReg, HLReg.
            // Input: HLReg - Address that contains the action at end bits.
            //                  This is only useful when setting the action
            //                  at end.
            //-----------------------------------------------------------------
            HLReg = &CTMRFlags;
            ctmrDecField();                         

            // Display the editable field.
            tmrDisplayEditField();                  

            // Request blinking on the editable field.
            tmrRequestBlinking();                   

            // Display the new data.            
    		LCD_DISP_4HZ_DATA_FIRST;

            break;


        case    CTMR_TMOUTDONEHIRES:
            //*****************************************************************
            // 
            //              HIGH RESOLUTION TIMEOUT DONE EVENT
            //
            //*****************************************************************

            //-----------------------------------------------------------------
            // This will result to a fast edit field select.
            // The editable field will change every 4Hz if a key is still 
            //   depressed.
            //-----------------------------------------------------------------

            if( CTMRTimeoutArgument == CTMRTMOUTDELAYBEFORESCANNING )
            {
                //-------------------------------------------------------------
                // Cancel the blinking since it will not be used due to fast 
                //   edit field selection.
                //-------------------------------------------------------------
                CORE_CANCEL_BLINK_4HZ;              
    
                // Load the next timeout argument.
                CTMRTimeoutArgument = CTMRTMOUTSCANNINGENABLED;
            }

            //-----------------------------------------------------------------
            // Continuously requesting for a 250ms timeout to have a fast edit 
            //   field selection at 4Hz rate.
            //-----------------------------------------------------------------
            CORE_REQ_TIMEOUT_HIRES CTMRHIRESTO_250MSECONDS;

            if( CTMRKeyDepressed == CTMR_MODEDEPRESS )
            {
                ctmrGoToNextSetItem();
            }

            else
            {
                ctmrGoToPrevSetItem();
            }

            //-----------------------------------------------------------------
            // Refresh the entire display depending on the field to set.  
            //   Only those who are not editable on that field.
            // Input: BReg - Variable that contains the action at end bits.
            //-----------------------------------------------------------------
            BReg = CTMRFlags;
            ctmrStaticDisplayOfField();             
                                                    
            // Display the editable field.
            tmrDisplayEditField();                  

            break;


        case COREEVENT_CROWN_HOME:
            //*****************************************************************
            // 
            //                      CROWN HOME EVENT
            //
            //*****************************************************************

            // Disable set mode
            CORE_DISABLE_SET_MODE;                  

            BReg = CTMRDEFAULTSTATE;
            CORE_REQ_STATE_CHANGE;

            //-----------------------------------------------------------------
            //  Updates the individual bits according to the following:
            //      - Less than 1 minute
            //      - Less than 15 seconds
            //      - Zero data.
            //  Disables the halfway reminder alert if user set is less than 
            //    1 minute.
            //  Destroys IXReg.
            //-----------------------------------------------------------------
            ctmrUpdateFlagStatus();

            //-----------------------------------------------------------------
            // Writes the status flag and user set data.
            // Input: AReg  - Record number to write to.
            //        HLReg - Absolute address of DB file in external memory.
            //-----------------------------------------------------------------
            AReg = CTMRLastViewedEntry;
            HLReg = CORECurrentADDAddress;
            ctmrWriteStatusFlagAndHMSDataToEEPROM();

            break;

    }
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF   SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'tmrSetStateManager'"

        GLOBAL  tmrSetStateManager

        GLOBAL  tmrSetCrownHomeEvent

tmrSetStateManager:                                 ; **SUBROUTINE tmrSetStateManager

                ; Load the event to be processed.
                ld      A, [CORECurrentEvent]

                ; Check if mode depress event.
                cp      A, #CTMR_MODEDEPRESS
                jr      Z, tmrSetModeDepressEvent

                ; Check if stop/reset depress event.
                cp      A, #CTMR_STOPRESETDEPRESS
                jr      Z, tmrSetStopResetDepressEvent

                ; Check if hi-res timeout done event.
                cp      A, #CTMR_TMOUTDONEHIRES
                jr      Z, tmrSetHiResTmoutDoneEvent

                ; Check if CW pulse event.
                cp      A, #CTMR_CWPULSES
                jr      Z, tmrSetCWPulseEvent

                ; Check if CCW pulse event.
                cp      A, #CTMR_CCWPULSES
                jr      Z, tmrSetCCWPulseEvent

                ; Check if stop/reset release event.
                cp      A, #CTMR_STOPRESETRELEASE
                jr      Z, tmrSetStopResetReleaseEvent

                ; Check if mode release event.
                cp      A, #CTMR_MODERELEASE
                jr      Z, tmrSetModeReleaseEvent

                ; Check if its state entry event.
                cp      A, #CTMR_STATEENTRY
                jr      Z, tmrSetStateEntryEvent

                ; Check if crown home event.
                cp      A, #COREEVENT_CROWN_HOME
                jr      Z, tmrSetCrownHomeEvent

tmrSetStateExit:

                ret


tmrSetCrownHomeEvent:
                ;**************************************************************
                ; 
                ;                       CROWN HOME EVENT
                ;
                ;**************************************************************
                
                ; Disable set mode
                CORE_DISABLE_SET_MODE

                ld      B, #CTMRDEFAULTSTATE
                CORE_REQ_STATE_CHANGE

                ;--------------------------------------------------------------
                ;   Updates the individual bits according to the following:
                ;       - Less than 1 minute
                ;       - Less than 15 seconds
                ;       - Zero data.
                ;   Disables the halfway reminder alert if user set is less 
                ;     than 1 minute.
                ;  Destroys IXReg.
                ;--------------------------------------------------------------
                car     ctmrUpdateFlagStatus

                ;--------------------------------------------------------------
                ; Writes the status flag and user set data.
                ; Input: AReg  - Record number to write to.
                ;        HLReg - Absolute address of DB file in external memory.
                ;--------------------------------------------------------------
                ld      A, [CTMRLastViewedEntry]
                ld      HL, [CORECurrentADDAddress]
                jr      ctmrWriteStatusFlagAndHMSDataToEEPROM   ; **EXTERNAL JUMP


tmrSetStateEntryEvent:
                ;**************************************************************
                ;
                ;                      STATE ENTRY EVENT
                ;
                ;**************************************************************

                USG_INCREMENT_EEPROM_COUNTERS   USGENTRANCESTOANYTMRSETSTATE

                ;--------------------------------------------------------------
                ; Don't pass start/split switch depress and release event to 
                ;   this state.
                ;--------------------------------------------------------------
                CORE_MASK_KEYS  CTMRSWITCHMASK_STARTSPLIT
                
                ; Enable pulse mode so that accelerated data can be used.
                CORE_ENABLE_PULSE_MODE

                CORE_ENABLE_SWITCH_RELEASE

                ;--------------------------------------------------------------
                ; Enable set mode.  Setting this bit would indicate that timer 
                ;   has entered set state.  This bit will be checked in the 
                ;   background handler task exit event.
                ;--------------------------------------------------------------
                CORE_ENABLE_SET_MODE

                ;--------------------------------------------------------------
                ; Copies the ASD data to a common foreground buffer to be used 
                ;    for foreground processing.
                ; Copies the entry specified in last viewed entry to a common 
                ;    foreground buffer.
                ;--------------------------------------------------------------
                car     tmrCopyDataToCommonBuffer

                ;--------------------------------------------------------------
                ; Resets the timer resource.
                ; Destroys HLReg.
                ; Input: AReg - Resource index.
                ;--------------------------------------------------------------
                ld      A, [CTMRActiveResource]
                KTMR_RESET_RESOURCE
                
                ; Cancels any pending countdown timer popup.
                ; Destroys AReg, IYReg, HLReg.
                ld      L, A
                CORETASK_POPUP_CANCEL_TMR

                ; Turns off the LCD resource flags.
                ; Destroys IXReg, HLReg.
                car     ctmrUpdateTurnOffResourceFlags

                ; Hour and minute is the first set field.
                ld      A, #CTMRSETMINHOUR
                ld      [CTMRCurrentSetPosition], A
            
                ;--------------------------------------------------------------
                ; Refresh the entire display.
                ; Request blinking on the edit field.
                ;--------------------------------------------------------------
                jr      tmrSetUpdDispAndReqBlink

tmrSetModeDepressEvent:
                ;**************************************************************
                ;
                ;                  MODE SWITCH DEPRESS EVENT
                ;
                ;**************************************************************
            
                ;--------------------------------------------------------------
                ; Copy the event passed by the core.  This will be used during 
                ;   fast selection of data to edit.     
                ;--------------------------------------------------------------
                ld      A, [CORECurrentEvent]
                ld      [CTMRKeyDepressed], A

                ; Go to the next set item.
                car     ctmrGoToNextSetItem

                ;--------------------------------------------------------------
                ; Refresh entire display.
                ; Request blinking on the edit field.
                ; Request 1sec timeout for the automatic changing of edit field
                ;   while this switch is being held.
                ;--------------------------------------------------------------
                jr      tmrSetUpdDispAndReqBlinkAndTmout


tmrSetStopResetDepressEvent:
                ;**************************************************************
                ;
                ;              STOP/RESET SWITCH DEPRESS EVENT
                ;
                ;**************************************************************
                
                ;--------------------------------------------------------------
                ; Copy the event passed by the core.  This will be used during 
                ;  fast selection of data to edit.      
                ;--------------------------------------------------------------
                ld      A, [CORECurrentEvent]
                ld      [CTMRKeyDepressed], A

                ; Go to the previous set item.
                car     ctmrGoToPrevSetItem

tmrSetUpdDispAndReqBlinkAndTmout:

                ;--------------------------------------------------------------
                ; Request for 1 second timeout before the fast edit field 
                ;   selection is started.
                ;--------------------------------------------------------------
                ld      A, #CTMRTMOUTDELAYBEFORESCANNING
                ld      [CTMRTimeoutArgument], A
                CORE_REQ_TIMEOUT_HIRES CTMRHIRESTO_1SECOND
                    
tmrSetUpdDispAndReqBlink:

                ;--------------------------------------------------------------
                ; Refresh the entire display depending on the field to set.  
                ;   Only those who are not editable on that field.
                ; Input: BReg - Variable that contains the action at end bits.
                ;--------------------------------------------------------------
                ld      B, [CTMRFlags]
                car     ctmrStaticDisplayOfField

                ; Display the editable field.
                car     tmrDisplayEditField

;------------------------------------------------------------------------------
;
; W A R N I N G !!! This is a fall through.  Do not rearrange.
;
;------------------------------------------------------------------------------


tmrSetStopResetReleaseEvent:
tmrSetModeReleaseEvent:
                ;**************************************************************
                ;
                ;                 MODE SWITCH RELEASE EVENT
                ;              STOP/RESET SWITCH RELEASE EVENT
                ;
                ;**************************************************************

                ; Request blinking on the editable field.
                jr      tmrRequestBlinking          ; **EXTERNAL JUMP


tmrSetCWPulseEvent:
                ;**************************************************************
                ; 
                ;                       CW PULSE EVENT
                ;
                ;**************************************************************
                
                ;--------------------------------------------------------------
                ; Increments a value of the selected field.
                ; Destroys AReg, HLReg.
                ; Input: HLReg - Address that contains the action at end bits.
                ;                  This is only useful when setting the action
                ;                  at end.
                ;--------------------------------------------------------------
                ld      HL, #CTMRFlags
                car     ctmrIncField

                ; Display the editable field.
                car     tmrDisplayEditField

                ; Request blinking on the editable field.
                car     tmrRequestBlinking
                               
                ; Display the new data.            
        		LCD_DISP_4HZ_DATA_FIRST

                ret
    
            

tmrSetCCWPulseEvent:
                ;**************************************************************
                ; 
                ;                       CCW PULSE EVENT
                ;
                ;**************************************************************
            
                ;--------------------------------------------------------------
                ; Decrements a value of the selected field.
                ; Destroys AReg, HLReg.
                ; Input: HLReg - Address that contains the action at end bits.
                ;                  This is only useful when setting the action
                ;                  at end.
                ;--------------------------------------------------------------
                ld      HL, #CTMRFlags
                car     ctmrDecField

                ; Display the editable field.
                car     tmrDisplayEditField
                
                ; Request blinking on the editable field.
                car     tmrRequestBlinking
            
                ; Display the new data.            
        		LCD_DISP_4HZ_DATA_FIRST

                ret

tmrSetHiResTmoutDoneEvent:
                ;**************************************************************
                ; 
                ;               HIGH RESOLUTION TIMEOUT DONE EVENT
                ;
                ;**************************************************************

                ;--------------------------------------------------------------
                ; This will result to a fast edit field select.
                ; The editable field will change every 4Hz if a key is still 
                ;   depressed.
                ;--------------------------------------------------------------

                ld      B, [CTMRTimeoutArgument]

                cp      B, #CTMRTMOUTDELAYBEFORESCANNING
                jr      NZ, tmrSet4HzScanningHasStarted

                ;--------------------------------------------------------------
                ; Cancel the blinking since it will not be used due to fast 
                ;   edit field selection.
                ;--------------------------------------------------------------
                CORE_CANCEL_BLINK_4HZ
                
                ; Load the next timeout argument.
                ld      B, #CTMRTMOUTSCANNINGENABLED
                ld      [CTMRTimeoutArgument], B

tmrSet4HzScanningHasStarted:

                ;--------------------------------------------------------------
                ; Continuously requesting for a 250ms timeout to have a fast 
                ;   edit field selection at 4Hz rate.
                ;--------------------------------------------------------------
                CORE_REQ_TIMEOUT_HIRES CTMRHIRESTO_250MSECONDS

                ld      B, [CTMRKeyDepressed]
                cp      B, #CTMR_MODEDEPRESS
                jr      NZ, tmrSetTmoutDoneGoToPrev
                
                car     ctmrGoToNextSetItem

                jr      tmrSetTmoutDoneDisplayNewSetItem

tmrSetTmoutDoneGoToPrev:

                car     ctmrGoToPrevSetItem
                
tmrSetTmoutDoneDisplayNewSetItem:
                ;--------------------------------------------------------------
                ; Refresh the entire display depending on the field to set.  
                ;   Only those who are not editable on that field.
                ; Input: BReg - Variable that contains the action at end bits.
                ;--------------------------------------------------------------
                ld      B, [CTMRFlags]
                car     ctmrStaticDisplayOfField
                                                    
                ; Display the editable field.
                jr      tmrDisplayEditField         ; **EXTERNAL JUMP


                                                    ; **END SUBROUTINE tmrSetStateManager

        
        CALLS   SUBROUTINE, 'ctmrUpdateFlagStatus'
        CALLS   SUBROUTINE, 'tmrCopyDataToCommonBuffer'
        CALLS   SUBROUTINE, 'ctmrGoToNextSetItem'
        CALLS   SUBROUTINE, 'ctmrGoToPrevSetItem'
        CALLS   SUBROUTINE, 'ctmrStaticDisplayOfField'
        CALLS   SUBROUTINE, 'tmrDisplayEditField'
        CALLS   SUBROUTINE, 'tmrRequestBlinking'
        CALLS   SUBROUTINE, 'ctmrIncField'
        CALLS   SUBROUTINE, 'ctmrDecField'
        CALLS   SUBROUTINE, 'ctmrUpdateTurnOffResourceFlags'

;**END FILE tmrset.asm


        END

