;**FILE tmrutil.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2000 - 2002 TIMEX Corporation.
; 
; This document is the property of, and contains information proprietary to
; TIMEX Corporation.  It is to be returned upon request.  Reproduction by any
; means, disclosure, or use of its contents for other than the expressed
; purpose for which it is loaned, without the written consent of TIMEX
; Corporation, is strictly prohibited.
;
; File Name    : tmrutil.asm
;
; Module Names : tmrCopyDataToCommonBuffer - Copies the ASD data to a common foreground 
;                                              buffer to be used for foreground processing.
;                                            Copies the entry specified in last viewed entry 
;                                              to a common foreground buffer.
;                 
;                tmrSetupActionAtEnd       - Sets up the resource action at end.
;
;                tmrRequestBlinking        - Request blinking on a specified field.
;
;
;
; Purpose      : Countdown Timer Application Common Utilities
;
; File Change History:
;
;                12/12/2001  - Created for M851 development.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

$CASE ON
$LIST OFF

        INCLUDE '..\..\CORE\H\MODEL.H'
        INCLUDE '..\..\HW\H\HW88349.H'
        INCLUDE '..\..\HW\H\HWI2CEE.H'
        INCLUDE '..\..\HW\H\HWBOND.H'
        INCLUDE '..\..\HW\H\HWKBD.H'
        INCLUDE '..\..\HW\H\HWMACRO.H'
        INCLUDE '..\..\HW\H\HWMAIN.H'
        INCLUDE '..\..\HW\H\HWRESET.H'
        INCLUDE '..\..\HW\H\HWVARS.H'
        INCLUDE '..\..\HW\H\HWSERMAC.H'
        INCLUDE '..\..\CORE\H\CORE.H'
        INCLUDE '..\..\CORE\H\COREMAC.H'
        INCLUDE '..\..\CORE\H\CORESVMC.H'
        INCLUDE '..\..\COMM\H\COMM.H'
        INCLUDE '..\..\LCD\H\LCD.H'
        INCLUDE '..\..\LCD\H\LCDMACRO.H'
        INCLUDE '..\..\LCD\H\LCDRAM.H'
        INCLUDE '..\..\DATABASE\H\DB.H'
        INCLUDE '..\..\DATABASE\H\DBMACRO.H'
        INCLUDE '..\..\AUDIO\H\AUD.H'
        INCLUDE '..\..\AUDIO\H\AUDMAC.H'
        INCLUDE '..\..\UTIL\H\UTLMATH.H'
        INCLUDE '..\..\UTIL\H\UTLMTHMC.H'
        INCLUDE '..\..\UTIL\H\UTLEDT.H'
        INCLUDE '..\..\UTIL\H\UTLEDTMC.H'
        INCLUDE '..\..\UTIL\H\UTLDSP.H'
        INCLUDE '..\..\UTIL\H\UTLDSPMC.H'
        INCLUDE '..\..\UTIL\H\UTLACCMC.H'
        INCLUDE '..\..\UTIL\H\UTLMSC.H'
        INCLUDE '..\..\UTIL\H\UTLMSCMC.H'
        INCLUDE '..\..\UTIL\H\UTLVARS.H'
        INCLUDE '..\..\KRES\H\KRES.H'
        INCLUDE '..\..\KRES\H\KRESMAC.H'
        INCLUDE '..\..\KRES\H\KSTP.H'
        INCLUDE '..\..\KRES\H\KSTPMAC.H'
        INCLUDE '..\..\KRES\H\KSYN.H'
        INCLUDE '..\..\KRES\H\KSYNMAC.H'
        INCLUDE '..\..\KRES\H\KTMR.H'
        INCLUDE '..\..\KRES\H\KTMRMAC.H'
        INCLUDE '..\..\KRES\H\KTOD.H'
        INCLUDE '..\..\KRES\H\KTODMAC.H'
        INCLUDE '..\..\KRES\H\KTZC.H'
        INCLUDE '..\..\KRES\H\KTZCMAC.H'
        INCLUDE '..\..\KRES\H\KBCK.H'
        INCLUDE '..\..\KRES\H\KBCKMAC.H'
        INCLUDE '..\..\USAGE\H\USG.H'
        INCLUDE '..\..\USAGE\H\USGVARS.H'
        INCLUDE '..\..\USAGE\H\USGMAC.H'
        INCLUDE '..\..\CORE\H\SECTION.H'

        INCLUDE '..\..\TOD\H\TOD.H'
        INCLUDE '..\..\TOD\H\TODVARS.H'
        
        INCLUDE '..\..\CTMR\H\CTMR.H'
        INCLUDE '..\..\CTMR\H\CTMRVARS.H'

        INCLUDE '..\..\TMR\H\TMRVARS.H'

$LIST ON

        SECT    ".text_tmr"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : tmrCopyDataToCommonBuffer
;
; Description : Copies the ASD data to a common foreground buffer to be
;                 used for foreground processing.
;
;               Copies the entry specified in last viewed entry to a common 
;                 foreground buffer.
;
; Assumptions : None
;
; Input(s)    : None
;
; Output(s)   : CTMRActiveResource  - Timer resource index used by timer app.
;               CTMRLastViewedEntry - Last viewed timer entry.
;               CTMRNumberOfRecords - Number of records in DB.
;               User set data is stored in:
;                   -> CTMRFlags         - Status flags.
;                   -> CTMRUserSetSecond - User set seconds data.
;                   -> CTMRUserSetMinute - User set minute data.
;                   -> CTMRUserSetHour   - User set hour data.
;                   -> CTMRNameX         - 9 char timer name, where x = 0 to 8
;
;               ( Destroyed: IYReg, HLReg, BAReg )
;
; Module Change History:
;
;               12/12/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'tmrCopyDataToCommonBuffer'"

        GLOBAL  tmrCopyDataToCommonBuffer

tmrCopyDataToCommonBuffer:                          ; **SUBROUTINE tmrCopyDataToCommonBuffer

                ; Copies the resource index and last viewed entry to AReg and 
                ;   BReg respectively.
                ld      IY, [CORECurrentASDAddress]
                ld      BA, [IY]
                ld      [CTMRActiveResource], BA

                ;--------------------------------------------------------------
                ; Reads a record from the timer database.
                ; Destroys BAReg, HLReg.
                ; Input: HLReg - Absolute address of the DB file in external 
                ;                  memory.
                ;        AReg  - Record number to read.
                ;--------------------------------------------------------------
                ld      HL, [CORECurrentADDAddress]
                ld      A, B
                jr      ctmrReadOneRecord           ; **EXTERNAL JUMP


                                                    ; **END SUBROUTINE tmrCopyDataToCommonBuffer



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : tmrSetupActionAtEnd
;
; Description : Sets up the resource action at end.
;
; Assumptions : None
;
; Input(s)    : None
;
; Output(s)   : None
;
;               ( Destroyed: BReg, IYReg, IXReg, HLReg )
;
; Module Change History:
;
;               12/14/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void tmrSetupActionAtEnd()
{
    //-------------------------------------------------------------------------
    // Cancels a link to TMR and STP.
    // Destroys nothing.
    // Input: AReg - Resource index.
    //-------------------------------------------------------------------------
    AReg = CTMRActiveResource;
    KTMR_CANCEL_ALL_LINKS;

    //-------------------------------------------------------------------------
    // Disables repeat at end action.
    // Destroys BReg.
    // Input: AReg - Resource index.
    //-------------------------------------------------------------------------
    KTMR_CANCEL_REPEAT_AT_END;

    // Get the action at end.
    BReg = CTMRFlags;
    BReg &= CTMRACTIONATENDMASKBITS;

    switch( BReg )
    {
        case    CTMRREPEATATEND:
            //=================================================================
            //
            //                      REPEAT AT END
            //
            //=================================================================

            //-----------------------------------------------------------------
            // Enables the repeat at end.
            // Destroys nothing.
            // Input: AReg - Resource index.
            //        BReg - Number of repetitions. (HEX data)
            // If number of repetitions is 0, then it is a continuous repeat.
            //-----------------------------------------------------------------
            BReg = 0;
            KTMR_SETUP_REPEAT_AT_END;

            //-----------------------------------------------------------------
            // Change the value of the repeat counter.
            // Destroys nothing.
            // Input: AReg - Resource index.
            //        BReg - Starting repetition counter (HEX data)
            //-----------------------------------------------------------------
            BReg = 1;
            KTMR_SET_RUNNING_REPEAT_COUNTER;

            // Set the update repeat number display.
            HLReg = CORECurrentASDAddress;
            HLReg += TMRFLAGS2OFFSET;
            *HLReg |= bTMRUpdRepeatNumDisplay;

            break;

        case    CTMRCHRONOATEND:
            //=================================================================
            //
            //                      CHRONO AT END
            //
            //=================================================================
        
            //-----------------------------------------------------------------
            // Sets the bit that indicates to link a stopwatc when timer expires.
            // Input: AReg - Timer resource to link STP resource to.
            //-----------------------------------------------------------------
            KTMR_SETUP_STP_LINK;

            break;

    }
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'tmrSetupActionAtEnd'"

        GLOBAL  tmrSetupActionAtEnd

tmrSetupActionAtEnd:                                ; **SUBROUTINE tmrSetupActionAtEnd

                ;--------------------------------------------------------------
                ; Cancels a link to TMR and STP.
                ; Destroys nothing.
                ; Input: AReg - Resource index.
                ;--------------------------------------------------------------
                ld      A, [CTMRActiveResource]
                KTMR_CANCEL_ALL_LINKS

                ;--------------------------------------------------------------
                ; Disables repeat at end action.
                ; Destroys BReg.
                ; Input: AReg - Resource index.
                ;--------------------------------------------------------------
                KTMR_CANCEL_REPEAT_AT_END
                
                ; Get the action at end.
                ld      B, [CTMRFlags]
                and     B, #CTMRACTIONATENDMASKBITS

                cp      B, #CTMRREPEATATEND
                jr      Z, tmrSetupRepeatAtEnd
                
                cp      B, #CTMRCHRONOATEND
                jr      NZ, tmrSetupActionAtEndExit


                ;==============================================================
                ;
                ;                      CHRONO AT END
                ;
                ;==============================================================
                        
                ;--------------------------------------------------------------
                ; Sets the bit that indicates to link a stopwatc when timer 
                ;   expires.
                ; Input: AReg - Timer resource to link STP resource to.
                ;--------------------------------------------------------------
                KTMR_SETUP_STP_LINK

tmrSetupActionAtEndExit:

                ret


tmrSetupRepeatAtEnd:
                ;==============================================================
                ;
                ;                      REPEAT AT END
                ;
                ;==============================================================

                ;--------------------------------------------------------------
                ; Enables the repeat at end.
                ; Destroys nothing.
                ; Input: AReg - Resource index.
                ;        BReg - Number of repetitions. (HEX data)
                ; If number of repetitions is 0, then it is a continuous repeat.
                ;--------------------------------------------------------------
                ld      B, #0
                KTMR_SETUP_REPEAT_AT_END

                ;--------------------------------------------------------------
                ; Change the value of the repeat counter.
                ; Destroys nothing.
                ; Input: AReg - Resource index.
                ;        BReg - Starting repetition counter (HEX data)
                ;--------------------------------------------------------------
                ld      B, #1
                KTMR_SET_RUNNING_REPEAT_COUNTER

                ; Set the update repeat number display.
                ld      HL, [CORECurrentASDAddress]
                add     HL, #TMRFLAGS2OFFSET
                or      [HL], #bTMRUpdRepeatNumDisplay

                ret
                                                    
                                                    ; **END SUBROUTINE tmrSetupActionAtEnd













;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : tmrRequestBlinking
;
; Description : Request blinking on a specified field.
;
; Assumptions : None
;
; Input(s)    : None
;
; Output(s)   : None
;               
;               ( Destroyed: BAReg )
;
; Module Change History:
;
;               12/17/2001  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void tmrRequestBlinking()
{

    CORE_REQ_BLINK_4HZ;

    switch( CTMRCurrentSetPosition )
    {

        case    CTMRSETMINHOUR:
            //=================================================================
            //
            //                      HOUR AND MINUTE
            //
            //=================================================================

            // Use the TOD clear routine. TOD and Timer hour and minute position are the same.
            LCD_WRITE_4HZ_GEN_BLINK_DISP_ROUTINE_ADDR   ctmrDisplayUserSetHourMinuteAtBDM;
            LCD_WRITE_4HZ_GEN_BLINK_CLR_ROUTINE_ADDR    todClearHourMinuteLDM;  

            break;

        case    CTMRSETSECOND:
            //=================================================================
            //
            //                          SECOND
            //
            //=================================================================

            LCD_WRITE_4HZ_GEN_BLINK_DISP_ROUTINE_ADDR   ctmrDisplayUserSetSecondAtBDM;
            LCD_WRITE_4HZ_GEN_BLINK_CLR_ROUTINE_ADDR    ctmrClearSecondsLDM;

            break;

        case    CTMRSETACTIONATEND:
            //=================================================================
            //
            //                       ACTION AT END
            //
            //=================================================================

            LCD_WRITE_4HZ_GEN_BLINK_DISP_ROUTINE_ADDR   tmrDisplayActionAtEndMsg;
            LCD_WRITE_4HZ_GEN_BLINK_CLR_ROUTINE_ADDR    lcdClearMainDMLine1;

            break;
    }
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF

        DEFINE  SUBROUTINE      "'tmrRequestBlinking'"

        GLOBAL  tmrRequestBlinking

        ; Shared for interval timer request blinking.
        GLOBAL  tmrReqBlinkHrMin
        GLOBAL  tmrReqBlinkSecond


tmrRequestBlinking:                                 ; **SUBROUTINE tmrRequestBlinking

                CORE_REQ_BLINK_4HZ

                ld      A, [CTMRCurrentSetPosition]

                cp      A, #CTMRSETMINHOUR
                jr      Z, tmrReqBlinkHrMin

                cp      A, #CTMRSETSECOND
                jr      Z, tmrReqBlinkSecond

                ;==============================================================
                ;
                ;                       ACTION AT END
                ;
                ;==============================================================

                LCD_WRITE_4HZ_GEN_BLINK_DISP_ROUTINE_ADDR   tmrDisplayActionAtEndMsg
                LCD_WRITE_4HZ_GEN_BLINK_CLR_ROUTINE_ADDR    lcdClearMainDMLine1

                ret

tmrReqBlinkHrMin:
                ;==============================================================
                ;
                ;                      HOUR AND MINUTE
                ;
                ;==============================================================
                
                ; Use the TOD clear routine. TOD and Timer hour and minute position are the same.
                LCD_WRITE_4HZ_GEN_BLINK_DISP_ROUTINE_ADDR   ctmrDisplayUserSetHourMinuteAtBDM
                LCD_WRITE_4HZ_GEN_BLINK_CLR_ROUTINE_ADDR    todClearHourMinuteLDM

                ret

tmrReqBlinkSecond:
                ;==============================================================
                ;
                ;                          SECOND
                ;
                ;==============================================================

                LCD_WRITE_4HZ_GEN_BLINK_DISP_ROUTINE_ADDR   ctmrDisplayUserSetSecondAtBDM
                LCD_WRITE_4HZ_GEN_BLINK_CLR_ROUTINE_ADDR    ctmrClearSecondsLDM

                ret


                                                    ; **END SUBROUTINE tmrRequestBlinking






;**END FILE tmrutil.asm



        END









