; **FILE utlMisc.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2001-2002 TIMEX Corporation.
; 
; This document is the property of, and contains information proprietary to
; TIMEX Corporation.  It is to be returned upon request.  Reproduction by any
; means, disclosure, or use of its contents for other than the expressed
; purpose for which it is loaned, without the written consent of TIMEX
; Corporation, is strictly prohibited.
;
; File Name    : utlMisc.asm
;
; Module Names : utlProcessStateManagerTable - Jump to the current state specified 
;                                                in CORECurrentState.
;
;                utlJumpToAddress            - Jump to the address specified in a table.
;                
;                utlCompare4ByteBuffer       - Compares the data in a 4 byte buffer, and 
;                                                sets the SC flags  to indicate if one data
;                                                set is <,>, or = to the other.         
;
;                utlCompareHLRegWithIXReg    - Compares two structure as pointed by 
;                                                HLReg and IXReg.
;
;                utlCompareHLRegWithIYReg    - Compares two structures as pointed by 
;                                                HLReg and IYReg.
;
;                utlCopyIYRegToIXReg         - Copies the data pointed by IYReg to the 
;                                                location pointed by IXReg.
;                
;
; Purpose      : Miscellaneous routines.
;
; File Change History:
;
;               02/25/2002  - Created for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

$CASE ON
$LIST OFF

        INCLUDE '..\..\CORE\H\MODEL.H'
        INCLUDE '..\..\HW\H\HW88349.H'
        INCLUDE '..\..\HW\H\HWI2CEE.H'
        INCLUDE '..\..\HW\H\HWBOND.H'
        INCLUDE '..\..\HW\H\HWKBD.H'
        INCLUDE '..\..\HW\H\HWMACRO.H'
        INCLUDE '..\..\HW\H\HWMAIN.H'
        INCLUDE '..\..\HW\H\HWRESET.H'
        INCLUDE '..\..\HW\H\HWVARS.H'
        INCLUDE '..\..\HW\H\HWSERMAC.H'
        INCLUDE '..\..\CORE\H\CORE.H'
        INCLUDE '..\..\CORE\H\COREMAC.H'
        INCLUDE '..\..\CORE\H\CORESVMC.H'
        INCLUDE '..\..\COMM\H\COMM.H'
        INCLUDE '..\..\LCD\H\LCD.H'
        INCLUDE '..\..\LCD\H\LCDMACRO.H'
        INCLUDE '..\..\LCD\H\LCDRAM.H'
        INCLUDE '..\..\DATABASE\H\DB.H'
        INCLUDE '..\..\DATABASE\H\DBMACRO.H'
        INCLUDE '..\..\AUDIO\H\AUD.H'
        INCLUDE '..\..\AUDIO\H\AUDMAC.H'
        INCLUDE '..\..\UTIL\H\UTLMATH.H'
        INCLUDE '..\..\UTIL\H\UTLMTHMC.H'
        INCLUDE '..\..\UTIL\H\UTLEDT.H'
        INCLUDE '..\..\UTIL\H\UTLEDTMC.H'
        INCLUDE '..\..\UTIL\H\UTLDSP.H'
        INCLUDE '..\..\UTIL\H\UTLDSPMC.H'
        INCLUDE '..\..\UTIL\H\UTLACCMC.H'
        INCLUDE '..\..\UTIL\H\UTLMSC.H'
        INCLUDE '..\..\UTIL\H\UTLMSCMC.H'
        INCLUDE '..\..\UTIL\H\UTLVARS.H'
        INCLUDE '..\..\KRES\H\KRES.H'
        INCLUDE '..\..\KRES\H\KRESMAC.H'
        INCLUDE '..\..\KRES\H\KSTP.H'
        INCLUDE '..\..\KRES\H\KSTPMAC.H'
        INCLUDE '..\..\KRES\H\KSYN.H'
        INCLUDE '..\..\KRES\H\KSYNMAC.H'
        INCLUDE '..\..\KRES\H\KTMR.H'
        INCLUDE '..\..\KRES\H\KTMRMAC.H'
        INCLUDE '..\..\KRES\H\KTOD.H'
        INCLUDE '..\..\KRES\H\KTODMAC.H'
        INCLUDE '..\..\KRES\H\KTZC.H'
        INCLUDE '..\..\KRES\H\KTZCMAC.H'
        INCLUDE '..\..\KRES\H\KBCK.H'
        INCLUDE '..\..\KRES\H\KBCKMAC.H'
        INCLUDE '..\..\USAGE\H\USG.H'
        INCLUDE '..\..\USAGE\H\USGVARS.H'
        INCLUDE '..\..\USAGE\H\USGMAC.H'
        INCLUDE '..\..\CORE\H\SECTION.H'

        INCLUDE '..\..\TOD\H\TOD.H'
        INCLUDE '..\..\TOD\H\TODVARS.H'

$LIST ON


        SECT    ".text_utl"




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : utlProcessStateManagerTable 
;
; Description : Jump to the current state specified in CORECurrentState.
;
; Assumptions : CORECurrentState - State to go to.
;               
; Input(s)    : HLReg - Address of the table where the addresses to go to is
;                         defined.
;
; Output(s)   : None
;
;               ( Destroyed: ALL registers )
;
; Module Change History:
;
;               02/26/2002  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

                IF @DEF('SUBROUTINE')
                    UNDEF   SUBROUTINE
                ENDIF

                DEFINE  SUBROUTINE      "'utlProcessStateManagerTable'"
        
                GLOBAL  utlProcessStateManagerTable

utlProcessStateManagerTable:                        ; **SUBROUTINE utlProcessStateManagerTable

                ld      A, [CORECurrentState]


                                                    ; **END SUBROUTINE utlProcessStateManagerTable



;------------------------------------------------------------------------------
;
; W A R N I N G !!! This is a fall through.  Do not rearrange.
;
;------------------------------------------------------------------------------



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : utlJumpToAddress 
;
; Description : Jump to the address specified in a table.
;
; Assumptions : None
;               
; Input(s)    : AReg  - Index to the table.
;               HLReg - Address of the table where the addresses to go to is
;                         defined.
;
; Output(s)   : None
;
;               ( Destroyed: ALL registers )
;
; Module Change History:
;
;               02/26/2002  - Created module for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

                IF @DEF('SUBROUTINE')
                    UNDEF   SUBROUTINE
                ENDIF

                DEFINE  SUBROUTINE      "'utlJumpToAddress'"
        
                GLOBAL  utlJumpToAddress


utlJumpToAddress:                                   ; **SUBROUTINE utlJumpToAddress

                ld      B, #0
                sll     A

                ; Get the address to go to.
                add     HL, BA
                ld      HL, [HL]

                ; Jump to that address.
                jp      HL                          ; **EXTERNAL JUMP


                                                    ; **END SUBROUTINE utlJumpToAddress


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : utlCompare4ByteBuffer
;
; Description : Compares the data in a 4 byte buffer, and sets the SC flags
;                 to indicate if one data set is <,>, or = to the other.
;
; Assumptions : Both datasets are stored in a consecutive 4 byte buffer, order 
;                 least significant to most significant. Data is unsigned.
;
; Input(s)    : HLReg - points to most significant byte of data to be compared to.
;               IXReg - points to most significany byte of data we are comparing.
;
; Output(s)   : HLReg = IXReg ; bZeroFlag = TRUE  , bCarryFlag = FALSE
;               HLReg < IXReg ; bZeroFlag = FALSE , bCarryFlag = TRUE
;               HLReg > IXReg ; bZeroFlag = FALSE , bCarryFlag = FALSE
;
;               ( Destroyed: HLReg, IXReg, BAReg )
;
; Module Change History:
;
;               06/03/2000  - Created module for M841
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void utlCompare4ByteBuffer()
{
    //-------------------------------------------------------------------------
    // Compares two structures as pointed by HLReg and IXReg.
    // Destroys BAReg, HLReg, IXReg.
    // Input:  HLReg and IXReg should point to the higher significant byte
    //           where the comparison should start.
    //         BReg should contain how many byte fields are to be compared.
    // Output: HLReg = IXReg ; bZeroFlag = TRUE  , bCarryFlag = FALSE
    //         HLReg < IXReg ; bZeroFlag = FALSE , bCarryFlag = TRUE
    //         HLReg > IXReg ; bZeroFlag = FALSE , bCarryFlag = FALSE
    //-------------------------------------------------------------------------

    BReg = 4;

    utlCompareHLRegWithIXReg();
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        GLOBAL  utlCompare4ByteBuffer
        
        IF  @DEF('SUBROUTINE')
            UNDEF   SUBROUTINE
        ENDIF
        DEFINE      SUBROUTINE      "'utlCompare4ByteBuffer'"            


utlCompare4ByteBuffer:                              ; **SUBROUTINE utlCompare4ByteBuffer

                ;--------------------------------------------------------------
                ; Compares two structures as pointed by HLReg and IXReg.
                ; Destroys BAReg, HLReg, IXReg.
                ; Input:  HLReg and IXReg should point to the higher significant 
                ;           byte where the comparison should start.
                ;         BReg should contain how many byte fields are to be compared.
                ; Output: HLReg = IXReg ; bZeroFlag = TRUE  , bCarryFlag = FALSE
                ;         HLReg < IXReg ; bZeroFlag = FALSE , bCarryFlag = TRUE
                ;         HLReg > IXReg ; bZeroFlag = FALSE , bCarryFlag = FALSE
                ;--------------------------------------------------------------
                ld      B, #4
                
;               jr      utlCompareHLRegWithIXReg    ; **EXTERNAL JUMP

                                                    ; **END SUBROUTINE utlCompare4ByteBuffer


;------------------------------------------------------------------------------
;
; W A R N I N G !!! This is a fall through.  Do not rearrange.
;
;------------------------------------------------------------------------------

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : utlCompareHLRegWithIXReg
;
; Description : Compares two structures as pointed by HLReg and IXReg.
;
; Assumptions : None
;
; Input(s):     HLReg and IXReg should point to the higher significant byte
;                 where comparison should start.   
;               BReg should contain how many byte fields are to be compared.
;
; Output(s)   : HLReg = IXReg ; bZeroFlag = TRUE  , bCarryFlag = FALSE
;               HLReg < IXReg ; bZeroFlag = FALSE , bCarryFlag = TRUE
;               HLReg > IXReg ; bZeroFlag = FALSE , bCarryFlag = FALSE
;
;               ( Destroyed: HLReg, IXReg, BAReg )
;
; Module Change History:
;
;               12/07/2001  - Created utlCompareHLRegWithIXReg for M851
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


COMMENT `

void utlCompareHLRegWithIXReg()
{
    
    do
    {
        //;--------------------------------------------------------------------
        //; Compare the HLReg with IXReg.
        //;--------------------------------------------------------------------
        
        if( *HLReg != *IXReg)
        { 
            //;----------------------------------------------------------------
            //; If the resource is not equal to that of the Buffer, then go out
            //;   from the compare loop.
            //;----------------------------------------------------------------
        
            break;
        }

        //;--------------------------------------------------------------------
        //; Let the pointers point to next field to compare. The next field to
        //;   compare is the lower significant field.
        //;--------------------------------------------------------------------
        
        HLReg--;
        IXReg--;
        
        //;--------------------------------------------------------------------
        //; Decrement the counter. When BReg is Zero, bZeroFlag is Set or TRUE.
        //;--------------------------------------------------------------------
        
        BReg--;

    }while( bZeroFlag == TRUE )
    
}   

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        GLOBAL  utlCompareHLRegWithIXReg
        
        IF  @DEF('SUBROUTINE')
            UNDEF   SUBROUTINE
        ENDIF
        DEFINE      SUBROUTINE      "'utlCompareHLRegWithIXReg'"            

utlCompareHLRegWithIXReg:                           ; **SUBROUTINE utlCompareHLRegWithIXReg

                ;--------------------------------------------------------------
                ; Compare the HLReg with IXReg.
                ; If the structures are not equal then go out from the compare
                ;   loop.
                ;--------------------------------------------------------------
                cp      [HL], [IX]
                jr      NZ, utlCompareHLWithIXExit
                
                ;--------------------------------------------------------------
                ; Let the pointers point to next field to compare. The next 
                ;   field to compare is the lower significant field.
                ;--------------------------------------------------------------
                dec     HL
                dec     IX        
        
                ;--------------------------------------------------------------
                ; Decrement the counter. When BReg is Zero, bZeroFlag is Set 
                ;   or TRUE.
                ;--------------------------------------------------------------
        
                djr     NZ, utlCompareHLRegWithIXReg
                      

utlCompareHLWithIXExit:

                ret

                                                    ; **END SUBROUTINE utlCompareHLRegWithIXReg

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : utlCompareHLRegWithIYReg
;
; Description : Compares two structures as pointed by HLReg and IYReg.
;
; Assumptions : None
;
; Input(s):     HLReg and IYReg should point to the higher significant byte
;                 where comparison should start.   
;               BReg should contain how many byte fields are to be compared.
;
; Output(s)   : HLReg = IYReg ; bZeroFlag = TRUE  , bCarryFlag = FALSE
;               HLReg < IYReg ; bZeroFlag = FALSE , bCarryFlag = TRUE
;               HLReg > IYReg ; bZeroFlag = FALSE , bCarryFlag = FALSE
;
; Destroyed:    HLReg, IYReg, BReg 
;
; Module Change History:
;
;               12/07/2001  - Created utlCompareHLRegWithIYReg for M851
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        GLOBAL  utlCompareHLRegWithIYReg
        
        IF  @DEF('SUBROUTINE')
            UNDEF   SUBROUTINE
        ENDIF
        DEFINE      SUBROUTINE      "'utlCompareHLRegWithIYReg'"            

utlCompareHLRegWithIYReg:                           ; **SUBROUTINE utlCompareHLRegWithIYReg

 

                ;--------------------------------------------------------------
                ; Compare the HLReg with IYReg.
                ; If the structures are not equal then go out from the compare
                ;   loop.
                ;--------------------------------------------------------------
                cp      [HL], [IY]
                jr      NZ, utlCompareHLWithIYExit
                
                ;--------------------------------------------------------------
                ; Let the pointers point to next field to compare. The next 
                ;   field to compare is the lower significant field.
                ;--------------------------------------------------------------
                dec     HL
                dec     IY        
        
                ;--------------------------------------------------------------
                ; Decrement the counter. When BReg is Zero, bZeroFlag is Set 
                ;   or TRUE.
                ;--------------------------------------------------------------
        
                djr     NZ, utlCompareHLRegWithIYReg
                      

utlCompareHLWithIYExit:

                ret

                                                    ; **END SUBROUTINE utlCompareHLRegWithIYReg





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : utlCopyIYRegToIXReg
;
; Description : Copies the data pointed by IYReg to the location pointed by
;                 IXReg.
;
; Assumptions : None
;
; Input(s)    : BReg  - number of bytes to copy.
;               IYReg - lowest address of the source structure
;               IXReg - lowest address of the destination structure
;
; Output(s)   : IYReg - points to the next byte after the source structure
;               IXReg - points to the next byte after the destinations structure
;               BReg  - '0'
;
;               ( Destroyed: None )
;
; Module Change History:
;
;               03/04/2002  - Created utlCopyIYRegToIXReg for M851
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        GLOBAL  utlCopyIYRegToIXReg
        
        IF  @DEF('SUBROUTINE')
            UNDEF   SUBROUTINE
        ENDIF
        DEFINE      SUBROUTINE      "'utlCopyIYRegToIXReg'"            


utlCopyIYRegToIXReg:                                ; **SUBROUTINE utlCopyIYRegToIXReg

                ld      [IX], [IY]
                               
                inc     IY
                inc     IX
                               
                djr     NZ, utlCopyIYRegToIXReg

                ret


                                                    ; **END SUBROUTINE utlCopyIYRegToIXReg



; **END FILE utlMisc.asm


        END