;**FILE utlSetB.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2001-2002 TIMEX Corporation.
; 
; This document is the property of, and contains information proprietary to
; TIMEX Corporation.  It is to be returned upon request.  Reproduction by any
; means, disclosure, or use of its contents for other than the expressed
; purpose for which it is loaned, without the written consent of TIMEX
; Corporation, is strictly prohibited.
;
; File Name:    utlSetB.asm
;
; Module Names: utlSetBannerStateManager -
;               Set Banner State manager event handler for all set states.
;    
;
; File Change History:
;
;  M851 - Development:
;               03/11/2001  - Created for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
$CASE ON
$LIST OFF

        INCLUDE '..\..\CORE\H\MODEL.H'
        INCLUDE '..\..\HW\H\HW88349.H'
        INCLUDE '..\..\HW\H\HWI2CEE.H'
        INCLUDE '..\..\HW\H\HWBOND.H'
        INCLUDE '..\..\HW\H\HWKBD.H'
        INCLUDE '..\..\HW\H\HWMACRO.H'
        INCLUDE '..\..\HW\H\HWMAIN.H'
        INCLUDE '..\..\HW\H\HWRESET.H'
        INCLUDE '..\..\HW\H\HWVARS.H'
        INCLUDE '..\..\HW\H\HWSERMAC.H'
        INCLUDE '..\..\CORE\H\CORE.H'
        INCLUDE '..\..\CORE\H\COREMAC.H'
        INCLUDE '..\..\CORE\H\CORESVMC.H'
        INCLUDE '..\..\COMM\H\COMM.H'
        INCLUDE '..\..\LCD\H\LCD.H'
        INCLUDE '..\..\LCD\H\LCDMACRO.H'
        INCLUDE '..\..\LCD\H\LCDRAM.H'
        INCLUDE '..\..\DATABASE\H\DB.H'
        INCLUDE '..\..\DATABASE\H\DBMACRO.H'
        INCLUDE '..\..\AUDIO\H\AUD.H'
        INCLUDE '..\..\AUDIO\H\AUDMAC.H'
        INCLUDE '..\..\UTIL\H\UTLMATH.H'
        INCLUDE '..\..\UTIL\H\UTLMTHMC.H'
        INCLUDE '..\..\UTIL\H\UTLEDT.H'
        INCLUDE '..\..\UTIL\H\UTLEDTMC.H'
        INCLUDE '..\..\UTIL\H\UTLDSP.H'
        INCLUDE '..\..\UTIL\H\UTLDSPMC.H'
        INCLUDE '..\..\UTIL\H\UTLACCMC.H'
        INCLUDE '..\..\UTIL\H\UTLMSC.H'
        INCLUDE '..\..\UTIL\H\UTLMSCMC.H'
        INCLUDE '..\..\UTIL\H\UTLVARS.H'
        INCLUDE '..\..\KRES\H\KRES.H'
        INCLUDE '..\..\KRES\H\KRESMAC.H'
        INCLUDE '..\..\KRES\H\KSTP.H'
        INCLUDE '..\..\KRES\H\KSTPMAC.H'
        INCLUDE '..\..\KRES\H\KSYN.H'
        INCLUDE '..\..\KRES\H\KSYNMAC.H'
        INCLUDE '..\..\KRES\H\KTMR.H'
        INCLUDE '..\..\KRES\H\KTMRMAC.H'
        INCLUDE '..\..\KRES\H\KTOD.H'
        INCLUDE '..\..\KRES\H\KTODMAC.H'
        INCLUDE '..\..\KRES\H\KTZC.H'
        INCLUDE '..\..\KRES\H\KTZCMAC.H'
        INCLUDE '..\..\KRES\H\KBCK.H'
        INCLUDE '..\..\KRES\H\KBCKMAC.H'
        INCLUDE '..\..\USAGE\H\USG.H'
        INCLUDE '..\..\USAGE\H\USGVARS.H'
        INCLUDE '..\..\USAGE\H\USGMAC.H'
        INCLUDE '..\..\CORE\H\SECTION.H'

        INCLUDE '..\..\TOD\H\TOD.H'
        INCLUDE '..\..\TOD\H\TODVARS.H'

$LIST ON

        SECT    ".text_commonappt"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name:  utlSetBannerStateManager
;
; Description:  Application Set banner state handler
;
; Assumptions:  1) Popups are suspended by the core.
;               2) LCD is Clear on entry.
;               3) Application displays its set banner prior 
;                  to calling this generic state handler.
;
; Input(s):     CORECurrentEvent    - system event to be processed
;
; Output(s):    NONE
;
; Module Change History:
;
;               03/11/2001  - Created for M851.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT `

utlSetBannerStateManager()         /* Set Banner event handler */
{

    switch(CORECurrentEvent) 
        {

        case COREEVENT_STATEENTRY:

            // ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            // ;
            // ; SET BANNER ENTRY
            // ;
            // ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

            // set popdown state should a popup occur during set banner timeout
            CORE_SET_POPDOWN_STATE  CORESETSTATE;

            // Request a 1.5 Second timer from the CORE
            CORE_REQ_TIMEOUT_HIRES TIMEOUTHIRES_1P5SEC;

            // Enable ANYSWITCHDEP
            CORE_ENABLE_GENERIC_SWITCH_EVENT;

            break;

        case COREEVENT_ANYSWITCHDEPRESS:
        case COREEVENT_CW_EDGE_TRAILING:
        case COREEVENT_CCW_EDGE_TRAILING:
        case COREEVENT_TIMEOUTDONE_HIGHRES:

            // ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            // ;
            // ; CANCEL SET BANNER VIA SWITCH, CROWN ROTATION or
            // ; TIMEOUT EVENTS
            // ;
            // ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            // Cancel 1.5 Second Timer
            // (Automatically happens due to event processing).

            // Request State change to APPT Set State
            BReg = CORESETSTATE;
            CORE_REQ_STATE_CHANGE;

            break;

        case COREEVENT_CROWN_HOME:

            // ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            // ;
            // ; CROWN HOME EVENT PROCESSING
            // ;
            // ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

            // Cancel 1.5 Second Timer via state change
            // Request State change to APPT Default State
            BReg = COREDEFAULTSTATE;
            CORE_REQ_STATE_CHANGE;
            break;

        default:
         
            // ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
            // ;     All cases handled.
            // ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        }
return;
}

ENDCOMMENT `

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; ACTUAL CODE STARTS HERE!!!
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


        IF @DEF('SUBROUTINE')
            UNDEF SUBROUTINE
        ENDIF
        DEFINE SUBROUTINE "'utlSetBannerStateManager'"

        GLOBAL  utlSetBannerStateManager

utlSetBannerStateManager:             ; **SUBROUTINE utlSetBannerStateManager

              ;---------------------------------------------------------------
              ;  Check what event was passed to the Set Banner State.
              ;  Check for State Entry first.
              ;---------------------------------------------------------------
              
              ld        A,      [CORECurrentEvent]
              cp        A,      #COREEVENT_STATEENTRY
              jr        NZ,     utlSetBChkNextEvent2

              ;***************************************************************
              ;                STATE ENTRY EVENT PROCESSING.
              ;***************************************************************

              ;---------------------------------------------------------------
              ; set popdown state should a popup occur during set banner timeout
              ;---------------------------------------------------------------
              CORE_SET_POPDOWN_STATE    CORESETSTATE

              ;---------------------------------------------------------------
              ; Request a 1.5 Second timer from the CORE
              ;---------------------------------------------------------------
              CORE_REQ_TIMEOUT_HIRES TIMEOUTHIRES_1P5SEC

              ;---------------------------------------------------------------
              ; Enable ANYSWITCHDEP
              ;---------------------------------------------------------------
              CORE_ENABLE_GENERIC_SWITCH_EVENT

              jr        utlSetBStateExit

utlSetBChkNextEvent2:

              cp        A,      #COREEVENT_CROWN_HOME
              jr        NZ,     utlSetBChkNextEvent3

              ;***************************************************************
              ;      CROWN HOME EVENT PROCESSING
              ;***************************************************************
                  
              ;---------------------------------------------------------------
              ; Cancel 1.5 Second Timer via state change
              ; Request State change to Default State
              ;---------------------------------------------------------------
              ld        B,      #COREDEFAULTSTATE;
              CORE_REQ_STATE_CHANGE

              jr        utlSetBStateExit

utlSetBChkNextEvent3:

              ;***************************************************************
              ; default: Handle all the following
              ; 
              ; case COREEVENT_ANYSWITCHDEPRESS:
              ; case COREEVENT_CW_EDGE_TRAILING:
              ; case COREEVENT_CCW_EDGE_TRAILING:
              ; case COREEVENT_TIMEOUTDONE_HIGHRES:
              ; 
              ; CANCEL SET BANNER VIA SWITCH, CROWN ROTATION or
              ; TIMEOUT EVENTS
              ;
              ;***************************************************************
              ;---------------------------------------------------------------
              ; Cancel 1.5 Second Timer
              ; (Automatically happens due to event processing).
              ;---------------------------------------------------------------

              ;---------------------------------------------------------------
              ; Request State change to Set State
              ;---------------------------------------------------------------
              ld        B,      #CORESETSTATE;
              CORE_REQ_STATE_CHANGE

utlSetBStateExit:

              ret

                                   ; **END SUBROUTINE utlSetBannerStateManager





;**END FILE utlSetB.asm

        END