;**FILE utlurock.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2001-2002 TIMEX Corporation.
; 
; This document is the property of, and contains information proprietary to
; TIMEX Corporation.  It is to be returned upon request.  Reproduction by any
; means, disclosure, or use of its contents for other than the expressed
; purpose for which it is loaned, without the written consent of TIMEX
; Corporation, is strictly prohibited.
;
; File Name    : utlurock.asm
;
; Module Names : utlYouRockStateManager - State manager for the generic "You Rock" state. 
;                                           Ignores all input except the "Crown Home" event, 
;                                           at which point we return to the default state or 
;                                           popdown if we are in a popup.
;
; Purpose      : Subroutines for controlling the generic "You Rock" state.
;
; File Change History:
;
;                01/10/2001  - Created utlurock.asm for M851
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


$CASE ON
$LIST OFF

        INCLUDE '..\..\CORE\H\MODEL.H'
        INCLUDE '..\..\HW\H\HW88349.H'
        INCLUDE '..\..\HW\H\HWI2CEE.H'
        INCLUDE '..\..\HW\H\HWBOND.H'
        INCLUDE '..\..\HW\H\HWKBD.H'
        INCLUDE '..\..\HW\H\HWMACRO.H'
        INCLUDE '..\..\HW\H\HWMAIN.H'
        INCLUDE '..\..\HW\H\HWRESET.H'
        INCLUDE '..\..\HW\H\HWVARS.H'
        INCLUDE '..\..\HW\H\HWSERMAC.H'
        INCLUDE '..\..\CORE\H\CORE.H'
        INCLUDE '..\..\CORE\H\COREMAC.H'
        INCLUDE '..\..\CORE\H\CORESVMC.H'
        INCLUDE '..\..\COMM\H\COMM.H'
        INCLUDE '..\..\LCD\H\LCD.H'
        INCLUDE '..\..\LCD\H\LCDMACRO.H'
        INCLUDE '..\..\LCD\H\LCDRAM.H'
        INCLUDE '..\..\DATABASE\H\DB.H'
        INCLUDE '..\..\DATABASE\H\DBMACRO.H'
        INCLUDE '..\..\AUDIO\H\AUD.H'
        INCLUDE '..\..\AUDIO\H\AUDMAC.H'
        INCLUDE '..\..\UTIL\H\UTLMATH.H'
        INCLUDE '..\..\UTIL\H\UTLMTHMC.H'
        INCLUDE '..\..\UTIL\H\UTLEDT.H'
        INCLUDE '..\..\UTIL\H\UTLEDTMC.H'
        INCLUDE '..\..\UTIL\H\UTLDSP.H'
        INCLUDE '..\..\UTIL\H\UTLDSPMC.H'
        INCLUDE '..\..\UTIL\H\UTLACCMC.H'
        INCLUDE '..\..\UTIL\H\UTLMSC.H'
        INCLUDE '..\..\UTIL\H\UTLMSCMC.H'
        INCLUDE '..\..\UTIL\H\UTLVARS.H'
        INCLUDE '..\..\KRES\H\KRES.H'
        INCLUDE '..\..\KRES\H\KRESMAC.H'
        INCLUDE '..\..\KRES\H\KSTP.H'
        INCLUDE '..\..\KRES\H\KSTPMAC.H'
        INCLUDE '..\..\KRES\H\KSYN.H'
        INCLUDE '..\..\KRES\H\KSYNMAC.H'
        INCLUDE '..\..\KRES\H\KTMR.H'
        INCLUDE '..\..\KRES\H\KTMRMAC.H'
        INCLUDE '..\..\KRES\H\KTOD.H'
        INCLUDE '..\..\KRES\H\KTODMAC.H'
        INCLUDE '..\..\KRES\H\KTZC.H'
        INCLUDE '..\..\KRES\H\KTZCMAC.H'
        INCLUDE '..\..\KRES\H\KBCK.H'
        INCLUDE '..\..\KRES\H\KBCKMAC.H'
        INCLUDE '..\..\USAGE\H\USG.H'
        INCLUDE '..\..\USAGE\H\USGVARS.H'
        INCLUDE '..\..\USAGE\H\USGMAC.H'
        INCLUDE '..\..\CORE\H\SECTION.H'

        INCLUDE '..\..\TOD\H\TOD.H'
        INCLUDE '..\..\TOD\H\TODVARS.H'

$LIST ON

        SECT    ".text_utl"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : utlYouRockStateManager
;
; Description : State manager for the generic "You Rock" state. Ignores all input
;                 except the "Crown Home" event, at which point we return to the 
;                 default state or popdown if we are in a popup.
;
; Assumptions : None
;
; Input(s)    : CORECurrentEvent
;
; Output(s)   : None
;
;               ( Destroyed: ALL Registers )
;
; Module Change History:
;
;               01/10/2002  - Created module for M851
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

COMMENT     `

void utlYouRockStateManager()
{

    //*************************************************************************
    // The "You Rock" state is a generic display, designed to inform the user 
    //   that the crown was pulled to a "Set" position when the "Set" position 
    //   is invalid (for example, pulling the crown out in a mode lacking a 
    //   setting state). The state is generic because it operates in the same 
    //   fashion for all modes, hence the need for only one state. The only 
    //   input that can get the user out of this state is to push the crown to 
    //   the "Home" position. This returns the user to the default state.
    //
    // If the crown is pulled out during a popup, the same condition occurs, 
    //   i.e., the "Set" position is invalid, so the "You Rock" message is 
    //   displayed. In this case, when the user pushes the crown to the "Home" 
    //   position, we do not return to the default state of the current mode, 
    //   but to the mode and state which was "popped over".
    //*************************************************************************

    //-------------------------------------------------------------------------
    // Check what event was passed to the "You Rock" manager.
    //-------------------------------------------------------------------------    
    switch( CORECurrentEvent )                                                     
    {                                                                              
                                                                                   
        case COREEVENT_STATEENTRY:                                                 
            //*****************************************************************    
            //
            //                         STATE ENTRY
            //
            //*****************************************************************

            // Suspend popups, since the crown is out and we do NOT want anything 
            //   "popping over" this warning message.
            CORE_SUSPEND_POPUPS;

            // Prohibit the switch and ring events to be passed to this state.
            CORE_MASK_KEYS (bCORESwitch1 | bCORESwitch2 | bCORESwitch3 | bCORECWSwitch | bCORECCWSwitch | bCOREELSwitch);  

            // Display "YOU ROCK" warning message, and exit.                
            LCD_DISP_SMALL_DM_MSG_YOU_ROCK;                                 

            break;


        case COREEVENT_CROWN_HOME:
            //*****************************************************************
            //
            //                         CROWN HOME
            //
            //*****************************************************************

            //=================================================================
            // The user has acknowledged the warning and we should now return 
            //   control back to the default state if we are not in a popup, or 
            //   to the "popped over" mode and state if we are.
            //=================================================================
            
            // Check if we are in a popup.
            if( CORESystemFlag4.bCOREPopupMode == TRUE )
            {       
                // We ARE in a popup.
                // Ask the core to return to the "popped over" state.
                CORE_REQ_POPDOWN;
            }
            else
            {            
                // We are NOT in a popup.
                // Ask core to return to default state.
                BReg = COREDEFAULTSTATE;
                CORE_REQ_STATE_CHANGE;
            }

            break;
    }
}

ENDCOMMENT  `

;------------------------------------------------------------------------------
;
; ASSEMBLY CODE
;
;------------------------------------------------------------------------------

        GLOBAL  utlYouRockStateManager
        
        IF  @DEF('SUBROUTINE')
            UNDEF   SUBROUTINE
        ENDIF
        DEFINE      SUBROUTINE      "'utlYouRockStateManager'"            



utlYouRockStateManager:                             ; **SUBROUTINE utlYouRockStateManager

                ;**************************************************************
                ; The "You Rock" state is a generic display, designed to inform 
                ;   the user that the crown was pulled to a "Set" position when 
                ;   the "Set" position is invalid (for example, pulling the 
                ;   crown out in a mode lacking a setting state). The state is 
                ;   generic because it operates in the same fashion for all modes, 
                ;   hence the need for only one state. The only input that can get 
                ;   the user out of this state is to push the crown to the "Home" 
                ;   position. This returns the user to the default state.
                ; 
                ; If the crown is pulled out during a popup, the same condition 
                ;   occurs, i.e., the "Set" position is invalid, so the "You Rock" 
                ;   message is displayed. In this case, when the user pushes the 
                ;   crown to the "Home" position, we do not return to the default 
                ;   state of the current mode, but to the mode and state which 
                ;   was "popped over".
                ;**************************************************************

                ;--------------------------------------------------------------
                ; Check what event was passed to the "You Rock" manager.
                ;--------------------------------------------------------------
                ld      A, [CORECurrentEvent]

                ; Check if state entry event.
                cp      A, #COREEVENT_STATEENTRY
                jr      NZ, utlYouRockStMgrCheckEvent2

                ;**************************************************************
                ;
                ;                         STATE ENTRY
                ;
                ;**************************************************************

                ; Suspend popups, since the crown is out and we do NOT want anything 
                ;   "popping over" this warning message.
                CORE_SUSPEND_POPUPS

                ; Prohibit the switch and ring events to be passed to this state.
                CORE_MASK_KEYS (bCORESwitch1 | bCORESwitch2 | bCORESwitch3 | bCORECWSwitch | bCORECCWSwitch | bCOREELSwitch)

                ; Display "YOU ROCK" warning message, and exit.
                LCD_DISP_SMALL_DM_MSG_YOU_ROCK

                ret


utlYouRockStMgrCheckEvent2:

                ; Check if crown home event.
                cp      A, #COREEVENT_CROWN_HOME
                jr      NZ, utlYouRockStMgrExit

                ;**************************************************************
                ;
                ;                         CROWN HOME
                ;
                ;**************************************************************

                ;==============================================================
                ; The user has acknowledged the warning and we should now 
                ;   return control back to the default state if we are not 
                ;   in a popup, or to the "popped over" mode and state if 
                ;   we are.
                ;==============================================================
            
                ; Check if we are in a popup.
                ld      A, [CORESystemFlag4]
                bit     A, #bCOREPopupMode
                jr      Z, utlYouRockGoToDefaultState

                ; We ARE in a popup.
                ; Ask the core to return to the "popped over" state.
                CORE_REQ_POPDOWN

                ret

utlYouRockGoToDefaultState:
                
                ; We are NOT in a popup.
                ; Ask core to return to default state.
                ld      B, #COREDEFAULTSTATE
                CORE_REQ_STATE_CHANGE

utlYouRockStMgrExit:

                ret


                                                    ; **END SUBROUTINE utlYouRockStateManager


;**END FILE utlurock.asm


        END